<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Comment List Controller
 * @license GoLevel LLC
 */

Class SAA_Controller_Comment_List extends SAA_Controller
{
    private $_bSimple = true;
    
    /*
     * Controller preloader
     */
    public function preload()
    {
        SAA_Sys::getModel('public')->loadAjax();
    }

    /*
     * Controller loader
     */
    public function load()
    {
        $oComment = SAA_Sys::getModel('comment');

        if( !SAA_Sys::getSett('comment.is_enabled') )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('comment.the_comments_have_been_disabled'));
        }
        elseif( !SAA_Sys::isUser() && !SAA_Sys::getSett('comment.guest_can_view') )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('comment.you_must_be_logged_in_to_view_this_page'));
        }
        elseif( !($sType = $this->request()->get('type')) )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('comment.the_page_you_are_looking_for_cannot_be_found'));
        }
        elseif( !($iItem = $this->request()->get('item_id')) )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('comment.the_page_you_are_looking_for_cannot_be_found'));
        }
        elseif( !($aItem = $oComment->getItem($sType, $iItem)) )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('comment.the_page_you_are_looking_for_cannot_be_found'));
        }

        $iPg = $this->request()->get('pg', 1);
        $bIsController = $this->request()->getInt('is_controller');
        $iLimit = ($bIsController ? SAA_Sys::getSett('comment.per_page_controller') : SAA_Sys::getSett('comment.per_page_block'));

        if( ($iParent = $this->request()->get('parent_id')) && $iParent > 0 )
        {
            if( !SAA_Sys::getSett('comment.enable_child') )
            {
                return SAA_Error::add(SAA_Sys::getPhrase('comment.the_feature_has_been_disabled'));
            }

            $aParent = $oComment->get($iParent);

            if( !$aParent || $aParent['item_id'] != $iItem )
            {
                return SAA_Error::add(SAA_Sys::getPhrase('comment.the_page_you_are_looking_for_cannot_be_found'));
            }

            $this->template()
                ->assign(array(
                    'aParentComment' => $aParent
            ));

            list($iTotal, $aList) = $oComment->getAll($iItem, $iParent, $sType, $iPg, $iLimit);

            $this->_bSimple = false;
        }
        else
        {
            list($iTotal, $aList) = $oComment->getAll($iItem, null, $sType, $iPg, $iLimit);
        }

        $this->template()
            ->assign(array(
                'iTotal' => $iTotal,
                'iPg' => $iPg,
                'iPgSize' => $iLimit,
                'sType' => $sType,
                'iItem' => $iItem,
                'aItem' => $aItem,
                'aList' => $aList,
                'iLeapTo' => $this->request()->getInt('leap_to'),
                'bController' => $bIsController
        ));
    }

    public function view()
    {
        if( $this->_bSimple )
        {
            $sTpl = 'comment/list';
        }
        else
        {
            $sTpl = 'comment/list-parent';
        }

        return $sTpl;
    }
}
?>
