<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Process Achievement Class
 * @license GoLevel LLC
 */

Class SAA_Ajax_Achievement extends SAA_Ajax
{
    /**
     * Updates text for a given achievement
     */
    public function edit()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iAward = $this->_request()->get('id');
        $sTitle = $this->_request()->get('title');
        $sDesc = $this->_request()->get('description');

        if( !SAA_Sys::getModel('game.achievement')->getById($iAward) )
        {
            return $this->error('The achievement cannot be found.');
        }

        if( SAA_Sys::getModel('game.achievement.process')->edit($iAward, $sTitle, $sDesc) )
        {
            $this->_call('$(\'#edit-achievement\').dialog(\'close\');')
                ->message('Achievement has been saved.');
        }
    }

    /**
     * Gets the achievements for admin edit user
     */
    public function forUserAdmin()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#js_achievements', $this->_loadController('admin.user.achievement'));
    }

    /**
     * Gets the achievment block for the game page
     */
    public function forGame()
    {
        if( SAA_Sys::getSett('achievement.is_on') )
        {
            $this->html('#achievements', $this->_loadController('game.achievement.block', array('bHeader' => true)));
        }
    }

    /**
     * Gets the achievements for admin edit game
     */
    public function forGameAdmin()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#js_achievements', $this->_loadController('admin.game.achievement'));
    }

    /**
     * Gets the achievements for user's profile
     */
    public function forProfile()
    {
        if( !SAA_Sys::getSett('achievement.is_on') || !SAA_Sys::getSett('achievement.on_profile_pg') )
        {
            return $this->error(SAA_Sys::getPhrase('profile.the_achievements_have_been_disabled'));
        }

        $this->html('#achievements', $this->_loadController('profile.achievement'));
    }

    /**
     * Turn on/off achievements
     */
    public function toggle()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iAward = $this->_request()->get('award_id');
        $iToggle = $this->_request()->get('status');

        if( !SAA_Sys::getModel('game.achievement')->getById($iAward) )
        {
            return $this->error('The achievement cannot be turned '.($iToggle ? 'on' : 'off').'.');
        }

        if( SAA_Sys::getModel('game.achievement.process')->toggle($iAward, $iToggle) )
        {
            $this->_call('$(\'#js_award_'.$iAward.'\').find(\'.js_award_toggle\').html(\''.($iToggle ? 'Disable' : 'Activate').'\');')
                ->message('Achievement has been turned '.($iToggle ? 'on' : 'off').'.');
        }
        else
        {
            $this->error('There was an error in processing your request.');
        }
    }

    /**
     * Turn on/off achievement for a given user
     */
    public function toggleForUser()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iUser = $this->_request()->get('user_id');
        $iAward = $this->_request()->get('award_id');
        $iToggle = $this->_request()->get('status');

        if( !SAA_Sys::getModel('game.achievement')->getById($iAward) )
        {
            return $this->error('The achievement cannot be turned '.($iToggle ? 'on' : 'off').' for the given user.');
        }

        if( SAA_Sys::getModel('game.achievement.process')->toggleForUser($iAward, $iUser, $iToggle) )
        {
            $this->_call('$(\'#js_award_'.$iAward.'\').find(\'.js_award_toggle\').html(\''.($iToggle ? 'Disable' : 'Activate').'\');')
                ->message('Achievement has been turned '.($iToggle ? 'on' : 'off').'.');
        }
        else
        {
            $this->error('There was an error in processing your request.');
        }
    }
}
?>
