<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Process Category Class
 * @license GoLevel LLC
 */

Class SAA_Ajax_Category extends SAA_Ajax
{
    /**
     * Deletes a given category
     */
    public function delete()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iCategory = $this->_request()->get('id');

        if( SAA_Sys::getModel('category.process')->delete($iCategory) )
        {
            $this->message('Category has been deleted.');
        }
        else
        {
            $this->alert('Opps! There was an error in deleting the category.');
        }
    }

    /**
     * Saves the order
     */
    public function editOrder()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aOrder = $this->_request()->getArray('aOrder');

        if( SAA_Sys::getModel('category.process')->editOrder($aOrder) )
        {
            $this->message('New order has been saved.');
        }
        else
        {
            $this->alert('Opps! There was an error in saving the order.');
        }
    }

    /**
     * Rebuilds the category cache table
     */
    public function rebuildCache()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        $iLimit = 50;

        if( $aForm['total'] == 0 )
        {
            SAA_Sys::getModel('game.category.process')->clearTable();
        }

        list($iCnt, $iAdded) = SAA_Sys::getModel('game.category.process')->rebuild($aForm['page'], $iLimit);

        $iTotal = $iAdded + $aForm['total'];
        $iPosition = $aForm['page']*$iLimit;

        if( $iCnt > $iPosition )
        {
            $sSrc = SAA_Sys::getLib('url')->makeUrl(array(
                'folder' => SAA_Sys::getLib('theme')->getFile('design.theme', 'images.admin', 'loader-circle-small.gif'),
                'file' => 'loader-circle-small.gif'
            ));

            $sMsg = '<div class="goodBox"><img src="'.$sSrc.'"> Cached <strong>'.$iTotal.'</strong> out of <strong>'.$iCnt.'</strong></div>';

            $this->val('#js_page', ($aForm['page']+1))
                ->val('#js_total', $iTotal)
                ->html('#js_content', $sMsg)
                ->trigger('#js_form', 'submit');
        }
        else
        {
            $this->hide('#js_content')
                ->val('#js_page', 1)
                ->val('#js_total', 0)
                ->message('All cache has been rebuilt.', array('auto_close' => false))
                ->show('#js_form');
        }
    }

    /**
     * Checks if the url slug is unique
     */
    public function uniqueUrl()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        if( isset($aForm['slug'], $aForm['category_id']) && strlen($aForm['slug']) )
        {
            $sUrl = SAA_Sys::getLib('parse.input')->toAlphaNumeric($aForm['slug'], 100, '-', '-');

            $aCategory = SAA_Sys::getModel('category')->get($sUrl);

            if( $aForm['slug'] != $sUrl )
            {
                $this->val('#js_category_slug', $sUrl);

                $this->html('#js_category_slug_msg', '<br/><i>Url has been sanitized. '.($aCategory && $aCategory['category_id'] != $aForm['category_id'] ? 'The url is already in use by another category.' : '').'</i>');
                $this->show('#js_category_slug_msg');
            }
            elseif( $aCategory && $aCategory['category_id'] != $aForm['category_id'] )
            {
                $this->html('#js_category_slug_msg', '<br/><i>The url is already in use by another category.</i>');
                $this->show('#js_category_slug_msg');
            }
            else
            {
                $this->hide('#js_category_slug_msg');
            }
        }
    }
}
?>
