<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Process Comment Class
 * @license GoLevel LLC
 */

Class SAA_Ajax_Comment extends SAA_Ajax
{
    /**
     * Adds a given comment
     */
    public function add()
    {
        if( !SAA_Sys::isUser() )
        {
            return $this->error(SAA_Sys::getPhrase('comment.you_must_be_logged_in_to_add_a_comment'));
        }

        $iItem = $this->_request()->get('item_id');
        $iParent = $this->_request()->get('parent_id');
        $sType = $this->_request()->get('type');
        $sText = $this->_request()->get('text');
        $bIsController = $this->_request()->getInt('is_controller');

        $iPageSize = ($bIsController ? SAA_Sys::getSett('comment.per_page_controller') : SAA_Sys::getSett('comment.per_page_block'));

        if( ($aComment = SAA_Sys::getModel('comment.process')->add($sType, $iItem, $iParent, $sText)) )
        {
            $sCondition = 'item_id = '. (int) $aComment['item_id'] .' AND model_id = "'. $aComment['model_id'] .'" AND parent_id = '. (int) $iParent .' AND comment_id <= '. (int) $aComment['comment_id'];

            $iCurrentPg = SAA_Sys::getLib('paginator')->getPageNum('comment', $sCondition, $iPageSize, array('time_stamp', 'ASC'));

            if( $iParent )
            {
                $sStr = '{pg:'.$iCurrentPg.', leap_to:'.$aComment['comment_id'].', parent_id:'.(int) $iParent.'}';
            }
            else
            {
                $sStr = '{pg:'.$iCurrentPg.', leap_to:'.$aComment['comment_id'].'}';
            }

            $this->_call('loadComments('.$sStr.');');
        }
        else
        {
            $this->error(SAA_Sys::getPhrase('comment.there_is_an_error_in_processing_you_request'));
        }

    }

    /**
     * Deletes a given comment
     */
    public function delete()
    {
        if( !SAA_Sys::isUser() )
        {
            return $this->error(SAA_Sys::getPhrase('comment.you_must_be_logged_in_to_delete_a_comment'));
        }

        $iId = $this->_request()->get('comment_id');
        $bIsController = $this->_request()->getInt('is_controller');

        $iPageSize = ($bIsController ? SAA_Sys::getSett('comment.per_page_controller') : SAA_Sys::getSett('comment.per_page_block'));

        $aComment = SAA_Sys::getModel('comment')->isUsers($iId);

        if( !$aComment )
        {
            return $this->error(SAA_Sys::getPhrase('comment.there_is_an_error_in_processing_you_request'));
        }

        $sCondition = 'item_id = '. (int) $aComment['item_id'] .' AND model_id = "'. $aComment['model_id'] .'" AND parent_id = '. (int) $aComment['parent_id'] .' AND comment_id < '. (int) $iId;
        $iCurrentPg = SAA_Sys::getLib('paginator')->getPageNum('comment', $sCondition, $iPageSize, array('time_stamp', 'ASC'));

        if( SAA_Sys::getModel('comment.process')->delete($iId, $aComment) )
        {
            $sStr = '{pg:'. $iCurrentPg . ($aComment['parent_id'] ? ', parent_id:'.$aComment['parent_id'] : '') .'}';
            $this->_call('loadComments('.$sStr.');')
                ->message(SAA_Sys::getPhrase('comment.comment_has_been_removed'));
        }
        else
        {
            $this->error(SAA_Sys::getPhrase('comment.there_is_an_error_in_processing_you_request'));
        }
    }

    /**
     * Displays comment list
     */
    public function load()
    {
        $this->html('#js_comment_div', $this->_loadController('comment.list'));
    }

    /**
     * Rebuilds the comment tables
     */
    public function rebuild()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aVal = $this->_request()->getArray('aForm');

        if( $aVal['total'] == 0 )
        {
            $aVal['total'] = SAA_Sys::getModel('comment')->getTotal();

            $this->val('#js_total', $aVal['total']);
        }

        if( $aVal['stage'] == 'remove-missing' )
        {
            $iLeft = SAA_Sys::getModel('comment.process')->removeMissing();

            if( $iLeft > 0 )
            {
                $this->show('#js_content')
                    ->val('#js_total', $iLeft)
                    ->trigger('#js_form', 'submit');
            }
            else
            {
                $this->val('#js_stage', 'reparsing')
                    ->show('#js_content')
                    ->trigger('#js_form', 'submit');
            }
        }
        elseif( $aVal['stage'] == 'reparsing' )
        {
            $iIncre = 50;

            SAA_Sys::getModel('comment.process')->reparseText($aVal['page'], $iIncre);

            $iCompleted = $aVal['completed'] + $iIncre;

            if( $aVal['total'] - $iCompleted > 0 )
            {
                $this->show('#js_content')
                    ->val('#js_completed', $iCompleted)
                    ->val('#js_page', ($aVal['page'] + 1))
                    ->trigger('#js_form', 'submit');
            }
            else
            {
                $this->val('#js_stage', 'remove-missing')
                    ->val('#js_total', 0)
                    ->val('#js_completed', 0)
                    ->val('#js_page', 0)
                    ->show('#js_form')
                    ->message('Comments have been rebuilt.');
            }
        }
    }
}
?>
