<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Process Game Class
 * @license GoLevel LLC
 */

Class SAA_Ajax_Game extends SAA_Ajax
{
    /**
     * Deletes a given game(s)
     */
    public function delete()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#ajax-dialog-box', $this->_loadController('game.delete'));
    }

    /**
     * Remove game as featured
     */
    public function deleteFeature()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iGame = $this->_request()->get('id');

        $this->_call('$(\'#dialog-message\').dialog(\'close\');');

        if( !$iGame )
        {
            return $this->error('There was an error in trying to process your request.');
        }

        if( SAA_Sys::getModel('game.process')->toggleFeatured(array($iGame), 'remove') )
        {
            $this->remove('#js_game_'.$iGame)
                ->message('Game is no longer featured.', array('display' => 'div', 'location' => '#js_order_form_msg'));
        }
        else
        {
            $this->error('There was an error in trying to process your request.');
        }
    }

    /**
     * Remove games
     */
    public function deleteMass()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        if( !$aForm )
        {
            return $this->error('Opps! There was an error in processing your request.')->hide('#js_content')->show('#js_form');
        }

        $mReturn = SAA_Sys::getModel('game.process')->deleteMass($aForm);

        if( is_array($mReturn) )
        {
            list($iLeft, $iRemoved) = $mReturn;

            $sMessage = ($iRemoved > 1 ? $iRemoved . ' games have' : ($iRemoved == 1 ? 'One game has' : 'No games have')).' been removed.';

            if( $iLeft == 0 )
            {
                $this->message('Removal of selected games have been completed. ' . $sMessage, array('display' => 'div', 'location' => '#js_content'))
                    ->show('#js_form');
            }
            else
            {
                $this->error($sMessage . ' There '.($iLeft > 1 ? 'are '.$iLeft.' games' : 'is one game').' left to remove.', array('display' => 'div', 'location' => '#js_content'))
                    ->val('#js_total', $iRemoved)
                    ->_call('$(\'#js_form\').trigger(\'submit\');');
            }
        }
        else
        {
            $this->error('There was an error in trying to process your request.');
        }
    }

    /**
     * Update game info
     */
    public function edit()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        if( !($aGame = SAA_Sys::getModel('game')->getForAdmin($aForm['game_id'], false)) )
        {
            return $this->error('Error in processing your request. Please refesh the page and try again.');
        }

        if( SAA_Sys::getModel('game.process')->edit($aForm, $aGame) )
        {
            $this->_call('storeCategories();')->message('Game has been saved.');
        }
        else
        {
            return $this->error('Error in processing your request. Please refesh the page and try again.');
        }
    }

    public function favoriteRemove()
    {
        if( !SAA_Sys::isUser() )
        {
            return $this->error(SAA_Sys::getPhrase('favorite.you_must_be_logged_in_to_preform_this_action'));
        }

        $iUser = SAA_Sys::getUserId();
        $iGame = $this->_request()->get('game_id');

        if( !SAA_Sys::getSett('favorite.is_enabled') )
        {
            return $this->error(SAA_Sys::getPhrase('favorite.favorites_have_been_disabled'));
        }

        if( !SAA_Sys::getModel('game.favorite')->is($iGame) )
        {
            return $this->error(SAA_Sys::getPhrase('favorite.there_was_an_error_in_processing_your_request'));
        }

        if( SAA_Sys::getModel('game.favorite.process')->delete($iGame, $iUser) )
        {
            if( ($sCallFunc = $this->_request()->get('func')) )
            {
                $sFunc = 'loadFavorites();';
            }
            else
            {
                $sFunc = 'loadGames(\'favorite\');';
            }

            $this->_call($sFunc);
        }
        else
        {
            $this->error(SAA_Sys::getPhrase('favorite.there_was_an_error_in_processing_your_request'));
        }
    }

    /**
     * Make/unmake a game as favorite for a given user.
     */
    public function favoriteToggle()
    {
        if( !SAA_Sys::isUser() )
        {
            return $this->error(SAA_Sys::getPhrase('favorite.you_must_be_logged_in_to_preform_this_action'));
        }

        $iUser = SAA_Sys::getUserId();
        $iGame = $this->_request()->get('game_id');

        if( !SAA_Sys::getSett('favorite.is_enabled') )
        {
            return $this->error(SAA_Sys::getPhrase('favorite.favorites_have_been_disabled'));
        }

        if( SAA_Sys::getModel('game.favorite')->is($iGame) )
        {
            if( SAA_Sys::getModel('game.favorite.process')->delete($iGame, $iUser) )
            {
                $this->_call('$(\'.js_game_'.$iGame.'\').find(\'.js_fav\').hide(\'\'); $(\'.js_game_'.$iGame.'\').find(\'.js_make\').show(\'\');');
            }
            else
            {
                $this->error(SAA_Sys::getPhrase('favorite.there_was_an_error_in_processing_your_request'));
            }
        }
        else
        {
            if( SAA_Sys::getModel('game.favorite.process')->add($iGame, $iUser) )
            {
                $this->_call('$(\'.js_game_'.$iGame.'\').find(\'.js_fav\').show(\'\'); $(\'.js_game_'.$iGame.'\').find(\'.js_make\').hide(\'\');');
            }
            else
            {
                $this->error(SAA_Sys::getPhrase('favorite.there_was_an_error_in_processing_your_request'));
            }
        }
    }

    /**
     * Gets the various scores for a given game
     */
    public function getScores()
    {
        $this->html('#js_scores', $this->_loadController('game.score.index'));
    }

    /**
     * Gets the refeshed content for the game page
     */
    public function getUpdated()
    {
        $iGame = $this->_request()->get('game-id');

        $aGame = SAA_Sys::getModel('game')->getSimple($iGame);

        if( !$aGame )
        {
            return;
        }

        if( $aGame['scoring'] )
        {
            if( SAA_Sys::getSett('score.scores_on') )
            {
                $this->html('#top-scores', $this->_loadController('game.score.block'));
            }

            if( SAA_Sys::getSett('achievement.is_on') && SAA_Sys::getSett('achievement.on_game_page') )
            {
                $this->html('#achievements', $this->_loadController('game.achievement.block'));
            }
        }

        $this->html('#js_total_plays', $aGame['total_plays'])
            ->html('#js_total_scores', $aGame['total_scores'])
            ->html('#js_total_favorites', $aGame['total_favorites'])
            ->html('#js_total_votes', $aGame['total_votes']);

        ($aGame['total_plays'] ? $this->show('#js_total_plays_parent') : $this->hide('#js_total_plays_parent'));
        ($aGame['total_scores'] ? $this->show('#js_total_scores_parent') : $this->hide('#js_total_scores_parent'));
        (SAA_Sys::getSett('favorite.is_enabled') && $aGame['total_favorites'] ? $this->show('#js_total_favorites_parent') : $this->hide('#js_total_favorites_parent'));
        (SAA_Sys::getSett('rating.on_games') && $aGame['total_votes'] ? $this->show('#js_total_votes_parent') : $this->hide('#js_total_votes_parent'));
    }

    /**
     * Update featured games order
     */
    public function orderFeature()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aOrder = $this->_request()->getArray('aOrder');

        if( SAA_Sys::getModel('game.process')->editOrder($aOrder) )
        {
            $this->message('Order has been updated.');
        }
        else
        {
            $this->alert('Opps! There was an error in saving the order.');
        }
    }

    /**
     * Search games
     */
    public function search()
    {
        $aForm = $this->_request()->getArray('aForm');

        $sElement = $aForm['location'];

        $this->html('#'. $sElement, $this->_loadController('game.search'));
    }

    /**
     * Search games for admin
     */
    public function searchByAdmin()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#js_content', $this->_loadController('admin.game.search'));
    }

    /**
     * Rates a given game
     */
    public function rate()
    {
        $aRating = $this->_request()->get('rating');

        if( !isset($aRating['star']) )
        {
            $aRating['star'] = 0;
        }

        if( !SAA_Sys::isUser() )
        {
            return $this->error(SAA_Sys::getPhrase('game.you_must_be_logged_in_to_rate_this_game'));
        }
        elseif( !isset($aRating['item_id']) )
        {
            return $this->error(SAA_Sys::getPhrase('game.there_was_an_error_in_rating_the_game_please_refresh_the_page_and_try_again'));
        }
        elseif( ($mReturn = SAA_Sys::getModel('rating.process')->add('game', $aRating['item_id'], $aRating['star'])) )
        {
            return $this->_call('$(\'#js_rating_item_'.$aRating['item_id'].'\').parents(\'.js_rating:first\').find(\'.js_rating_total\').val('.$mReturn[0].');')
                    ->_call('$(\'#js_rating_item_'.$aRating['item_id'].'\').parents(\'.js_rating:first\').find(\'.js_rating_value\').val('.$mReturn[1].');');
        }
        else
        {
            return $this->error(SAA_Sys::getPhrase('game.there_was_an_error_in_rating_the_game_please_refresh_the_page_and_try_again'));
        }
    }

    /**
     * Feature/Unfeature game
     */
    public function toggleFeature()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aEdit');

        if( !$aForm )
        {
            $iGame = $this->_request()->get('game_id');

            if( !$iGame )
            {
                return $this->error('Opps! There was an error in processing your request.');
            }

            $aForm['game_id'][] = $iGame;
            $aForm['action'] = $this->_request()->get('action');
        }

        if( SAA_Sys::getModel('game.process')->toggleFeatured($aForm['game_id'], $aForm['action']) )
        {
            $sAction = ($aForm['action'] == 'feature' ? '.removeClass(\'ui-icon-radio-on\').addClass(\'ui-icon-bullet\')' : '.removeClass(\'ui-icon-bullet\').addClass(\'ui-icon-radio-on\')');

            foreach($aForm['game_id'] as $iGame)
            {
                $this->_call('$(\'#js_game_'.(int) $iGame.'\').find(\'.js_game_feature\')'.$sAction.';');
            }

            $this->message('The selected '.(count($aForm['game_id']) > 1 ? 'games have' : 'game has').' been '.($aForm['action'] == 'feature' ? 'featured' : 'unfeatured').'.');
        }
        else
        {
            $this->error('There was an error in trying to process your request.');
        }
    }

    /**
     * Checks if the text is an unique slug
     */
    public function uniqueSlug()
    {
        $sSlug = $this->_request()->get('slug');

        if( SAA_Sys::getModel('game')->isUniqueSlug($sSlug) )
        {
            $this->hide('#js_url_slug_msg');
        }
        else
        {
            $this->show('#js_url_slug_msg');
        }
    }

    /**
     * Update game plays counter
     */
    public function updateCounter()
    {
        $iGame = $this->_request()->get('game-id');

        SAA_Sys::getModel('game.process')->updateCounter($iGame);
    }
}
?>
