<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Process Language Class
 * @license GoLevel LLC
 */

Class SAA_Ajax_Language extends SAA_Ajax
{
    /*
     * Makes a language the default
     */
    public function defaultLanguage()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iLanguage = $this->_request()->getInt('id');

        if( SAA_Sys::getModel('language.process')->setDefault($iLanguage) )
        {
           $this->hide('.is_default')
                ->removeClass('.js_default', 'is_default')
                ->show('.js_item_default')
                ->show('#js_item_default_' . $iLanguage)
                ->hide('#js_item_not_default_' . $iLanguage)
                ->addClass('#js_item_default_' . $iLanguage, 'is_default')
                ->message('A new default language has been selected.');
        }
        else
        {
           $this->error('Language could not be set as default.');
        }
    }

    /*
     * Removes a language
     */
    public function deleteLanguage()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iLanguage = $this->_request()->getInt('id');

        if( SAA_Sys::getModel('language.process')->delete($iLanguage) )
        {
            $this->remove('#js_language_loading_' . $iLanguage)
                ->remove('#js_language_' . $iLanguage)
                ->message('Language has been deleted.');
        }
        else
        {
            $this->error('Language could not be deleted.');
        }
    }

    /*
     * Removes a phrase
     */
    public function deletePhrase()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aEdit = $this->_request()->getArray('aEdit');

        if( isset($aEdit['phrase_id']) && count($aEdit['phrase_id']) )
        {
            $oPhraseProcess = SAA_Sys::getModel('language.phrase.process');

            foreach($aEdit['phrase_id'] as $iPhrase)
            {
                $iPhrase = (int) $iPhrase;

                $oPhraseProcess->delete($iPhrase, true);
            }

            $this->message('Language phrase'.(count($aEdit['phrase_id']) > 1 ? 's' : '').' were deleted.');
        }
        else
        {
            $this->error('Language phrase(s) could not be deleted.');
        }
    }

    /*
     * Removes a section
     */
    public function deleteSection()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iId = $this->_request()->getInt('id');

        if( SAA_Sys::getModel('language.section.process')->delete($iId) )
        {
            $this->remove('#js_section_' . $iId);
        }
        else
        {
            $this->error('Language section could not be deleted.');
        }
    }

    /*
     * Gets the number of phrases in a section
     */
    public function deleteSectionCheck()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iId = $this->_request()->getInt('id');

        $iPhrases = SAA_Sys::getModel('language.section')->getPhraseNum($iId);

        $this->_call($iPhrases);
    }

    /*
     * Turns a language on/off
     */
    public function enableLanguage()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iLanguage = $this->_request()->getInt('id');
        $iSwitch = ($this->_request()->get('action') == 'on' ? 1 : 0);

        if( SAA_Sys::getModel('language.process')->toggleLanguage($iLanguage, $iSwitch) )
        {
           if( $iSwitch == 1 )
           {
               $sElementShow = '#js_item_disable_' . $iLanguage;
               $sElementHide = '#js_item_enable_' . $iLanguage;
           }
           else
           {
               $sElementShow = '#js_item_enable_' . $iLanguage;
               $sElementHide = '#js_item_disable_' . $iLanguage;
           }

           $this->show($sElementShow)
                ->hide($sElementHide)
                ->message('Language has been turned '.($iSwitch == 1 ? 'on' : 'off').'.');
        }
        else
        {
           $this->error('Language could not be turned '.($iSwitch == 1 ? 'on' : 'off').'.');
        }
    }

    /*
     * Imports a language
     */
    public function import()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aVal = $this->_request()->getArray('aVal');

        if( $aVal['stage'] == 'missing-phrases' )
        {
            if( SAA_Sys::getModel('language.phrase.process')->addMissing($aVal['language_id']) )
            {
                $this->val('#js_import_stage', 'missing-game-phrases')
                    ->show('#js_language_loader')
                    ->trigger('#js_import_submit', 'submit');
            }
        }
        elseif( $aVal['stage'] == 'missing-game-phrases' )
        {
            $iLeft = SAA_Sys::getModel('language.game.process')->addMissing($aVal['language_id']);

            if( $iLeft > 0 )
            {
                $this->show('#js_language_loader')
                    ->val('#js_import_left', $iLeft)
                    ->trigger('#js_import_submit', 'submit');
            }
            else
            {
                $this->val('#js_import_stage', 'adding-phrases')
                    ->show('#js_language_loader')
                    ->trigger('#js_import_submit', 'submit');
            }
        }
        elseif( $aVal['stage'] == 'adding-phrases' )
        {
            if( SAA_Sys::getModel('language.phrase.process')->addMissingToAll($aVal['language_id']) )
            {
                $this->val('#js_import_stage', '')
                    ->val('#js_import_id', '')
                    ->show('#js_file_table')
                    ->message('Language has been imported to the arcade.', array('auto_close' => 0));
            }
        }
    }

    /*
     * Restores a phrase to original text
     */
    public function revertPhrase()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aEdit = $this->_request()->getArray('aEdit');

        if( isset($aEdit['phrase_id']) && count($aEdit['phrase_id']) )
        {
            $oPhraseProcess = SAA_Sys::getModel('language.phrase.process');

            foreach($aEdit['phrase_id'] as $iPhrase)
            {
                $iPhrase = (int) $iPhrase;

                $oPhraseProcess->revert($iPhrase);
            }

            $this->message('Language phrase'.(count($aEdit['phrase_id']) > 1 ? 's' : '').' were reverted to original phrase'.(count($aEdit['phrase_id']) > 1 ? 's' : '').'.');
        }
        else
        {
            $this->error('Language phrase(s) could not be changed back to the original.');
        }
    }

    /*
     * Cleans input in section_name field
     */
    public function sanitizeSection()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        if( isset($aForm['section_name']) && strlen($aForm['section_name']) )
        {
            $sSectionName = SAA_Sys::getLib('parse.input')->toAlphaNumeric($aForm['section_name'], 20, '_', '_');

            if( $aForm['section_name'] != $sSectionName )
            {
                $this->val("#js_input_name", $sSectionName);
                $this->_call('$(\'#js_input_name_msg\').show().delay(800).fadeOut(1000);');
            }
        }
    }

    /*
     * Cleans input in var_name field
     */
    public function sanitizeVarName()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        if( isset($aForm['var_name']) && strlen($aForm['var_name']) )
        {
            $oParseInput = SAA_Sys::getLib('parse.input');

            $sPhraseName = $oParseInput->toAlphaNumeric($aForm['var_name'], 250, '_', '_');
            $sPhraseName = $oParseInput->getUnique($sPhraseName, 'language_phrase', array('var_name', 'section_id' => $aForm['section_id']), 250);

            if( $aForm['var_name'] != $sPhraseName )
            {
                $this->val("#js_language_name", $sPhraseName);
                $this->_call('$(\'#js_language_name_msg\').show().delay(800).fadeOut(1000);');
            }
        }
    }

    /*
     * Updates phrase data
     */
    public function savePhrase()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aEdit = $this->_request()->getArray('aEdit');

        if( isset($aEdit['phrase_id']) && count($aEdit['phrase_id']) )
        {
            $oPhraseProcess = SAA_Sys::getModel('language.phrase.process');

            if( !isset($aEdit['text']) )
            {
                SAA_Error::add('Phrases could not be saved. Error in processing');
            }
            else
            {
                foreach($aEdit['phrase_id'] as $iPhrase)
                {
                    if( !isset($aEdit['text'][$iPhrase]) )
                    {
                        continue;
                    }

                    $aPhrase = array(
                        'phrase_id' => (int) $iPhrase,
                        'text' => $aEdit['text'][$iPhrase],
                    );

                    $oPhraseProcess->edit($aPhrase);
                }

                $this->message('Language phrase'.(count($aEdit['phrase_id']) > 1 ? 's' : '').' were saved');
            }
        }
        else
        {
            $this->error('Language phrase(s) could not be saved.');
        }
    }

    /*
     * Loads result from phrase search
     */
    public function searchPhrases()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#js_search_content', $this->_loadController('admin.language.phrase.search'));
    }

    /*
     * Check input for use free
     */
    public function uniqueSection()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aForm');

        if( isset($aForm['section_name'], $aForm['section_id']) && strlen($aForm['section_name']) )
        {
            $sSectionName = SAA_Sys::getLib('parse.input')->toAlphaNumeric($aForm['section_name'], 20, '_', '_');

            $aSection = SAA_Sys::getModel('language.section')->get($sSectionName, false);

            if( $aForm['section_name'] != $sSectionName )
            {
                $this->val("#js_input_name", $sSectionName);
            }

            if( $aSection && $aSection['section_id'] != $aForm['section_id'] )
            {
                $this->html("#js_input_name_msg", '<i>'.($sSectionName != $aForm['section_name'] ? 'Name has been sanitized. ' : '').'The name is already in use by another section.</i>');
                $this->show('#js_input_name_msg');
            }
            else
            {
                $this->hide('#js_input_name_msg');
            }
        }
    }
}
?>
