<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Process User Class
 * @license GoLevel LLC
 */

Class SAA_Ajax_User extends SAA_Ajax
{
    /**
     * Activates user(s)
     */
    public function activate()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aEdit');

        if( !$aForm )
        {
            $iUser = $this->_request()->get('user_id');

            if( !$iUser )
            {
                return $this->error('Opps! There was an error in processing your request.');
            }

            $aForm['user_id'][] = $iUser;
        }

        if( SAA_Sys::getModel('user.process')->activate($aForm['user_id']) )
        {
            $this->message('The selected '.(count($aForm['user_id']) > 1 ? 'users have' : 'user has').' been activated.')
                ->_call('iPage = $(\'#js_search_page\').val(); triggerSearch(iPage);');
        }
        else
        {
            $this->error('There was an error in trying to process your request.');
        }
    }

    /**
     * Deactivate user(s)
     */
    public function deactivate()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aEdit');

        if( !$aForm )
        {
            $iUser = $this->_request()->get('user_id');

            if( !$iUser )
            {
                return $this->error('Opps! There was an error in processing your request.');
            }

            $aForm['user_id'][] = $iUser;
        }

        if( SAA_Sys::getModel('user.process')->deactivate($aForm['user_id']) )
        {
            $this->message('The selected '.(count($aForm['user_id']) > 1 ? 'users have' : 'user has').' been deactivated.')
                ->_call('iPage = $(\'#js_search_page\').val(); triggerSearch(iPage);');
        }
        else
        {
            $this->error('There was an error in trying to process your request.');
        }
    }

    /**
     * Deletes user(s)
     */
    public function delete()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#ajax-dialog-box', $this->_loadController('admin.user.delete'));
    }

    public function findLike()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $iLimit = 10;
        $sLike = $this->_request()->get('term');

        return SAA_Sys::getModel('user')->findLike($sLike, $iLimit);
    }

    public function forgotPassword()
    {
        $sEmail = $this->_request()->get('email');

        if( !SAA_Sys::getLib('validator')->checkRegex('email', $sEmail) )
        {
            return $this->error(SAA_Sys::getPhrase('user.the_email_address_is_not_valid'));
        }

        if( SAA_Sys::getModel('user.process')->emailPassword($sEmail) )
        {
            $this->message(SAA_Sys::getPhrase('user.email_has_been_sent_to_email', array('email' => $sEmail)), array('display' => 'div', 'location' => '#form', 'auto_close' => false));
        }
    }

    /**
     * Search users for admin
     */
    public function searchAdmin()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $this->html('#js_content', $this->_loadController('admin.user.search'));
    }

    /**
     * Send verify account email
     */
    public function sendActivationEmail()
    {
        if( !$this->isAdmin() )
        {
            return false;
        }

        $aForm = $this->_request()->getArray('aEdit');

        if( !$aForm )
        {
            $iUser = $this->_request()->get('user_id');

            if( !$iUser )
            {
                return $this->error('Opps! There was an error in processing your request.');
            }

            $aForm['user_id'][] = $iUser;
        }

        if( SAA_Sys::getModel('user.process')->emailVerificationMass($aForm['user_id']) )
        {
            $this->message('The verification email has been sent to the selected '.(count($aForm['user_id']) > 1 ? 'users' : 'user').'.')
                ->_call('iPage = $(\'#js_search_page\').val(); triggerSearch(iPage);');
        }
        else
        {
            $this->error('There was an error in trying to process your request.');
        }
    }
}
?>