<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Admin Category Edit Controller
 * @license GoLevel LLC
 */

Class SAA_Controller_Admin_Category_Edit extends SAA_Controller
{
    /**
     * Controller preloader
     */
    public function preload()
    {
        SAA_Sys::getModel('admin')->preload();
    }

    /**
     * Controller loader
     */
    public function load()
    {
        $oCategoryMap = SAA_Sys::getModel('category.map');

        $iId = $this->request()->getInt('id');
        $aLanguages = SAA_Sys::getModel('language')->getAll(true);

        $this->template()->setHeader('Edit Category', true);

        if( !count($aLanguages) )
        {
            return SAA_Error::add('At least one language must exist in order to continue.');
        }
        elseif( !($aCategory = SAA_Sys::getModel('category')->get($iId, true)) )
        {
            return SAA_Error::add('Category cannot be found.');
        }

        $aMaps = $oCategoryMap->getByCategoryId($iId); //Get category maps for the given category
        $aTabs = $oCategoryMap->getAllForList(); //Get all the category maps in tab format

        //store given category's maps into tab
        foreach($aTabs as $iType => $aRows)
        {
            foreach($aRows as $iRow => $aRow)
            {
                $aTabs[$iType][$iRow]['default'] = (in_array($aRow['map_id'], $aMaps) ? 1 : 0);
            }
        }

        $aPhrases = SAA_Sys::getModel('language.phrase')->getByVar($aCategory['name']); //get phrases for the given category

        //store given category's phrases into language
        foreach($aLanguages as $iLanguage => $sLanguage)
        {
            $aLanguages[$iLanguage] = array(
                'title' => $sLanguage,
                'default' => $aPhrases[$iLanguage]['text']
            );
        }

        if( ($aVal = $this->request()->get('aForm'))  )
        {
            if( SAA_Sys::getModel('category.process')->edit($aVal, $aCategory) )
            {
               $this->url()->reload(array('id' => $iId), 'Category has been modified. <a href="'. $this->url()->makeSimple('admin.category') .'">View All</a>');
            }
        }

        $this->template()
            ->assign(array(
                'aGameTypes' => SAA_Sys::getModel('game.type')->getAll(),
                'aTabs' => $aTabs,
                'aLanguages' => $aLanguages,
                'aCategory' => $aCategory
        ));
    }
}
?>
