<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Admin Language Export Controller
 * @license GoLevel LLC
 */

Class SAA_Controller_Admin_Language_Export extends SAA_Controller
{
    /*
     * Controller preloader
     */
    public function preload()
    {
        SAA_Sys::getModel('admin')->preload();
    }

    /*
     * Controller loader
     */
    public function load()
    {
        $aExport = array();
        $iId = $this->request()->getInt('id');

        $aLanguage = SAA_Sys::getModel('language')->get($iId);

        if( !$aLanguage )
        {
            return SAA_Error::add('Language you are trying to export cannot be found.');
        }

        $aPhrases = SAA_Sys::getModel('language.phrase')->getAllById($iId);
        $aSections = SAA_Sys::getModel('language.section')->getByLanguage($iId, true);

        $aSectionsInUse = array();

        foreach($aSections as $aSection)
        {
            $aSectionsInUse[] = $aSection['section_name'];
        }

        $aExport['language']['tag'] = 'language';

        //Language Info
        $aExport['language']['children'][] = array(
            'tag' => 'info',
            'children' => array(
                array(
                    'tag' => 'title',
                    'value' => $aLanguage['title']
                ),
                array(
                    'tag' => 'language_abrv',
                    'value' => $aLanguage['language_abrv']
                ),
                array(
                    'tag' => 'direction',
                    'value' => $aLanguage['direction']
                ),
                array(
                    'tag' => 'creator',
                    'value' => SAA_Sys::getLib('parse.output')->htmlify($aLanguage['created_by'])
                ),
            )
        );

        //Phrases
        $aChild = array(
            'tag' => 'phrases'
        );

        foreach($aPhrases as $aPhrase)
        {
            if( !in_array($aPhrase['section_id'], $aSectionsInUse) )
            {
                continue;
            }

            $aChild['children'][] = array(
                'tag' => 'phrase',
                'value' => $aPhrase['text'],
                'attributes' => array(
                    'section_id' => $aPhrase['section_id'],
                    'var_name' => $aPhrase['var_name'],
                    'time_added' => $aPhrase['time_added']
                )
            );
        }

        $aExport['language']['children'][] = $aChild;

        //Sections
        $aChild = array(
            'tag' => 'sections'
        );

        foreach($aSections as $aSection)
        {
            $aChild['children'][] = array(
                'tag' => 'section',
                'value' => $aSection['section_title'],
                'attributes' => array(
                    'name' => $aSection['section_name']
                )
            );
        }

        $aExport['language']['children'][] = $aChild;

        $sFile = SAA_Sys::getLib('parse.input')->toAlphaNumeric('saa_language_'.$aLanguage['title'].'.xml', '._', '_');
        $sXml = SAA_Sys::getLib('xml.builder')->generateFile($aExport, $sFile);
        exit;
    }
}
?>
