<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Browse Controller
 * @license GoLevel LLC
 */

Class SAA_Controller_Browse extends SAA_Controller
{
    /**
     * Controller preloader
     */
    public function preload()
    {
        SAA_Sys::getModel('public')->load();
    }

    /**
     * Controller loader
     */
    public function load()
    {
        if( !SAA_Sys::isUser() && !SAA_Sys::getSett('game.guests_can_browse') )
        {
            $this->url()->sendWithCrumb('login', SAA_Sys::getPhrase('browse.must_be_logged_in_to_browse_games'));
        }

        $this->template()->setHeader(SAA_Sys::getSett('meta.browse_title'), true);

        $aFilter = array();
        $aVars = array();

        $aCategories = SAA_Sys::getLib('category')->getAllSimple();

        if( ($sCategory = $this->request()->get('slug_1')) && ($aCategory = SAA_Sys::getLib('category')->get($sCategory, false)) )
        {
            $aFilter['category'] = $sCategory;
        }

        if( ($sTitle = $this->request()->get('title') ) )
        {
            $aFilter['title'] = $this->url()->decodeBase64($sTitle);
            $aVars['title'] = $sTitle;
        }

        if( ($sTag = $this->request()->get('tag') ) )
        {
            $aFilter['tag'] = $sTag;
            $aVars['tag'] = $this->url()->encode($sTag);
        }

        if( ($aForm = $this->request()->get('aForm')) )
        {
            $aFilter = $aVars = $aForm;

			if( isset($aFilter['category']) )
			{
				$aCategory = SAA_Sys::getLib('category')->get($aFilter['category'], false);

				if( !$aCategory )
				{
					unset($aFilter['category'], $aVars['category']);
				}
			}

            if( strlen($aVars['title']) )
            {
                $aVars['title'] = $this->url()->encodeBase64($aVars['title']);
            }
            else
            {
                unset($aVars['title']);
            }

            unset($aVars['category']);
        }
        else
        {
            $this->request()->set('aForm', array(
                'title' => (isset($aFilter['title']) ? $aFilter['title'] : ''),
                'category' => (isset($aFilter['category']) ? $aFilter['category'] : ''),
            ));
        }

        if( isset($aFilter['category']) )
        {
            $this->template()->setHeader(SAA_Sys::getPhrase($aCategory['name']), true);
        }

        $iPg = $this->request()->getInt('pg', 1);
        $iPgSize = SAA_Sys::getSett('game.per_pg_on_browse');

        list($iGames, $aGames) = SAA_Sys::getModel('game.search')->execute(false, $aFilter, $iPg, $iPgSize);

        $this->template()
            ->setDescription(SAA_Sys::getSett('meta.browse_description'))
            ->assign(array(
                'bRatingOn' => (SAA_Sys::getModel('rating')->isEnabled('game') ? true : false),
                'aCategories' => $aCategories,
                'iPg' => $iPg,
                'iPgSize' => $iPgSize,
                'sPgLink' => $this->url()->makeUrl(array(
                    'controller' => 'browse' . (isset($aFilter['category']) ? '.' . $aFilter['category'] : ''),
                    'vars' => array_merge($aVars, array('pg' => '{pg}'))
                )),
                'iGames' => $iGames,
                'aGames' => $aGames
        ));
    }
}
?>
