<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Category Map Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Category_Map extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Gets a category map
     *
     * @param int iId
     * @return mixed
     */
    public function get($iId)
    {
        $aMap = $this->database()
            ->select('*')
            ->from('category_map')
            ->where('map_id = '. (int) $iId)
            ->get('row');

        if( !$aMap )
        {
            return SAA_Error::add('Category does not exist.');
        }

        $aMap['categories'] = $this->database()
                ->select('COUNT(*)')
                ->from('category_data')
                ->where('map_id = '. $iId)
                ->get('column');

        $aMap['games'] = $this->database()
                ->select('COUNT(*)')
                ->from('game_category_map')
                ->where('map_id = '. $iId)
                ->get('column');

        return $aMap;
    }

     /**
     * Gets all category maps
     *
     * @return array
     */
    public function getAll()
    {
	return $this->database()
                ->select('*')
                ->from('category_map')
                ->get('rows');
    }

    /**
     * Gets category maps, using game type as organizing
     *
     * @return array
     */
    public function getAllForList()
    {
        $aMaps = array();

        $aRows = $this->getAll();

        foreach($aRows as $aMap)
        {
            $aMaps[$aMap['type_id']][] = $aMap;
        }

        return $aMaps;
    }

     /**
     * Gets all category maps for game type
     *
     * @return array
     */
    public function getAllType($iType, $bByName = true)
    {
        $aMap = array();

	$aRows = $this->database()
                ->select('*')
                ->from('category_map')
                ->where('type_id = '. (int) $iType)
                ->get('rows');

        if( $bByName )
        {
            foreach($aRows as $aRow)
            {
                $aMap[$aRow['map_name']] = $aRow['map_id'];
            }
        }
        else
        {
            foreach($aRows as $aRow)
            {
                $aMap[$aRow['map_id']] = $aRow;
            }
        }

        return $aMap;
    }

    /**
     * Gets all mapped categories for all categories
     *
     * @return array
     */
    public function getByCategories()
    {
        return $this->database()
            ->select('*')
            ->from('category_data')
            ->get('rows');
    }

    /**
     * Gets all mapped categories for a given category
     *
     * @return array
     */
    public function getByCategoryId($iCategory)
    {
        $aMaps = array();

        $aRows = $this->database()
            ->select('map_id')
            ->from('category_data')
            ->where('category_id = '. (int) $iCategory)
            ->get('rows');

        foreach($aRows as $aRow)
        {
            $aMaps[] = $aRow['map_id'];
        }

        return $aMaps;
    }

    /**
     * Gets the categories, using the map_id or category_id as sorting
     *
     * @param bool $bMapId
     * @return array
     */
    public function getKeys($bMapId = false)
    {
        $aCategories = array();

        $aRows = $this->getByCategories();

        if( $bMapId )
        {
            foreach($aRows as $aRow)
            {
                $aCategories[$aRow['map_id']][] = $aRow['category_id'];
            }

            return $aCategories;
        }

        foreach($aRows as $aRow)
        {
            $aCategories[$aRow['category_id']][] = $aRow['map_id'];
        }

        return $aCategories;
    }
}
?>
