<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Category Map Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Category_Map_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

     /**
     * Adds a category map
     *
     * @param array aVal
     * @return bool
     */
    public function add($aVal)
    {
        if( !SAA_Sys::getModel('game.type')->get($aVal['type_id']) )
        {
            SAA_Error::add('Invalid game type.');
        }

        if( !strlen($aVal['map_title']) )
        {
            SAA_Error::add('Please provide the title of the map category.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $iExist = $this->database()
                ->select('COUNT(*)')
                ->from('category_map')
                ->where('type_id = "' . $this->database()->esc($aVal['type_id']). '" AND map_title = "'. $this->database()->esc($aVal['map_name']) .'"')
                ->get('column');

        if( $iExist > 0 )
        {
            return SAA_Error::add('Category map already exists.');
        }

        $aSql = array(
            'type_id' => $aVal['type_id'],
            'map_title' => $aVal['map_title'],
            'map_name' => $aVal['map_name']
        );

        $iId = $this->database()->insert('category_map', $aSql);

        if( !$iId )
        {
            return SAA_Error::add('There was an error in adding your category map. Please try again.');
        }

        return true;
    }

    /**
     * Adds any missing category maps
     *
     * @return array
     */
    public function auto()
    {
        $aAll = array();

        $aInDb = SAA_Sys::getModel('category.map')->getAll();
        $aSource = SAA_Sys::getModel('game.type')->getCategoryMaps();

        foreach($aSource as $iGameType => $aMaps)
        {
            foreach($aMaps as $sTitle => $sName)
            {
                $mExists = false;
                foreach($aInDb as $aExist)
                {
                    if( $aExist['type_id'] == $iGameType && $aExist['map_name'] == $sName )
                    {
                        $mExists = $aExist['map_id'];
                    }
                }

                if( !$mExists )
                {
                    $aSql = array(
                        'type_id' => $iGameType,
                        'map_title' => $sTitle,
                        'map_name' => $sName
                    );

                    $mExists = $this->database()->insert('category_map', $aSql);
                }

                $aAll[$iGameType][$sName] = $mExists;
            }
        }

        return $aAll;
    }

    /**
     * Updates the category maps for a given category
     *
     * @param mixed $mVal array or string containing all map_id
     * @param int $iCategory
     * @return bool
     */
    public function modifyForCategory($mVal, $iCategory)
    {
        if( is_string($mVal) )
        {
            $mVal = (array) $mVal;
        }

        $aMapped = SAA_Sys::getModel('category.map')->getByCategoryId($iCategory);

        //insert
        foreach($mVal as $iMap => $sBlank)
        {
            if( !in_array($iMap, $aMapped) )
            {
                $this->database()->insert('category_data', array('category_id' => $iCategory, 'map_id' => $iMap));
            }
        }

        //delete
        foreach($aMapped as $iMap)
        {
            if( !in_array($iMap, $mVal) )
            {
                $this->database()->delete('category_data', 'category_id = '. (int) $iCategory .' AND map_id = '. (int) $iMap);
            }
        }

        return true;
    }

    /**
     * Edits an existing category map
     *
     * @param array aVal
     * @return bool
     */
    public function edit($aVal)
    {
        if( !SAA_Sys::getModel('game.type')->get($aVal['type_id']) )
        {
            SAA_Error::add('Invalid game type.');
        }

        if( !strlen($aVal['map_title']) )
        {
            SAA_Error::add('Please provide the title of the map category.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $iExist = $this->database()
                ->select('COUNT(*)')
                ->from('category_map')
                ->where('type_id = "' . $this->database()->esc($aVal['type_id']). '" AND map_title = "'. $this->database()->esc($aVal['map_name']) .'" AND map_id != '. (int) $aVal['map_id'])
                ->get('column');

        if( $iExist > 0 )
        {
            return SAA_Error::add('Category map with the given name already exists.');
        }

        $aSql = array(
            'type_id' => $aVal['type_id'],
            'map_title' => $aVal['map_title'],
            'map_name' => $aVal['map_name']
        );

        $this->database()->update('category_map', $aSql, 'map_id = '. (int) $aVal['map_id']);

        return true;
    }

    /**
     * Deletes a category map
     * Removes the map data from category_data and game_category
     *
     * @param int iId
     * @return bool
     */
    public function delete($iId)
    {
        $this->database()->delete('category_map', 'map_id = '. (int) $iId);

        $this->database()->delete('category_data', 'map_id = '. (int) $iId);

        $this->database()->delete('game_category_map', 'map_id = '. (int) $iId);

        $this->cache()->remove('category_map');

        return true;
    }
}
?>
