<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Category Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Category_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Creates a category
     *
     * @param array $aVal insert data
     * @return bool
     */
    public function add($aVal)
    {
        //if the language phrase is not created, return false
        reset($aVal['text']);
        $sRaw = current($aVal['text']);

        $aPhrase = array(
            'section_id' => 'category',
            'text' => $aVal['text'],
            'var_name' => $sRaw
        );

        if( !($mPhrase = SAA_Sys::getModel('language.phrase.process')->add($aPhrase)) )
        {
            return false;
        }

        $aSql = array(
            'name' => $mPhrase['section_id'] .'.'. $mPhrase['var_name'],
            'slug' => $this->parse()->getUnique($sRaw, 'category', 'slug'),
            'time_stamp' => SAA_TIME
        );

        $iId = $this->database()->insert('category', $aSql);

        if( !$iId )
        {
            return SAA_Error::add('The category could not be added.');
        }

        //Add any mapped categories to added category
        if( isset($aVal['map']) && count($aVal['map']) )
        {
            SAA_Sys::getModel('category.map.process')->modifyForCategory($aVal['map'], $iId);
        }

        $this->clearCache();

        return $iId;
    }

    /**
     * Adds any missing categories, using game type sources
     *
     * @param array $aVals
     * @param array $aCategories
     * @return bool
     */
    public function addMass($aVals, $aCategories)
    {
        /**
         * Adds any missing category maps
         */
        $aMaps = SAA_Sys::getModel('category.map.process')->auto();

        $aLanguages = SAA_Sys::getModel('language')->getAll(true); //get languages

        $aSelected = $aVals['category']; //get the selected category data

        foreach($aSelected as $sCategory)
        {
            if( isset($aCategories[$sCategory]) )
            {
                continue;
            }

            $aVal = array();

            foreach($aLanguages as $iLanguage => $sLanguage)
            {
                $aVal['text'][$iLanguage] = $sCategory;
            }

            foreach($aCategories[$sCategory] as $iGameType => $sMapName)
            {
                $aVal['map'][ $aMaps[$iGameType][$sMapName] ] = 1;
            }

            $this->add($aVal);
        }

        return true;
    }

    /**
     * Removes all the cache files for categories
     *
     * @return bool
     */
    public function clearCache()
    {
        return $this->cache()->remove(array('category', 'category_map'));
    }

    /**
     * Update the data for a given category
     *
     * @param array $aVals updating data
     * @param array $aCategory current array data
     * @return bool
     */
    public function edit($aVals, $aCategory)
    {
        //if the language phrases are not updated, return error
        if( !($mPhrase = SAA_Sys::getModel('language.phrase.process')->editAll($aVals['text'], $aCategory['name'])) )
        {
            return false;
        }

        $aSql = array(
            'time_stamp' => SAA_TIME
        );

        if( strlen($aVals['slug']) && $aVals['slug'] != $aCategory['slug'] )
        {
            $aCheck = SAA_Sys::getModel('category')->get($aVals['slug']);

            if( $aCheck && $aCheck['category_id'] != $aCategory['category_id'] )
            {
                return SAA_Error::add('The url slug is already in use by another category.');
            }

            $aSql['slug'] = $this->parse()->getUnique($aVals['slug'], 'category', 'slug');
        }

        $this->database()->update('category', $aSql, 'category_id = '. (int) $aCategory['category_id']);

        if( isset($aVals['map']) )
        {
            $aMapped = $aVals['map'];
        }
        else
        {
            $aMapped = array();
        }

        //Update the mapped categories
        SAA_Sys::getModel('category.map.process')->modifyForCategory($aMapped, $aCategory['category_id']);

        $this->clearCache();

        return true;
    }

    /**
     * Updates the order of the categories
     *
     * @param array $aOrder
     * @return true
     */
    public function editOrder($aOrder)
    {
        foreach($aOrder as $iOrder => $iCategory)
        {
            $this->database()->update('category', array('ordering' => (int) $iOrder), 'category_id = '. (int) $iCategory);
        }

        $this->clearCache();

        return true;
    }

    /**
     * Deletes a given category by category_id
     *
     * @param int $iId
     * @return bool
     */
    public function delete($iId)
    {
        if( !($aCategory = SAA_Sys::getModel('category')->get($iId, true)) )
        {
            return false;
        }

        //deletes from category related rows
        $this->database()->delete('category', 'category_id = '. (int) $iId);
        $this->database()->delete('category_data', 'category_id = '. (int) $iId);

        //deletes language phrase
        SAA_Sys::getModel('language.phrase.process')->delete($aCategory['name']);

        //clears the category cache
        $this->clearCache();

        return true;
    }
}
?>
