<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Comment Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Comment extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /*
     * Gets Comment
     *
     * @param int iComment comment id
     *
     * @return mixed
     */
    public function get($iComment)
    {
        $sDate = SAA_Sys::getSett('comment.date_format');

        $aRow = $this->database()
                ->select('c.*, ct.*, '. SAA_Sys::getModel('user')->getUserFields('u', 'comment'))
                ->from('comment', 'c')
                ->join('comment_text', 'ct', 'ct.comment_id = c.comment_id')
                ->join('user', 'u', 'u.user_id = c.user_id')
                ->where('c.comment_id = '. (int) $iComment)
                ->get('row');

        if( !$aRow )
        {
            return false;
        }

        $aRow['time_date'] = date($sDate, $aRow['time_stamp']);

        if(  $aRow['update_time'] )
        {
            $aRow['update_date'] = date($sDate, $aRow['update_time']);
        }

        return $aRow;
    }

    /*
     * Get comments for given item
     *
     * @param int iItem item id
     * @param mixed iParent parent id
     * @param string sModel model
     * @param int iPage page
     * @param int iLimit limit
     *
     * @return mixed
     */
    public function getAll($iItem, $iParent, $sModel, $iPg, $iLimit)
    {
        $sDate = SAA_Sys::getSett('comment.date_format');

        if( $iParent == null )
        {
            $iParent = 0;
        }

        $iCnt = $this->database()
            ->select('COUNT(*)')
            ->from('comment')
            ->where('item_id = '. (int) $iItem .' AND model_id = "'.$this->database()->esc($sModel).'" AND parent_id = '. (int) $iParent)
            ->get('column');

        $aComments = $this->database()
            ->select('c.*, ct.*, '. SAA_Sys::getModel('user')->getUserFields('u', 'comment'))
            ->from('comment', 'c')
            ->join('comment_text', 'ct', 'c.comment_id = ct.comment_id')
            ->join('user', 'u', 'u.user_id = c.user_id')
            ->where('item_id = '. (int) $iItem .' AND model_id = "'.$this->database()->esc($sModel).'" AND parent_id = '. (int) $iParent)
            ->order('time_stamp', 'ASC')
            ->limit($iLimit, $iPg)
            ->key('comment_id')
            ->get('rows');

        foreach($aComments as $iComment => $aComment)
        {
            $aComments[$iComment]['time_date'] = date($sDate, $aComment['time_stamp']);

            if(  $aComment['update_time'] )
            {
                $aComments[$iComment]['update_date'] = date($sDate, $aComment['update_time']);
            }
        }

        if( SAA_Sys::getSett('comment.enable_child') )
        {
            $sIds = implode(', ', array_keys($aComments));

            if( $sIds )
            {
                $aChildCnt = $this->database()
                    ->select('COUNT(*) as total, comment_id, parent_id')
                    ->from('comment')
                    ->where('parent_id IN ('. $sIds .') AND model_id = "'.$this->database()->esc($sModel).'"')
                    ->group('parent_id')
                    ->key('parent_id')
                    ->get('rows');

                if( !isset($aChildCnt['']) )
                {
                    foreach($aComments as $iComment => $aComment)
                    {
                        if( !isset($aChildCnt[$iComment]) || $aChildCnt[$iComment]['total'] == 0 )
                        {
                            continue;
                        }

                        $aChildren = $this->database()
                            ->select('c.*, ct.*, '. SAA_Sys::getModel('user')->getUserFields('u', 'comment'))
                            ->from('comment', 'c')
                            ->join('comment_text', 'ct', 'c.comment_id = ct.comment_id')
                            ->join('user', 'u', 'u.user_id = c.user_id')
                            ->where('c.parent_id = '. (int) $iComment .' AND c.model_id = "'.$this->database()->esc($sModel).'"')
                            ->order('time_stamp', 'ASC')
                            ->limit($iLimit)
                            ->get('rows');

                        foreach($aChildren as $iKey => $aChild)
                        {
                            $aChildren[$iKey]['time_date'] = date($sDate, $aChild['time_stamp']);

                            if(  $aChild['update_time'] )
                            {
                                $aChildren[$iKey]['update_date'] = date($sDate, $aChild['update_time']);
                            }
                        }

                        $aComments[$iComment]['children'] = array(
                            'content' => $aChildren,
                            'is_link' => ($aChildCnt[$iComment]['total'] > $iLimit ? true : false)
                        );
                    }
                }
            }
        }

        return array($iCnt, $aComments);
    }

    /*
     * Gets Item Data
     *
     * @param string sModel model name
     * @param int iItem item id
     *
     * @return mixed
     */
    public function getItem($sModel, $iItem)
    {
        return $this->getCallback($sModel, $iItem);
    }

    /**
     * The total comments
     *
     * @param string $sModel
     * @return int
     */
    public function getTotal($sModel = '')
    {
        if( $sModel )
        {
            $this->database()->where('model_id = "'.$this->database()->esc($sModel).'"');
        }

        return $this->database()
                ->select('COUNT(*)')
                ->from('comment')
                ->get('column');
    }

    /**
     * Check if comment exists
     *
     * @param string $sType
     * @param int $iItem
     * @param int $iComment
     * @return mixed
     */
    public function is($sType, $iItem, $iComment)
    {
        return $this->database()
                ->select('*')
                ->from('comment')
                ->where('comment_id = '. (int) $iComment .' AND model_id = "'.$this->database()->esc($sType).'" AND item_id = '. (int) $iItem)
                ->get('row');
    }

    /**
     * Checks if the comment is owned by the current user
     *
     * @param int $iComment
     * @return mixed
     */
    public function isUsers($iComment)
    {
        if( !SAA_Sys::isUser() )
        {
            return false;
        }

        return $this->database()
                ->select('*')
                ->from('comment')
                ->where('comment_id = '. (int) $iComment . (SAA_Sys::isAdmin() ? '' : ' AND user_id = ' . SAA_Sys::getUserId()))
                ->get('row');
    }

    /*
     * Preforms all callbacks to other model classes
     */
    public function  __call($sMethod, $aArgs)
    {
        if( $sMethod == 'getCallback' )
        {
            $sMethod = $aArgs[0] . '.callback';
            return SAA_Sys::getModel($sMethod)->getComment($aArgs[1]);
        }
        elseif( $sMethod == 'getDb' )
        {
            $sMethod = $aArgs[0] . '.callback';
            return SAA_Sys::getModel($sMethod)->getCommentDb();
        }
    }
}
?>