<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Comment Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Comment_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Add a comment
     *
     * @param string $sType
     * @param int $iItem
     * @param int iParent parent comment id
     * @param string $sText text
     * @return mixed
     */
    public function add($sType, $iItem, $iParent, $sText)
    {
        $aSql = array(
            'model_id' => $this->parse()->removeTags($sType),
            'item_id' => (int) $iItem,
            'user_id' => SAA_Sys::getUserId(),
            'time_stamp' => SAA_TIME
        );

        if( $iParent )
        {
            if( !($aParent = SAA_Sys::getModel('comment')->is($sType, $iItem, $iParent)) )
            {
                return false;
            }

            $aSql['parent_id'] = (int) $iParent;
        }

        $iId = $this->database()->insert('comment', $aSql);

        if( !$iId )
        {
            return false;
        }

        $aText = array(
            'comment_id' => $iId,
            'text' => $this->parse()->cleanTags($sText, 'comment.allowed_tags')
        );

        $this->database()->insert('comment_text', $aText);

        $aTable = SAA_Sys::getModel('comment')->getDb($sType);

        if( $iParent )
        {
            $this->database()->updateCounter('comment', 'child_total', '+', 1, 'comment_id = '. (int) $iParent);
        }
        else
        {
            $this->database()->updateCounter($aTable['table'], $aTable['column_total'], '+', 1, $aTable['column_unique'] . ' = '. (int) $iItem);
        }

        SAA_Sys::getModel('feed.process')->add('comment_' . $aSql['model_id'], SAA_Sys::getUserId(), $iId, $iParent, serialize(array('parent_id' => $iItem)));

        $aSql['time_date'] = date(SAA_Sys::getSett('comment.date_format'), $aSql['time_stamp']);

        return array_merge($aSql, $aText);
    }

    /**
     * Removes comment and it's children
     *
     * @param int $iComment
     * @param array $aComment
     * @return bool
     */
    public function delete($iComment, $aComment = array())
    {
        if( !$aComment )
        {
            if( !($aComment = SAA_Sys::getModel('comment')->isUsers($iComment)) )
            {
                return false;
            }
        }

        $aTable = SAA_Sys::getModel('comment')->getDb($aComment['model_id']);

        $this->database()->delete('comment', 'comment_id = '. (int) $iComment . ' OR parent_id = '. (int) $iComment .' AND model_id = "comment"');
        $this->database()->delete('comment_text', 'comment_id = '. (int) $iComment);

        $this->database()->updateCounter($aTable['table'], $aTable['column_total'], '-', 1, $aTable['column_unique'] . ' = '. (int) $aComment['item_id']);

        SAA_Sys::getModel('feed')->deleteByParent('comment_' . $aComment['model_id'], $iComment, $iComment);

        return true;
    }

    /**
     * Removes all comments for a given game(s) and updates the total_comments column in the item's table
     * NOTE: will need to run the admin.tools.rebuild.comments
     *
     * @param string $sIds can be in comma format or int
     * @param string $sModel
     */
    public function deleteAll($sIds, $sModel)
    {
        $this->database()->delete('comment', 'model_id = "'.$this->database()->esc($sModel).'" AND (item_id IN('.$sIds.') AND parent_id IN('.$sIds.')');

        $aTable = SAA_Sys::getModel('comment')->getDb($aComment['model_id']);

        $this->database()->update($aTable['table'], array($aTable['column_total'] => 0), $aTable['column_unique'] . ' IN('.$sIds.')');

        return true;
    }

    /**
     * Removes all the user's comments
     *
     * @param int $iUser
     * @param int $iPage
     * @return bool
     */
    public function deleteUser($iUser, $iPage)
    {
        $iOffset = 50;

        $aComments = $this->database()
            ->select('comment_id, parent_id, model_id, item_id')
            ->from('comment')
            ->where('user_id = '. (int) $iUser)
            ->limit($iOffset, $iPage)
            ->get('rows');

        if( !$aComments )
        {
            return false;
        }

        $sIds = '';
        $oComment = SAA_Sys::getModel('comment');

        foreach($aComments as $aComment)
        {
            $sIds .= $aComment['comment_id'] . ', ';

            if( $aComment['parent_id'] )
            {
                continue;
            }

            $aDb = $oComment->getDb($aComment['model_id']);

            $this->database()->updateCounter($aDb['table'], $aDb['column_total'], '-', 1, $aDb['column_unique'] . ' = '. (int) $aComment['item_id']);
        }

        $sIds = rtrim(trim($sIds), ',');

        $this->database()->delete('comment', 'comment_id IN ('. $sIds . ')');
        $this->database()->delete('comment_text', 'comment_id IN ('. $sIds .')');

        return true;
    }

    /**
     * Removes any delete fragments from the $this->deleteAll function
     *
     * @return int
     */
    public function removeMissing()
    {
        $aMissing = $this->database()
                ->select('ct.comment_id')
                ->from('comment_text', 'ct')
                ->leftjoin('comment', 'c', 'ct.comment_id = c.comment_id')
                ->where('c.comment_id IS NULL')
                ->limit(50)
                ->key('comment_id')
                ->get('rows');

        if( $aMissing )
        {
            $sIn = implode(', ', array_keys($aMissing));

            $this->database()->delete('comment_text', 'comment_id IN('.$sIn.')');
        }

        return $this->database()
                ->select('COUNT(*)')
                ->from('comment_text', 'ct')
                ->leftjoin('comment', 'c', 'ct.comment_id = c.comment_id')
                ->where('c.comment_id IS NULL')
                ->get('column');
    }

    /**
     * Reparses the text field for all comments
     *
     * @param int $iPage
     * @param int $iOffset
     */
    public function reparseText($iPage, $iOffset)
    {
        $aRows = $this->database()
            ->select('*')
            ->from('comment_text')
            ->limit($iOffset, $iPage)
            ->get('rows');

        foreach($aRows as $aRow)
        {
            $sText = html_entity_decode($aRow['text']);

            $aSql = array(
                'text' => $this->parse()->cleanTags($sText, 'comment.allowed_tags')
            );

            $this->database()->update('comment_text', $aSql, 'comment_id = '. $aRow['comment_id']);
        }
    }
}
?>