<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Feed Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Feed_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Add a item to the feed
     *
     * @param string $sType
     * @param int $iUser
     * @param int $iItem
     * @param string $sText
     * @param int $iTime
     * @return mixed
     */
    public function add($sType, $iUser, $iItem, $iItemParent = 0, $sText = '', $iTime = null)
    {
        $aSql = array(
            'type_id' => $this->parse()->removeTags($sType),
            'user_id' => (int) $iUser,
            'item_id' => (int) $iItem,
            'parent_item_id' => (int) $iItemParent,
            'content' => ($sText ? $sText : null),
            'time_stamp' => ($iTime ? $iTime : SAA_TIME)
        );

        $iId = $this->database()->insert('feed', $aSql);

        if( !$iId )
        {
            return false;
        }

        return $iId;
    }

    /**
     * Removes feed provided a given feed_id
     *
     * @param int $iId
     * @return bool
     */
    public function delete($iId)
    {
        $this->database()->delete('feed', 'feed_id = '. (int) $iId);

        return true;
    }

    /**
     * Removes all feed for a given set of criteria
     *
     * @param string $sType
     * @param string $iItem
     * @param mixed $mParent
     * @param string $iUser
     *
     * @return bool
     */
    public function deleteBy($sType = '', $iItem  = '', $mParent = '', $iUser = '')
    {
        $sWhere = '';

        if( $sType )
        {
            $sWhere .= 'type_id = "'.$this->database()->esc($sType).'" AND ';
        }

        if( $iItem )
        {
            $sWhere .= 'item_id = '.(int) $iItem .' AND ';
        }

        if( $mParent )
        {
            $sWhere .= 'parent_item_id '. (is_numeric($mParent) ? '= ' . (int) $mParent : 'IN('.$mParent.')') .' AND ';
        }

        if( $iUser )
        {
            $sWhere .= 'user_id = '.(int) $iUser .' AND ';
        }

        $sWhere = rtrim($sWhere, 'AND ');

        $this->database()->delete('feed', $sWhere);

        return true;
    }

    /**
     * Removes feeds by a given parent_id
     *
     * @param string $sType
     * @param mixed $mParent
     * @param int $iItem
     */
    public function deleteByParent($sType, $mParent, $iItem = 0)
    {
        return $this->deleteBy($sType, $iItem, $mParent);
    }

    /**
     * Removes feeds by a given user id and optional type
     * 
     * @param int $iUser
     * @param string $sType
     * @return bool
     */
    public function deleteByUser($iUser, $sType = '')
    {
        return $this->deleteBy($sType, '', '', $iUser);
    }

    /**
     * Removes feeds by a given where
     *
     * @param string $sType
     * @param array $aWhere
     * @param array $mFind
     * @param array $mReplace
     *
     * @return true
     */
    public function deleteByWhere($sType, $aWhere, $mFind = array(), $mReplace = array())
    {
        if( $mFind && $mReplace )
        {
            $aWhere = str_replace($mFind, $mReplace, $aWhere);
        }

        $this->database()->delete('feed', 'type_id = "'.$this->database()->esc($sType).'"' . ($aWhere ? ' AND '. implode(' AND ', $aWhere) : ''));
    }

    /**
     * Removes any feeds with incomplete data
     *
     * @param int $iPage
     */
    public function deleteMissing($iPage)
    {
        $iOffset = 50;

        $aRows = $this->database()
            ->select('*')
            ->from('feed')
            ->order('time_stamp', 'DESC')
            ->limit($iOffset, $iPage)
            ->get('rows');

        if( !$aRows )
        {
            return false;
        }

        $sIn = '';
        $oFeed = SAA_Sys::getModel('feed');

        foreach($aRows as $aFeed)
        {
            $aRow = $aFeed;

            if( !is_null($aFeed['content']) )
            {
                $aRow = array_merge(unserialize($aFeed['content']), $aRow);
            }

            $aRow['text'] = $oFeed->getCallBack($aRow['type_id'], $aRow);

            if( $aRow['text'] )
            {
                continue;
            }

            $sIn .= $aFeed['feed_id'].', ';
        }

        if( $sIn )
        {
            $sIn = rtrim(trim($sIn), ',');

            $this->database()->delete('feed', 'feed_id IN('.$sIn.')');
        }

        return true;
    }
}
?>