<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Achievement Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Achievement extends SAA_Model
{
    /**
     * Gets an achievement for a given game
     *
     * @param int $iGame
     * @param string $sHash
     * @return array
     */
    public function get($iGame, $sHash)
    {
        return $this->database()
            ->select('*')
            ->from('game_award', 'ga')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->where('ga.game_id = '. (int) $iGame .' AND ga.award_hash = "'.$this->database()->esc($sHash).'"')
            ->get('row');
    }

    /**
     * Gets an achievement by id
     * 
     * @param int $iId 
     */
    public function getById($iId)
    {
        $aRow =  $this->database()
            ->select('*')
            ->from('game_award', 'ga')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->where('ga.award_id = '. (int) $iId)
            ->get('row');

        if( !$aRow )
        {
            return false;
        }

        $aRow['full_thumbnail'] = SAA_Sys::getModel('game.link')->expand($aRow['thumbnail'], SAA_APP_GAME_IMAGE_ACHIEVE_URL);

        return $aRow;
    }

    /**
     * Gets the awards for a given game
     *
     * @param int $iGame
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    public function getGameAll($iGame, $iPg, $iPgSize)
    {
        $oLink = SAA_Sys::getModel('game.link');

        $aRows = $this->database()
            ->select('*')
            ->from('game_award', 'ga')
            ->join('game_award_text', 'gat', 'ga.award_id = gat.award_id')
            ->where('ga.game_id = '. (int) $iGame)
            ->limit($iPgSize, $iPg)
            ->order('ga.title')
            ->get('rows');

        foreach($aRows as $i => $aRow)
        {
            $aRows[$i]['thumbnail'] = $oLink->expand($aRow['thumbnail'], SAA_APP_GAME_IMAGE_ACHIEVE_URL);
        }

        $iCnt = $this->database()
            ->select('COUNT(*)')
            ->from('game_award', 'ga')
            ->join('game_award_text', 'gat', 'ga.award_id = gat.award_id')
            ->where('ga.game_id = '. (int) $iGame)
            ->get('column');

        return array($iCnt, $aRows);
    }

    /**
     * Gets an achievement for a given user
     *
     * @param int $iAward
     * @param int $iUser
     * @return array
     */
    public function getUser($iAward, $iUser)
    {
        return $this->database()
            ->select('ua.*')
            ->from('user_award', 'ua')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->where('ua.user_id = '. (int) $iUser .' AND ua.award_id = '. (int) $iAward)
            ->get('row');
    }

    /**
     * Gets all the achievements for a given user provided a set of parameters
     *
     * @param int $iUser
     * @param array $aParam
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    public function getUserAll($iUser, $aParam, $iPg, $iPgSize)
    {
        if( isset($aParam['game_id']) && $aParam['game_id'] > 0 )
        {
            return $this->_getUserForGame($iUser, $aParam, $iPg, $iPgSize);
        }

        $oLink = SAA_Sys::getModel('game.link');
        $iLanguage = SAA_Sys::getLib('language')->getDefault();

        $aGames = $this->database()
            ->select('g.*, lgp.text as game_title')
            ->from('user_award', 'ua')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->join('game', 'g', 'g.game_id = ga.game_id')
            ->join('language_game_phrase', 'lgp', 'lgp.game_id = ga.game_id AND lgp.var_name = "title" AND lgp.language_id = '. (int) $iLanguage)
            ->where('ua.user_id = '. (int) $iUser)
            ->group('ga.game_id')
            ->order('lgp.text')
            ->limit($iPgSize, $iPg)
            ->key('game_id')
            ->get('rows');

        if( !$aGames )
        {
            return array(0, array());
        }

        foreach($aGames as $iGame => $aGame)
        {
            $aRows = $this->database()
                ->select('ga.*, gat.*, ua.date_added as time_awarded')
                ->from('user_award', 'ua')
                ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
                ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
                ->where('ua.user_id = '. (int) $iUser . ' AND ua.game_id = '. (int) $iGame)
                ->limit($iPgSize)
                ->order(array('ua.date_added' => 'DESC', 'ga.title' => 'ASC'))
                ->get('rows');

            foreach($aRows as $i => $aRow)
            {
                $aRows[$i]['thumbnail'] = $oLink->expand($aRow['thumbnail'], SAA_APP_GAME_IMAGE_ACHIEVE_URL);
            }

            $aGames[$iGame]['img_loc'] = $oLink->expand($aGame['img_loc'], SAA_APP_GAME_IMAGE_URL);
            $aGames[$iGame]['awards'] = $aRows;
        }

        $iCnt = $this->database()
            ->select('COUNT(DISTINCT ua.game_id)')
            ->from('user_award', 'ua')
            ->where('ua.user_id = '. (int) $iUser)
            ->get('column');

        return array($iCnt, $aGames);
    }

    /**
     * Get all the users that have achievements for a given game
     * 
     * @param int $iGame
     * @param array $aParam
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    public function getUsersForGame($iGame, $aParam, $iPg, $iPgSize)
    {
        $oLink = SAA_Sys::getModel('game.link');

        $aUsers = $this->database()
            ->select('u.*')
            ->from('user_award', 'ua')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->join('user', 'u', 'u.user_id = ua.user_id')
            ->where('ua.game_id = '. (int) $iGame)
            ->group('ua.user_id')
            ->order('u.user_name')
            ->limit($iPgSize, $iPg)
            ->get('rows');

        if( !$aUsers )
        {
            return array(0, array());
        }

        foreach($aUsers as $i => $aUser)
        {
            $aRows = $this->database()
                ->select('ga.*, gat.*, ua.date_added as time_awarded')
                ->from('user_award', 'ua')
                ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
                ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
                ->where('ua.user_id = '. (int) $aUser['user_id'] . ' AND ua.game_id = '. (int) $iGame)
                ->order(array('ua.date_added' => 'DESC', 'ga.title' => 'ASC'))
                ->get('rows');

            foreach($aRows as $iRow => $aRow)
            {
                $aRows[$iRow]['thumbnail'] = $oLink->expand($aRow['thumbnail'], SAA_APP_GAME_IMAGE_ACHIEVE_URL);
            }

            $aUsers[$i]['achievements'] = $aRows;
        }

        $iCnt = $this->database()
            ->select('COUNT(DISTINCT u.user_id)')
            ->from('user_award', 'ua')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->join('user', 'u', 'u.user_id = ua.user_id')
            ->where('ua.game_id = '. (int) $iGame)
            ->get('column');

        return array($iCnt, $aUsers);
    }

    /**
     * Gets achievements for a given user for a given game provided a set of parameters
     *
     * @param int $iUser
     * @param array $aParam
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    private function _getUserForGame($iUser, $aParam, $iPg, $iPgSize)
    {
        $oLink = SAA_Sys::getModel('game.link');

        $aRows = $this->database()
            ->select('ga.*, gat.*, ua.date_added as time_awarded')
            ->from('user_award', 'ua')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->where('ua.user_id = '. (int) $iUser . ' AND ga.game_id = '. (int) $aParam['game_id'])
            ->limit($iPgSize, $iPg)
            ->order(array('ua.date_added' => 'DESC', 'ga.title' => 'ASC'))
            ->get('rows');

        foreach($aRows as $i => $aRow)
        {
            $aRows[$i]['thumbnail'] = $oLink->expand($aRow['thumbnail'], SAA_APP_GAME_IMAGE_ACHIEVE_URL);
        }

        $iCnt = $this->database()
            ->select('COUNT(*)')
            ->from('user_award', 'ua')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->join('game_award_text', 'gat', 'gat.award_id = ga.award_id')
            ->where('ua.user_id = '. (int) $iUser . ' AND ga.game_id = '. (int) $aParam['game_id'])
            ->get('column');

        return array($iCnt, $aRows);
    }
}
?>
