<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Achievement Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Achievement_Process extends SAA_Model
{
    /**
     * Add an achievement for a given game
     *
     * @param int $iGame
     * @param array $aVals
     * @return mixed
     */
    public function add($iGame, $aVals)
    {
        $iId = $this->database()->insert('game_award', array(
            'game_id' => $iGame,
            'award_hash' => $aVals['achievementID'],
            'type_id' => ($aVals['kind'] == 'ingame' ? 0 : 1),
            'title' => $this->parse()->removeTags($aVals['name']),
            'thumbnail' => SAA_Sys::getModel('game.link')->shrink($aVals['thumbnail'])
        ));

        if( !$iId )
        {
            return false;
        }

        $this->database()->insert('game_award_text', array(
            'award_id' => $iId,
            'award_description' => $this->parse()->cleanTags($aVals['description'], 'global.allowed_tags')
        ));

        return $iId;
    }

    /**
     * Add an achievement for a given user
     *
     * @param array $aAchievement
     * @param int $iUser
     * @return mixed
     */
    public function addUser($aAchievement, $iUser)
    {
        $iId = $this->database()->insert('user_award', array(
            'user_id' => $iUser,
            'game_id' => $aAchievement['game_id'],
            'award_id' => $aAchievement['award_id'],
            'date_added' => SAA_TIME
        ));

        if( !$iId )
        {
            return false;
        }

        $this->database()->updateCounter('user', ($aAchievement['type_id'] ? 'total_medals' : 'total_achievements'), '+', 1, 'user_id = '. (int) $iUser);

        SAA_Sys::getModel('feed.process')->add('game.achievement', $iUser, $iId, $aAchievement['award_id']);

        return $iId;
    }

    /**
     * Removes all awards for a given set of games
     * 
     * @param mixed $mGame
     * @param bool $bSet 
     */
    public function delete($mGame, $bSet = false)
    {
        $oFile = SAA_Sys::getLib('file');

        (!$bSet ? (int) $mGame : null);

        $aRows = $this->database()
                ->select('award_id, thumbnail')
                ->from('game_award')
                ->where('game_id IN ('.$mGame.')')
                ->get('rows');

        $sIn = '';

        foreach($aRows as $aRow)
        {
            $sIn .= $aRow['award_id'] . ',';

            if( $oFile->exists(SAA_APP_GAME_IMAGE_ACHIEVE . $aRow['thumbnail']) )
            {
                $oFile->delete(SAA_APP_GAME_IMAGE_ACHIEVE . $aRow['thumbnail']);
            }
        }

        if( $sIn )
        {
            $this->database()->delete('game_award', 'game_id IN ('.$mGame.')');
            $this->database()->delete('game_award_text', 'game_id IN ('.rtrim($sIn, ',').')');
        }

        return true;
    }

    /**
     * Delete achievements for a given page
     * 
     * @param int $iGame
     * @param int $iPage 
     * @param return bool
     */
    public function deleteGame($iGame, $iPage)
    {
        $iOffset = 50;

        $aAwards = $this->database()
            ->select('award_id, thumbnail')
            ->from('game_award')
            ->where('game_id = '. (int) $iGame)
            ->limit($iOffset, $iPage)
            ->get('rows');

        if( !$aAwards )
        {
            return false;
        }

        $sIn = '';

        $oFile = SAA_Sys::getLib('file');

        foreach($aAwards as $aAward)
        {
            $sIn .= $aAward['award_id'].', ';

            if( $oFile->exists(SAA_APP_GAME_IMAGE_ACHIEVE . $aAward['thumbnail']) )
            {
                $oFile->delete(SAA_APP_GAME_IMAGE_ACHIEVE . $aAward['thumbnail']);
            }
        }

        $sIn = rtrim(trim($sIn), ',');

        $this->database()->delete('game_award', 'award_id IN('.$sIn.')');
        $this->database()->delete('game_award_text', 'award_id IN('.$sIn.')');

        return true;
    }

    /**
     * Delete users' achievement for a given game and updates the user's counter
     * 
     * @param int $iGame
     * @param int $iPage
     * @return bool
     */
    public function deleteGameUser($iGame, $iPage)
    {
        $iOffset = 50;

        $aAwards = $this->database()
            ->select('COUNT(*) as minus, ua.user_id, ga.type_id')
            ->from('user_award', 'ua')
            ->join('user', 'u', 'u.user_id = ua.user_id')
            ->join('game_award', 'ga', 'ga.award_id = ua.award_id')
            ->where('ua.game_id = '. (int) $iGame)
            ->group('ua.user_id, ga.type_id')
            ->limit($iOffset, $iPage)
            ->get('rows');

        if( !$aAwards )
        {
            $this->database()->delete('user_award', 'game_id = '. (int) $iGame);

            return false;
        }

        foreach($aAwards as $aAward)
        {
            $this->database()->updateCounter('user', ($aAward['type_id'] ? 'total_medals' : 'total_achievements'), '+', $aAward['minus'], 'user_id = '. (int) $aAward['user_id']);
        }

        return true;
    }

    /**
     * Removes awards for all users from user_award
     *
     * @param mixed $mGame
     * @param bool $bSet
     */
    public function deleteForUsers($mGame = false, $bSet = false)
    {
        if( $mGame )
        {
            $sWhere = ($bSet ? 'IN ('.$mGame.')' : '= '. (int) $mGame);

            $this->database()->delete('user_award', 'game_id '. $sWhere);
        }
        else
        {
            $this->database()->truncateTable('user_award');
        }

        return true;
    }

    /**
     * Updates the award's text
     *
     * @param int $iId
     * @param string $sTitle
     * @param string $sDesc
     * @return true
     */
    public function edit($iId, $sTitle, $sDesc)
    {
        $this->database()->update('game_award', array('title' => $this->parse()->rareToEntities($sTitle)), 'award_id = '. (int) $iId);
        $this->database()->update('game_award_text', array('award_description' => $this->parse()->rareToEntities($sDesc)), 'award_id = '. (int) $iId);

        return true;
    }

    /**
     * Rebuilds the 'total_achievements' in the user table
     *
     * @param int $iPg
     * @param int $iPgSize
     * @return bool
     */
    public function recountUsers($iPg, $iPgSize)
    {
        $aUsers = $this->database()
            ->select('user_id')
            ->from('user')
            ->limit($iPgSize, $iPg)
            ->get('rows');

        if( !$aUsers )
        {
            return false;
        }

        foreach($aUsers as $aUser)
        {
            $iAwards = $this->database()
                    ->select('COUNT(*)')
                    ->from('user_award')
                    ->where('user_id = '. (int) $aUser['user_id'])
                    ->get('column');

            $this->database()->update('user', array('total_achievements' => $iAwards), 'user_id = '. (int) $aUser['user_id']);
        }

        return true;
    }

    /**
     * Enables or disables an award
     *
     * @param int $iId
     * @param int $iToggle
     * @return true
     */
    public function toggle($iId, $iToggle)
    {
        $this->database()->update('game_award', array('is_active' => (int) $iToggle), 'award_id = '. (int) $iId);

        return true;
    }

    public function toggleForUser($iAward, $iUser, $iToggle)
    {
        $this->database()->update('user_award', array('is_active' => (int) $iToggle), 'user_id = '.(int) $iUser .' AND award_id = '. (int) $iAward);

        return true;
    }

    /**
     * Updates the thumbnail for the given award
     *
     * @param int $iId
     * @param string $sFile
     * @return true
     */
    public function updateImage($iId, $sFile)
    {
        $this->database()->update('game_award', array('thumbnail' => $sFile), 'award_id = '. (int) $iId);

        return true;
    }
}
?>
