<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Adder Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Adder extends SAA_Model
{
    private $_iAdded = 0;

    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Checks in the
     *
     * @param string $aRaw
     * @return bool
     */
    public function alreadyExists($aGame)
    {
        $iGame = $this->database()
            ->select('game_id')
            ->from('game_extra')
            ->where('source_id = "'.$this->database()->esc($aGame['extra']['source_id']).'" AND source_type = '. (int) $aGame['extra']['source_type'])
            ->get('column');
        return (bool) $iGame;
    }

    /**
     * Gets a single game
     *
     * @param string $sId
     * @param string $sType
     * @return bool
     */
    public function get($sId, $sType)
    {
        $oAdder = SAA_Sys::getModel('game.adder.' . $sType);
        $oFile = SAA_Sys::getLib('file');
        $oLangGameProcess = SAA_Sys::getModel('language.game.process');
        $oTagProcess = SAA_Sys::getModel('game.tag.process');
        $oLink = SAA_Sys::getModel('game.link');

        $aLanguages = SAA_Sys::getLib('language')->getLanguages();
        $aLanguages = array_keys($aLanguages);        

        $aGame = $oAdder->importSingle($sId);

        if( !is_array($aGame) )
        {
            return false;
        }

        $bFiles = $oAdder->importFiles();
        $iGame = SAA_Sys::getModel('game.stats')->largestGameId() + 1;

        $sDir = $oFile->buildDir($iGame);

        $sSwfUrl = $oAdder->getSwf($aGame);
        $sImgUrl = $oAdder->getImg($aGame);

        $aGame = $oAdder->get($aGame);

        $sSwfUrl = $this->prependLink($sSwfUrl);
        $sImgUrl = $this->prependLink($sImgUrl);

        $sImgExt = $oFile->getExt($sImgUrl);

        //If get files is enabled, download files from game server
        if( $bFiles )
        {
            $sSwfFile = $oFile->buildName(SAA_APP_GAME_SWF . $sDir, 'swf');
            $sImgFile = $oFile->buildName(SAA_APP_GAME_IMAGE . $sDir, $sImgExt);

            if( !$oFile->downloadRemote($sSwfUrl, SAA_APP_GAME_SWF . $sDir . $sSwfFile) )
            {
                return;
            }

            if( $oFile->downloadRemote($sImgUrl, SAA_APP_GAME_IMAGE . $sDir . $sImgFile) )
            {
                $aGame['game']['img_loc'] = $sDir . $sImgFile;
                $aGame['extra']['image_size'] = filesize(SAA_APP_GAME_IMAGE . $sDir . $sImgFile);
            }

            $aGame['game']['file_loc'] = $sDir . $sSwfFile;
            $aGame['extra']['file_size'] = filesize(SAA_APP_GAME_SWF . $sDir . $sSwfFile);
        }
        //If get files is disabled, prepare links
        else
        {
            $aGame['game']['file_loc'] = $oLink->shrink($sSwfUrl);
            $aGame['game']['img_loc'] = $oLink->shrink($sImgUrl);
        }

        //Insert Game
        $iGame = $this->database()->insert('game', $aGame['game']);

        //Insert Failure
        if( !$iGame )
        {
            if( $bFiles )
            {
                if( isset($aGame['game']['img_loc']) )
                {
                    $oFile->delete(SAA_APP_GAME_IMAGE . $sDir . $aGame['game']['img_loc']);
                }

                if( isset($aGame['game']['file_loc']) )
                {
                    $oFile->delete(SAA_APP_GAME_SWF . $sDir . $aGame['game']['file_loc']);
                }
            }

            return;
        }

        //Set Game Id
        $aGame['extra']['game_id'] = $iGame;

        //Insert Game Category
        if( isset($aGame['category']) )
        {
            foreach($aGame['category'] as $aCategory)
            {
                $aSql = array_merge(array('game_id' => $iGame), $aCategory);

                $this->database()->insert('game_category_map', $aSql);
            }
        }
        //Insert Game Extra
        $this->database()->insert('game_extra', $aGame['extra']);

        //Insert Game Scoring
        if( isset($aGame['scoring']) )
        {
            if( isset($aGame['scoring']['scoring_type']) )
            {
                $aScoring = array($aGame['scoring']);
            }
            else
            {
                $aScoring = $aGame['scoring'];
            }

            SAA_Sys::getModel('game.scoring.process')->add($aScoring, $iGame);
        }

        //Insert Game Tags
        $oTagProcess->add($aGame['tag'], $iGame);

        //Insert Game Text
        foreach($aGame['text'] as $sVarName => $sText)
        {
            $aVal = array(
                'game_id' => $iGame,
                'var_name' => $sVarName,
                'text' => array_fill_keys($aLanguages, $sText)
            );

            $oLangGameProcess->add($aVal);
        }

        return true;
    }

    /**
     * Adds multiple games
     *
     * @param string $sType
     * @param array $aParam
     * @param int $iAdded
     * @param int $iOffset
     * @param int $iLimit
     * @return mixed
     */
    public function getGames($sType, $aParam, $iAdded, $iOffset, $iLimit)
    {
        $oAdder = SAA_Sys::getModel('game.adder.' . $sType);
        $oFile = SAA_Sys::getLib('file');
        $oLangGameProcess = SAA_Sys::getModel('language.game.process');
        $oLink = SAA_Sys::getModel('game.link');
        $oTagProcess = SAA_Sys::getModel('game.tag.process');

        $aLanguages = SAA_Sys::getLib('language')->getLanguages();
        $aLanguages = array_keys($aLanguages);

        $mGames = $oAdder->import($aParam, $iOffset, $iLimit);

        if( !is_array($mGames) )
        {
            return $mGames;
        }

        $iMaxGames = $oAdder->getMax();
        $bFiles = $oAdder->importFiles();

        foreach($mGames as $aGame)
        {
            $sSwfUrl = $oAdder->getSwf($aGame);
            $sImgUrl = $oAdder->getImg($aGame);

            $aGame = $oAdder->get($aGame);

            if( $this->alreadyExists($aGame) )
            {
                continue;
            }

            $sDir = $oFile->buildDir(($this->_iAdded+$iAdded));

            $sSwfUrl = $this->prependLink($sSwfUrl);
            $sImgUrl = $this->prependLink($sImgUrl);

            $sImgExt = $oFile->getExt($sImgUrl);

            //If get files is enabled, download files from game server
            if( $bFiles )
            {
                $sSwfFile = $oFile->buildName(SAA_APP_GAME_SWF . $sDir, 'swf');
                $sImgFile = $oFile->buildName(SAA_APP_GAME_IMAGE . $sDir, $sImgExt);

                if( !$oFile->downloadRemote($sSwfUrl, SAA_APP_GAME_SWF . $sDir . $sSwfFile) )
                {
                    continue;
                }

                if( $oFile->downloadRemote($sImgUrl, SAA_APP_GAME_IMAGE . $sDir . $sImgFile) )
                {
                    $aGame['game']['img_loc'] = $sDir . $sImgFile;
                    $aGame['extra']['image_size'] = filesize(SAA_APP_GAME_IMAGE . $sDir . $sImgFile);
                }

                $aGame['game']['file_loc'] = $sDir . $sSwfFile;
                $aGame['extra']['file_size'] = filesize(SAA_APP_GAME_SWF . $sDir . $sSwfFile);
            }
            //If get files is disabled, prepare links
            else
            {
                $aGame['game']['file_loc'] = $oLink->shrink($sSwfUrl);
                $aGame['game']['img_loc'] = $oLink->shrink($sImgUrl);
            }

            //Insert Game
            $iGame = $this->database()->insert('game', $aGame['game']);

            //Insert Failure
            if( !$iGame )
            {
                if( $bFiles )
                {
                    if( isset($aGame['game']['img_loc']) )
                    {
                        $oFile->delete(SAA_APP_GAME_IMAGE . $sDir . $aGame['game']['img_loc']);
                    }

                    if( isset($aGame['game']['file_loc']) )
                    {
                        $oFile->delete(SAA_APP_GAME_SWF . $sDir . $aGame['game']['file_loc']);
                    }
                }

                continue;
            }

            //Set Game Id
            $aGame['extra']['game_id'] = $iGame;

            //Insert Game Category
            if( isset($aGame['category']) )
            {
                foreach($aGame['category'] as $aCategory)
                {
                    $aSql = array_merge(array('game_id' => $iGame), $aCategory);

                    $this->database()->insert('game_category_map', $aSql);
                }
            }
            //Insert Game Extra
            $this->database()->insert('game_extra', $aGame['extra']);

            //Insert Game Scoring
            if( isset($aGame['scoring']) )
            {
                $aScoring = $aGame['scoring'];

                SAA_Sys::getModel('game.scoring.process')->add($aScoring, $iGame);
            }

            //Insert Game Tags
            $oTagProcess->add($aGame['tag'], $iGame);

            //Insert Game Text
            foreach($aGame['text'] as $sVarName => $sText)
            {
                $aVal = array(
                    'game_id' => $iGame,
                    'var_name' => $sVarName,
                    'text' => array_fill_keys($aLanguages, $sText)
                );

                $oLangGameProcess->add($aVal);
            }

            $this->_iAdded++;

            if( $iMaxGames > 0 && $this->_iAdded + $iAdded >= $iMaxGames )
            {
                return 'complete';
            }
        }

        return 'continue';
    }

    /**
     * Returns the number of games added
     *
     * @return int
     */
    public function getAdded()
    {
        return $this->_iAdded;
    }

    /**
     * Prepends a url link
     *
     * @param string $sUrl
     * @return string
     */
    public function prependLink($sUrl)
    {
        return (strpos($sUrl, 'http://') === false ? 'http://' : '') . $sUrl;
    }
}
?>
