<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Callback Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Callback extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /*
     * Return info relating to the item:
     *  1. title
     *  2. thumb
     *  3. total comments
     *  4. link
     */
    public function getComment($iItem)
    {
        $aGame = SAA_Sys::getModel('game')->get($iItem, false);

        if( !$aGame )
        {
            return false;
        }

        return array(
            'title' => $aGame['text']['title'],
            'description' => $aGame['text']['description'],
            'thumbnail' => $aGame['img_loc'],
            'total' => $aGame['total_comments'],
            'url' => SAA_Sys::getLib('url')->makeSimple('game.'.$aGame['slug'])
        );
    }

    /**
     * For SAA_Sys::getModel('comment')->getDb function
     */
    public function getCommentDb()
    {
       return array(
            'table' => 'game',
            'column_unique' => 'game_id',
            'column_total' => 'total_comments'
       );
    }

    /**
     * Gets data for the feed comments
     *
     * @param array $aItem
     * @return mixed
     */
    public function getCommentFeed($aItem)
    {
        $aGame = SAA_Sys::getModel('game')->get($aItem['parent_id'], false);

        return array(
            'title' => $aGame['text']['title'],
            'description' => $aGame['text']['description'],
            'thumbnail' => $aGame['img_loc'],
            'url' => SAA_Sys::getLib('url')->makeSimple('game.'.$aGame['slug'])
        );
    }

    /*
     * For SAA_Sys::getModel('rating')->getDb function
     */
    public function getRatingDb()
    {
        return array(
            'table' => 'game',
            'table_unique' => 'game_id',
            'table_item' => 'game_rate'
        );
    }

    /*
     * For rating ajax messages
     */
    public function getRatingAjax()
    {
        return array(
            'error-disabled' => SAA_Sys::getPhrase('game.ratings_have_been_disabled'),
            'error-login' => SAA_Sys::getPhrase('game.you_must_be_logged_in_to_rate_this_game'),
            'error-other' => SAA_Sys::getPhrase('game.there_was_an_error_in_rating_the_game_please_refresh_the_page_and_try_again')
        );
    }

    /*
     * Return info relating to the item:
     *  1. title
     *  2. edit link
     *  3. view link
     */
    public function getReport($iItem)
    {
        $aGame = SAA_Sys::getModel('game')->get($iItem, false);

        if( !$aGame )
        {
            return false;
        }

        return array(
            'title' => $aGame['text']['title'],
            'edit_url' => SAA_Sys::getLib('url')->makeSimple(array(
                'controller' => 'admin.game.edit',
                'vars' => array('id' => $aGame['game_id'])
            )),
            'view_url' => SAA_Sys::getLib('url')->makeSimple('game.'.$aGame['slug'])
        );
    }

    /**
     * Determines if ratings are enabled for games
     */
    public function isRatingEnabled()
    {
        return SAA_Sys::getSett('rating.on_games');
    }
}
?>
