<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Category Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Category extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Get categories for a given game from cache table (game_category_cache)
     *
     * @param int $iGame
     * @return array
     */
    public function get($iGame)
    {
        $aCategories = array();

        $aRows = $this->database()
            ->select('category_id')
            ->from('game_category_cache')
            ->where('game_id = ' . (int) $iGame)
            ->get('rows');

        if( !$aRows )
        {
            $aRows = SAA_Sys::getModel('game.category.process')->rebuildSingle($iGame);
        }

        foreach($aRows as $aRow)
        {
            $aCategories[] = $aRow['category_id'];
        }

        return $aCategories;
    }

    /**
     * Get categories for an array of games from cache table (game_category_cache)
     * 
     * @param array $aGames 
     * @return array
     */
    public function getAll($aGames)
    {
        $oCategory = SAA_Sys::getLib('category');

        $aIds = array();
        $sIn = '';
        $iMaxCat = SAA_Sys::getSett('game.category_limit');

        foreach($aGames as $i=>$aGame)
        {
            $aGames[$i]['category'] = array();

            $aIds[$aGame['game_id']] = $i;

            $sIn .= (int) $aGame['game_id'] . ', ';
        }

        $sIn = rtrim(trim($sIn), ',');

        $aRows = $this->database()
            ->select('*')
            ->from('game_category_cache')
            ->where('game_id IN('.$sIn.')')
            ->order('RAND()')
            ->get('rows');

        foreach($aRows as $aRow)
        {
            $i = $aIds[$aRow['game_id']];

            if( !($aCategory = $oCategory->get($aRow['category_id'])) || isset($aGames[$i]['category']) && $iMaxCat > 0 && count($aGames[$i]['category']) >= $iMaxCat )
            {
                continue;
            }

            $aGames[$i]['category'][] = $aCategory;
        }

        return $aGames;
    }

    /**
     * Get categories for a given game from source table (game_category)
     *
     * @param int $iGame
     * @return array
     */
    public function getManual($iGame)
    {
        $aCategories = array();

        $aRows = $this->database()
            ->select('*')
            ->from('game_category')
            ->where('game_id = '. (int) $iGame)
            ->get('rows');

        foreach($aRows as $aRow)
        {
            $aCategories[$aRow['category_id']] = $aRow['is_excluded'];
        }

        return $aCategories;
    }

    /**
     * Get the category maps for a given game
     *
     * @param int $iGame
     * @return array
     */
    public function getMap($iGame)
    {
        $aMaps = array();

        $aRows = $this->database()
            ->select('*')
            ->from('game_category_map')
            ->where('game_id = '. (int) $iGame)
            ->get('rows');

        foreach($aRows as $aRow)
        {
            $aMaps[] = $aRow['map_id'];
        }

        return $aMaps;
    }
}
?>
