<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Category Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Category_Process extends SAA_Model
{
    private $_aCategories = array();
    private $_aCols = array(
        'game_id',
        'category_id'
    );

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $this->_aCategories = SAA_Sys::getModel('category.map')->getKeys(true);
    }

    /**
     * Clear game category cache
     */
    public function clearTable()
    {
        $this->database()->truncateTable('game_category_cache');
    }

    /**
     * Rebuild game category cache
     *
     * @param int $iPage
     * @param int $iLimit
     * @return array
     */
    public function rebuild($iPage, $iLimit)
    {
        $iCnt = $this->database()
            ->select('COUNT(*)')
            ->from('game')
            ->get('column');

        $aGames = $this->database()
            ->select('game_id, category_manual')
            ->from('game')
            ->limit($iLimit, $iPage)
            ->key('game_id')
            ->get('rows');

        $sIds = '';

        foreach($aGames as $iGame => $aRow)
        {
            $sIds .= $iGame.',';
        }

        $sIds = rtrim(trim($sIds), ',');

        if( !$sIds )
        {
            return array($iCnt, 0);
        }

        $aMaps = $this->database()
            ->select('game_id, map_id')
            ->from('game_category_map')
            ->where('game_id IN('. $sIds.')')
            ->get('rows');

        foreach($aMaps as $aMap)
        {
            if( $aGames[$aMap['game_id']]['category_manual'] )
            {
                continue;
            }

            $iMap = $aMap['map_id'];

            if( !isset($this->_aCategories[$iMap]) )
            {
                continue;
            }

            foreach($this->_aCategories[$iMap] as $iCategory)
            {
                $aGames[$aMap['game_id']]['category'][$iCategory] = 1;
            }
        }

        $aManuals = $this->database()
            ->select('*')
            ->from('game_category')
            ->where('game_id IN('. $sIds.')')
            ->get('rows');

        foreach($aManuals as $aRow)
        {
            if( $aGames[$aRow['game_id']]['category_manual'] )
            {
                $aGames[$aRow['game_id']]['category'][$aRow['category_id']] = 1;

                continue;
            }

            if( $aRow['is_excluded'] )
            {
                if( isset($aGames[$aRow['game_id']], $aGames[$aRow['game_id']]['category'], $aGames[$aRow['game_id']]['category'][$aRow['category_id']]) )
                {
                    unset($aGames[$aRow['game_id']]['category'][$aRow['category_id']]);
                }
                else
                {
                    $this->delete('game_category', 'game_id = '. (int) $aRow['game_id'] .' AND category_id = '. (int) $aRow['category_id']);
                }
            }
            else
            {
                $aGames[$aRow['game_id']]['category'][$aRow['category_id']] = 1;
            }
        }

        $aSql = array();

        foreach($aGames as $iGame => $aGame)
        {
            if( !isset($aGame['category']) )
            {
                continue;
            }

            foreach($aGame['category'] as $iCategory => $iBlank)
            {
                $aSql[] = array(
                    $iGame,
                    $iCategory
                );
            }
        }

        if( count($aSql) )
        {
            $this->database()->insertMultiple('game_category_cache', $this->_aCols, $aSql);
        }

        return array($iCnt, count($aGames));
    }

    /**
     * Rebuild a game's category cache
     *
     * @param int $iGame
     * @return array
     */
    public function rebuildSingle($iGame)
    {
        $this->database()->delete('game_category_cache', 'game_id = '. (int) $iGame);

        $bManual = $this->database()
            ->select('category_manual')
            ->from('game')
            ->where('game_id = '. (int) $iGame)
            ->get('column');

        if( $bManual )
        {
            return $this->_manualSingle($iGame);
        }

        return $this->_traditionalSingle($iGame);
    }

    /**
     * Adds to category cache from category manual table
     *
     * @param int $iGame
     * @return array
     */
    private function _manualSingle($iGame)
    {
        $aCategories = SAA_Sys::getModel('game.category')->getManual($iGame);

        $aSql = array();
        $aRows = array();

        foreach($aCategories as $iCategory => $iBlank)
        {
            $aRow = array(
                $iGame,
                $iCategory
            );

            $aRows[] = array_combine($this->_aCols, $aRow);
            $aSql[] = $aRow;
        }

        if( count($aSql) )
        {
            $this->database()->insertMultiple('game_category_cache', $this->_aCols, $aSql);
        }

        return $aRows;
    }

    /**
     * Rebuild the category cache for a given game
     *
     * @param int $iGame
     * @return array
     */
    private function _traditionalSingle($iGame)
    {
        $oGameCategory = SAA_Sys::getModel('game.category');

        $aIds = array();

        $aMaps = $oGameCategory->getMap($iGame);
        $aManual = $oGameCategory->getManual($iGame);

        foreach($aMaps as $iMap)
        {
            if( !isset($this->_aCategories[$iMap]) )
            {
                continue;
            }

            foreach($this->_aCategories[$iMap] as $iCategory)
            {
                $aIds[$iCategory] = 1;
            }
        }

        foreach($aManual as $iCategory => $bExclude)
        {
            if( $bExclude && isset($aIds[$iCategory]) )
            {
                unset($aIds[$iCategory]);
            }
            elseif( !$bExclude )
            {
                if( isset($aIds[$iCategory]) )
                {
                    $this->delete('game_category', 'game_id = '. (int) $iGame .' AND category_id = '. (int) $iCategory);
                }

                $aIds[$iCategory] = 1;
            }
        }

        $aSql = array();
        $aRows = array();

        foreach($aIds as $iCategory => $iBlank)
        {
            $aRow = array(
                $iGame,
                $iCategory
            );

            $aRows[] = array_combine($this->_aCols, $aRow);
            $aSql[] = $aRow;
        }

        if( count($aSql) )
        {
            $this->database()->insertMultiple('game_category_cache', $this->_aCols, $aSql);
        }

        return $aRows;
    }
}
?>
