<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Gets a game by id or url slug
     *
     * @param mixed $mId
     * @param bool $bBySlug
     * @return array
     */
    public function get($mId, $bBySlug = true)
    {
        $oUser = SAA_Sys::getModel('user');
        $iLanguage = SAA_Sys::getLib('language')->getDefault();
        $oLink = SAA_Sys::getModel('game.link');

        if( $bBySlug )
        {
            $this->database()->where('g.slug = "'. $this->database()->esc($mId) . '"');
        }
        else
        {
            $this->database()->where('g.game_id = '. (int) $mId);
        }

        if( SAA_Sys::isUser() )
        {
            if( SAA_Sys::getSett('rating.can_rate') )
            {
                SAA_Sys::getModel('rating')->getRowQuery('game_rate', 'r.user_id = '. SAA_Sys::getUserId() .' AND r.item_id = g.game_id');
            }

            $this->database()
                ->select('gfav.item_id as is_favorite, su.user_score, su.time_scored as user_time_score,')
                ->leftjoin('game_favorite', 'gfav', 'gfav.game_id = g.game_id')
                ->leftjoin('score_user', 'su', 'su.game_id = g.game_id AND su.user_id = '. SAA_Sys::getUserId());
        }
        else
        {
            if( SAA_Sys::getModel('rating')->isEnabled('game') )
            {
                $this->database()->select('0 as rate_id,');
            }
        }

        $aGame = $this->database()
                ->select('g.*, gf.feature_id, sh.high_score, sh.time_scored as high_score_time, '. $oUser->getUserFields('shu', 'high'))
                ->from('game', 'g')
                ->leftjoin('score_high', 'sh', 'sh.game_id = g.game_id')
                ->leftjoin('user', 'shu', 'shu.user_id = sh.user_id')
                ->leftjoin('game_featured', 'gf', 'gf.game_id = g.game_id')
                ->get('row');

        if( !$aGame )
        {
            return false;
        }

        if( $aGame['scoring'] )
        {
            $aGame['scoring_info'] = SAA_Sys::getModel('game.scoring')->get($aGame['game_id']);
        }

        $aGame['is_featured'] = (bool) $aGame['feature_id'];

        $aGame['tags'] = SAA_Sys::getModel('game.tag')->get($aGame['game_id']);

        if( $aGame['tags'] )
        {
            $aGame['tags_text'] = implode(', ', $aGame['tags']);
        }

        $aGame['total_rating'] = (int) $aGame['total_rating'];
        $aGame['categories'] = SAA_Sys::getModel('game.category')->get($aGame['game_id']);

        $aGame['file_loc'] = $oLink->expand($aGame['file_loc'], SAA_APP_GAME_SWF_URL);
        $aGame['img_loc'] = $oLink->expand($aGame['img_loc'], SAA_APP_GAME_IMAGE_URL);

        $aGame['text'] = SAA_Sys::getModel('language.game')->getText($aGame['game_id'], $iLanguage);

        return $aGame;
    }

    /**
     * Gets a game by the scoring hash and game type
     *
     * @param string $sHash
     * @param int $iType
     * @return array
     */
    public function getByHash($sHash, $iType)
    {
        return $this->database()
                ->select('g.*')
                ->from('game_scoring', 'gs')
                ->join('game', 'g', 'g.game_id = gs.game_id')
                ->where('gs.scoring_hash = "'.$this->database()->esc($sHash).'" AND gs.scoring_type = ' . (int) $iType)
                ->get('row');
    }

    /**
     * Gets a game for admin page
     *
     * @param mixed $mId
     * @param bool $bBySlug
     * @return array
     */
    public function getForAdmin($mId, $bBySlug = true)
    {
        $oGameCategory = SAA_Sys::getModel('game.category');

        if( $bBySlug )
        {
            $this->database()->where('g.slug = "'. $this->database()->esc($mId) . '"');
        }
        else
        {
            $this->database()->where('g.game_id = '. (int) $mId);
        }

        $aGame = $this->database()
                ->select('g.*, ge.*, gf.feature_id')
                ->from('game', 'g')
                ->join('game_extra', 'ge', 'ge.game_id = g.game_id')
                ->leftjoin('game_featured', 'gf', 'gf.game_id = g.game_id')
                ->get('row');

        if( !$aGame )
        {
            return false;
        }

        if( isset($aGame['scoring']) )
        {
            $aGame['scoring_info'] = SAA_Sys::getModel('game.scoring')->get($aGame['game_id']);
        }

        $aGame['is_featured'] = (bool) $aGame['feature_id'];

        $aGame['tags'] = SAA_Sys::getModel('game.tag')->get($aGame['game_id']);

        $aGame['tags_text'] = ($aGame['tags'] ? implode(', ', $aGame['tags']) : '');

        $aGame['text'] = SAA_Sys::getModel('language.game')->getText($aGame['game_id']);

        $aGame['categories'] = $oGameCategory->get($aGame['game_id']);
        $aGame['maps'] = $oGameCategory->getMap($aGame['game_id']);

        return $aGame;
    }

    /**
     * Gets all featured games for admin.game.featured
     *
     * @return array
     */
    public function getFeaturedForList()
    {
        $oLanguageGame = SAA_Sys::getModel('language.game');
        $iLanguage = SAA_Sys::getLib('language')->getDefault();

        $aList = array();

        $aGames = $this->database()
            ->select('g.*')
            ->from('game', 'g')
            ->join('game_featured', 'gf', 'gf.game_id = g.game_id')
            ->order('gf.ordering')
            ->get('rows');

        $i = 0;
        $iKey = 0;

        foreach($aGames as $aGame)
        {
            if( $i++ % 10 == 0 )
            {
                $iKey++;
            }

            $aGame['text'] = $oLanguageGame->getText($aGame['game_id'], $iLanguage);

            $aList[$iKey][] = $aGame;
        }

        return $aList;
    }

    /**
     * Gets a set of games by id or source id using a given set of params
     *
     * @param array $aGames
     * @param bool $bById
     * @param array $aParam
     * @return array
     */
    public function getGames($aGames, $bById = true, $aParam = array())
    {
        $sIn = '';

        if( $bById )
        {
            foreach($aGames as $iGame)
            {
                $sIn .= (int) $iGame . ', ';
            }

            $sIn = rtrim(trim($sIn), ',');

            $this->database()->where('g.game_id IN('.$sIn.')');

            if( isset($aParam['extra']) )
            {
                $this->database()->select('ge.*, ')->join('game_extra', 'ge', 'ge.game_id = g.game_id');
            }
        }
        else
        {
            foreach($aGames as $sGame)
            {
                $sIn .= '"'.$this->database()->esc($sGame). '", ';
            }

            $sIn = rtrim(trim($sIn), ',');

            if( isset($aParam['extra']) )
            {
                $this->database()->select('ge.*, ');
            }

            $this->database()
                    ->join('game_extra', 'ge', 'ge.game_id = g.game_id')
                    ->where('ge.source_id IN('.$sIn.')');
        }

        if( isset($aParam['extra']) )
        {
            $this->database()->select('ge.*, ')->join('game_extra', 'ge', 'ge.game_id = g.game_id');
        }

        if( isset($aParam['featured']) )
        {
            $sAction = (is_string($aParam['featured']) ? $aParam['featured'] : 'join');

            $this->database()->select('gf.*, ')->$sAction('game_featured', 'gf', 'gf.game_id = g.game_id');
        }

        $aRows = $this->database()
            ->select('g.*')
            ->from('game', 'g')
            ->get('rows');

        if( isset($aParam['scoring']) )
        {
            $aRows = SAA_Sys::getModel('game.scoring')->getForMultiple($aRows);
        }

        if( isset($aParam['tag']) )
        {
            $aRows = SAA_Sys::getModel('game.tag')->getForMultiple($aRows);
        }

        if( isset($aParam['text']) )
        {
            $iLanguage = SAA_Sys::getLib('language')->getDefault();

            $aRows = SAA_Sys::getModel('language.game')->getTexts($aRows, $iLanguage);
        }

        return $aRows;
    }

    /**
     * Get the simple data for a given game
     *
     * @param int $iGame
     * @return array
     */
    public function getSimple($iGame)
    {
        return $this->database()
                ->select('*')
                ->from('game')
                ->where('game_id = '. (int) $iGame)
                ->get('row');
    }

    /**
     * Checks if a given slug is unique
     *
     * @param string $sSlug
     * @return bool
     */
    public function isUniqueSlug($sSlug)
    {
        $bExists = $this->database()
            ->select('game_id')
            ->from('game')
            ->where('slug = "'.$this->database()->esc($sSlug).'"')
            ->get('column');

        $bExists = (bool) $bExists;

        return (!$bExists);
    }

    /**
     * Gets the set of games that are the most played
     *
     * @param int $iLimit
     * @return array
     */
    public function timesPlayed($iLimit)
    {
        $aGames = $this->database()
            ->select('*')
            ->from('game')
            ->order(array('total_plays' => 'DESC'))
            ->limit($iLimit)
            ->get('rows');

        if( !$aGames )
        {
            return false;
        }

        $iLanguage = SAA_Sys::getLib('language')->getDefault();

        $aGames = SAA_Sys::getModel('language.game')->getTexts($aGames, $iLanguage);

        return $aGames;
    }
}
?>