<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Link Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Link extends SAA_Model
{
    private $_aLinks = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $sCache = 'game_link_id';

        if( !($this->_aLinks = $this->cache()->get($sCache)) )
        {
            $this->_aLinks = $this->getAll();

            $this->cache()->create($sCache, $this->_aLinks);
        }
    }

    /**
     * Add link to list of links
     *
     * @param int $iId
     * @param string $sVal
     * @return bool
     */
    public function append($iId, $sVal)
    {
        $this->_aLinks[$iId] = $sVal;

        return true;
    }

    /**
     * Converts a link into its bigger footprint
     *
     * @param string $sFile
     * @param string $sFolder
     * @return string
     */
    public function expand($sFile, $sFolder)
    {
        $bLocal = true;

        foreach($this->_aLinks as $iLink => $sLink)
        {
            $sFile = str_replace('{'.$iLink.'}', $sLink, $sFile, $iReplaced);

            if( $iReplaced )
            {
                $bLocal = false;
            }
        }

        if( $bLocal )
        {
            $sFile = SAA_Sys::getLib('url')->makeUrl(array(
                'file' => $sFile,
                'folder' => $sFolder
            ));
        }

        return $sFile;
    }

    /**
     * Gets a given link by id
     *
     * @param int $iId
     * @return string
     */
    public function get($iId)
    {
        return $this->_aLinks[$iId];
    }

    /**
     * Gets all the links
     *
     * @param bool $bById
     * @return array
     */
    public function getAll($bById = true)
    {
        $aLinks = array();

        $aRows = $this->database()
            ->select('*')
            ->from('game_link')
            ->get('rows');

        if( !$bById )
        {
            return $aRows;
        }

        foreach($aRows as $aRow)
        {
            $aLinks[$aRow['game_link_id']] = $aRow['game_link'];
        }

        return $aLinks;
    }

    /**
     * Checks if a link exists and returns array key or false
     *
     * @param string $sLink
     * @return int
     */
    public function is($sLink)
    {
        return array_search($sLink, $this->_aLinks);
    }

    /**
     * Replaces static path with smaller footprint
     *
     * @param string $sUrl
     * @return int
     */
    public function shrink($sUrl)
    {
        $sDomain = SAA_Sys::getLib('url')->domain($sUrl);

        $iId = $this->is($sDomain);

        if( $iId === false )
        {
            $iId = SAA_Sys::getModel('game.link.process')->add($sDomain);
        }

        return str_replace($sDomain, '{'.$iId.'}', $sUrl);
    }
}
?>
