<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Add a game
     *
     * @param array $aVal
     * @param array $aLanguages
     * @return int
     */
    public function add($aVal, $aLanguages)
    {
        $oFile = SAA_Sys::getLib('file');
        $oGameAdder = SAA_Sys::getModel('game.adder');
        $oLangGameProcess = SAA_Sys::getModel('language.game.process');

        $aVal['slug'] = $this->parse()->toAlphaNumeric($aVal['slug']);

        foreach($aVal['title'] as $iLanguage => $sText)
        {
            $aVal['title'][$iLanguage] = $this->parse()->removeTags($sText);

            if( !strlen($aVal['title'][$iLanguage]) )
            {
                SAA_Error::add('Please provide title text for '. $aLanguages[$iLanguage]);
            }
        }

        if( !strlen($aVal['type']) )
        {
            SAA_Error::add('Please select a type.');
        }

        if( !strlen($aVal['width']) )
        {
            SAA_Error::add('Please provide a height for the flash file.');
        }
        elseif( $aVal['width'] <= 0 )
        {
            SAA_Error::add('Width must be greater than zero.');
        }

        if( !strlen($aVal['height']) )
        {
            SAA_Error::add('Please provide a width for the flash file.');
        }
        elseif( $aVal['height'] <= 0 )
        {
            SAA_Error::add('Height must be greater than zero.');
        }

        if( !strlen($aVal['slug']) )
        {
            SAA_Error::add('Please provide a url slug.');
        }
        elseif( !SAA_Sys::getModel('game')->isUniqueSlug($aVal['slug']) )
        {
            SAA_Error::add('Url slug is already in use.');
        }

        $aGame = $oFile->getUpload('file_game', false);
        $aImage = $oFile->getUpload('file_image', false);

        if( !$aGame )
        {
            SAA_Error::add('Please provide the flash game file.');
        }
        elseif( $aGame['type'] != 'application/x-shockwave-flash' )
        {
            SAA_Error::add('File type is not a valid flash file.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $iGame = SAA_Sys::getModel('game.stats')->largestGameId() + 1;

        $sFileGame = $oFile->upload('file_game', SAA_APP_GAME_SWF, $iGame, '', 'swf');

        if( $aImage )
        {
            $sFileImage = $oFile->upload('file_image', SAA_APP_GAME_IMAGE, $iGame, '', array('jpg', 'png', 'gif'));
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $aSql = array(
            'type_id' => (int) $aVal['type'],
            'slug' => $aVal['slug'],
            'category_manual' => (isset($aVal['category']) && count($aVal['category']) ? 1 : 0),
            'file_loc' => $sFileGame,
            'px_width' => (int) $aVal['width'],
            'px_height' => (int) $aVal['height'],
            'scoring' => (int) $aVal['scoring'],
            'time_added' => SAA_TIME
        );

        if( $aImage )
        {
            $aSql['img_loc'] = $sFileImage;
        }

        $iGame = $this->database()->insert('game', $aSql);

        if( !$iGame )
        {
            return false;
        }

        $this->database()->insert('game_extra', array(
            'game_id' => $iGame,
            'source_type' => (int) $aVal['type'],
            'file_size' => filesize(SAA_APP_GAME_SWF . $sFileGame),
            'image_size' => ($aImage ? filesize(SAA_APP_GAME_IMAGE . $sFileImage) : 0)

        ));

        if( $aVal['is_featured'] )
        {
            $this->database()->insert('game_featured', array('game_id' => $iGame));
        }

        if( strlen($aVal['tags']) )
        {
            $aTags = explode(',', $aVal['tags']);

            foreach($aTags as $iTag=>$sTag)
            {
                $sTag = trim($sTag);

                if( !strlen($sTag) )
                {
                    unset($aTags[$iTag]);
                }
            }

            SAA_Sys::getModel('game.tag.process')->add($aVal['tags'], $iGame);
        }

        if( isset($aVal['category']) && count($aVal['category']) )
        {
            foreach($aVal['category'] as $iCategory)
            {
                $this->database()->insert('game_category', array('game_id' => $iGame, 'category_id' => $iCategory));
            }
        }

        if( isset($aVal['map']) && count($aVal['map']) )
        {
            $aMaps = $oCategoryMap->getAllType($aGame['type_id'], false);

            foreach($aVal['map'] as $iMap)
            {
                $this->database()->insert('game_category_map', array('game_id' => $iGame, 'category_name' => (isset($aMaps[$iMap]) ? $aMaps[$iMap]['map_name'] : ''), 'map_id' => $iMap));
            }
        }

        if( isset($aVal['scoring_hash']) && count($aVal['scoring_hash']) )
        {
            SAA_Sys::getModel('game.scoring.process')->add($aVal['scoring_hash'], $iGame);
        }

        SAA_Sys::getModel('game.category.process')->rebuildSingle($iGame);

        $oLangGameProcess->add(array(
            'game_id' => $iGame,
            'var_name' => 'title',
            'text' => $aVal['title']
        ));

        $oLangGameProcess->add(array(
            'game_id' => $iGame,
            'var_name' => 'description',
            'text' => $aVal['description']
        ));

        $oLangGameProcess->add(array(
            'game_id' => $iGame,
            'var_name' => 'direction',
            'text' => $aVal['direction']
        ));

        return $iGame;
    }

    /**
     * Deletes a given set of game(s)
     *
     * @param array $aVars
     * @return mixed
     */
    public function delete($aVars)
    {
        $iGame = $aVars['current'];

        if( $aVars['stage'] == 'initial' )
        {
            $this->database()->delete('game_category', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_category_cache', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_category_map', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_extra', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_favorite', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_featured', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_rate', 'item_id = '. (int) $iGame);
            $this->database()->delete('game_scoring', 'game_id = '. (int) $iGame);
            $this->database()->delete('game_tag', 'game_id = '. (int) $iGame);

            $this->database()->update('game', array(
                'scoring' => '0'
            ), 'game_id = '. (int) $iGame);

            return array(
                'stage' => 'report',
                'page' => 1,
                'current' => $iGame,
                'remaining' => $aVars['remaining']
            );
        }
        elseif( $aVars['stage'] == 'report' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('report.process')->deleteBy('game', $iGame, $iPage) )
            {
                return array(
                    'stage' => 'report',
                    'page' => ($iPage + 1),
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                return array(
                    'stage' => 'user_award',
                    'page' => 1,
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'user_award' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('game.achievement.process')->deleteGameUser($iGame, $iPage) )
            {
                return array(
                    'stage' => 'user_award',
                    'page' => ($iPage + 1),
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                return array(
                    'stage' => 'award',
                    'page' => 1,
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'award' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('game.achievement.process')->deleteGame($iGame, $iPage) )
            {
                return array(
                    'stage' => 'award',
                    'page' => ($iPage + 1),
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                return array(
                    'stage' => 'score',
                    'page' => 1,
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'score' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('score.process')->resetGame($iGame) )
            {
                return array(
                    'stage' => 'score',
                    'page' => ($iPage + 1),
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                return array(
                    'stage' => 'feed',
                    'page' => 1,
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'feed' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('feed.process')->deleteMissing($iPage) )
            {
                return array(
                    'stage' => 'feed',
                    'page' => ($iPage + 1),
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                return array(
                    'stage' => 'phrase',
                    'page' => 1,
                    'current' => $iGame,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'phrase' )
        {
            $this->database()->delete('language_game_phrase', 'game_id = '. (int) $iGame);

            $aGame = $this->database()
                    ->select('file_loc, img_loc')
                    ->from('game')
                    ->where('game_id = '. (int) $iGame)
                    ->get('row');

            $oFile = SAA_Sys::getLib('file');

            if( $oFile->exists(SAA_APP_GAME_IMAGE . $aGame['img_loc']) )
            {
                $oFile->delete(SAA_APP_GAME_IMAGE . $aGame['img_loc']);
            }

            if( $oFile->exists(SAA_APP_GAME_SWF . $aGame['file_loc']) )
            {
                $oFile->delete(SAA_APP_GAME_SWF . $aGame['file_loc']);
            }

            $this->database()->delete('game', 'game_id = '. (int) $iGame);

            if( $aVars['remaining'] )
            {
                return array(
                    'stage' => 'initial',
                    'current' => array_shift($aVars['remaining']),
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                return array(
                    'stage' => 'finished'
                );
            }
        }

        return false;
    }

    /**
     * Removes given games using the remove games page
     *
     * @param array $aForm
     * @return array
     */
    public function deleteMass($aForm)
    {
        $oFile = SAA_Sys::getLib('file');

        $iRemoved = (int) $aForm['total'];

        $sGroupBy = '';
        $bJoinByCategory = false;
        $aWhere = array();

        if( isset($aForm['game_type']) )
        {
            $sIn = '';

            foreach($aForm['game_type'] as $iType)
            {
                $sIn .= (int) $iType . ', ';
            }

            $sIn = rtrim(trim($sIn), ',');

            if( strlen($sIn) )
            {
                $aWhere[] = 'g.type_id IN('.$sIn.') AND';
            }
        }

        if( isset($aForm['scoring']) )
        {
            if( count($aForm['scoring']) == 1 )
            {
                $aWhere[] = 'g.scoring = '.($aForm['scoring'][0] == 'yes' ? '1' : '0').' AND';
            }
        }

        if( isset($aForm['category']) )
        {
            $sIn = '';

            foreach($aForm['category'] as $iId)
            {
                $sIn .= (int) $iId . ', ';
            }

            $sIn = rtrim(trim($sIn), ',');

            if( strlen($sIn) )
            {
                $bJoinByCategory = true;

                $sGroupBy = 'g.game_id';

                $sCategory = 'cd.map_id = gc.map_id AND cd.category_id IN('.$sIn.')';
            }
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        if( $sGroupBy )
        {
            $this->database()->group($sGroupBy);
        }

        if( $bJoinByCategory )
        {
            $this->database()->join('game_category_map', 'gc', 'gc.game_id = g.game_id')->join('category_data', 'cd', $sCategory);
        }

        $aGames = $this->database()
            ->select('g.game_id, g.file_loc, g.img_loc, ge.file_size, ge.image_size')
            ->from('game', 'g')
            ->join('game_extra', 'ge', 'ge.game_id = g.game_id')
            ->limit(50)
            ->get('rows');

        $sIn = '';

        foreach($aGames as $aGame)
        {
            $iRemoved++;

            if( $aGame['file_size'] > 0  && strlen($aGame['file_loc']) )
            {
                $oFile->delete(SAA_APP_GAME_SWF . $aGame['file_loc']);
            }

            if( $aGame['image_size'] > 0  && strlen($aGame['img_loc']) )
            {
                $oFile->delete(SAA_APP_GAME_IMAGE . $aGame['img_loc']);
            }

            $sIn .= (int) $aGame['game_id'] . ', ';
        }

        if( strlen($sIn) )
        {
            $sIn = rtrim(trim($sIn), ',');

            $this->_delete($sIn);
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        if( $sGroupBy )
        {
            $this->database()->group($sGroupBy);
        }

        if( $bJoinByCategory )
        {
            $this->database()->join('game_category_map', 'gc', 'gc.game_id = g.game_id')->join('category_data', 'cd', $sCategory);
        }

        $iLeft = $this->database()
            ->select('COUNT(*)')
            ->from('game', 'g')
            ->get('column');

        return array($iLeft, $iRemoved);
    }

    /**
     * Updates a given game
     *
     * @param array $aForm
     * @param array $aGame
     * @return bool
     */
    public function edit($aForm, $aGame)
    {
        $iGame = $aGame['game_id'];
        $oLanguageGame = SAA_Sys::getModel('language.game.process');

        //update text
        $oLanguageGame->edit($aForm['title'], 'title', $iGame);
        $oLanguageGame->edit($aForm['description'], 'description', $iGame);
        $oLanguageGame->edit($aForm['direction'], 'direction', $iGame);

        //update tags
        if( strlen(trim($aForm['tags'])) )
        {
            SAA_Sys::getModel('game.tag.process')->update($aForm['tags'], $aGame['tags'], $iGame);
        }
        //remove all tags
        elseif( count($aGame['tags']) )
        {
            SAA_Sys::getModel('game.tag.process')->deleteAll($iGame);
        }

        //is featured
        if( $aForm['is_featured'] == 1 && !$aGame['is_featured'] )
        {
            $this->database()->insert('game_featured', array('game_id' => $iGame));
        }
        //is no longer featured
        elseif( $aForm['is_featured'] != 1 && $aGame['is_featured'] )
        {
            $this->database()->delete('game_featured', 'game_id = '. (int) $iGame);
        }

        //scoring hash
        if( $aGame['scoring'] )
        {

            if( isset($aForm['scoring_hash']) )
            {
                SAA_Sys::getModel('game.scoring.process')->update($aForm['scoring_hash'], $aGame, $iGame);
            }
            //if old scoring hashes existed, remove them
            elseif( count($aGame['scoring_info']) )
            {
                SAA_Sys::getModel('game.scoring.process')->deleteAll($iGame);
            }
        }

        //category maps
        if( isset($aForm['map']) )
        {
            $oCategoryMap = SAA_Sys::getModel('category.map');

            $aMaps = $oCategoryMap->getAllType($aGame['type_id'], false);

            //insert new
            foreach($aForm['map'] as $iMap)
            {
                if( in_array($iMap, $aGame['maps']) )
                {
                    continue;
                }

                $this->database()->insert('game_category_map', array('game_id' => $iGame, 'category_name' => (isset($aMaps[$iMap]) ? $aMaps[$iMap]['map_name'] : ''), 'map_id' => $iMap));
            }

            //remove old
            foreach($aGame['maps'] as $iMap)
            {
                if( in_array($iMap, $aForm['map']) )
                {
                    continue;
                }
                $this->database()->delete('game_category_map', 'game_id = '. (int) $iGame. ' AND map_id = '.(int) $iMap);
            }
        }
        //if old category maps existed, remove them
        elseif( $aGame['maps'] )
        {
            $this->database()->delete('game_category_map', 'game_id = '. (int) $iGame);
        }

        //categories
        if( isset($aForm['category']) && $aForm['category_manual'] )
        {
            $this->database()->delete('game_category', 'game_id = '. (int) $iGame);

            foreach($aForm['category'] as $iCategory)
            {
                $this->database()->insert('game_category', array('game_id' => $iGame, 'category_id' => $iCategory));
            }
        }
        //if no categories selected
        else
        {
            //if old categories exists remove them
            if( ($aForm['category_manual'] || $aGame['category_manual']) && $aGame['categories'] )
            {
                $this->database()->delete('game_category', 'game_id = '. (int) $iGame);
            }
        }

        $aSql = array(
            'category_manual' => (int) $aForm['category_manual']
        );

        $this->database()->update('game', $aSql, 'game_id = '. (int) $aGame['game_id']);

        SAA_Sys::getModel('game.category.process')->rebuildSingle($iGame);

        return true;
    }

    /**
     * Updates a given column for a game
     *
     * @param int $iGame
     * @param string $sCol
     * @param string $sVal
     */
    public function editCol($iGame, $sCol, $sVal)
    {
        $aSql = array(
            $sCol => $sVal
        );

        $this->database()->update('game', $aSql, 'game_id = '. (int) $iGame);
    }

    /**
     * Reorders the featured games list
     *
     * @param array $aOrder
     * @return bool
     */
    public function editOrder($aOrder)
    {
        foreach($aOrder as $iOrder => $iGame)
        {
            $this->database()->update('game_featured', array('ordering' => (int) $iOrder), 'game_id = '. (int) $iGame);
        }

        return true;
    }

    /**
     * Rebuilds the total_scores column
     *
     * @param int $iPage
     * @param int $iOffset
     */
    public function rebuildScores($iPage, $iOffset)
    {
        $aGames = $this->database()
            ->select('game_id')
            ->from('game')
            ->limit($iOffset, $iPage)
            ->key('game_id')
            ->get('rows');

        $aTotals = $this->database()
                ->select('COUNT(*) as total, game_id')
                ->from('score')
                ->where('game_id IN('.implode(',', array_keys($aGames)).')')
                ->group('game_id')
                ->get('rows');

        foreach($aTotals as $aGame)
        {
            $this->database()->update('game', array('total_scores' => $aGame['total']), 'game_id = '. $aGame['game_id']);
        }
    }

    /**
     * Features/Unfeatures a set of games
     *
     * @param array $aGames
     * @param string $sAction
     * @return bool
     */
    public function toggleFeatured($aGames, $sAction)
    {
        $sAction = ($sAction == 'feature' ? 'add' : 'remove');

        $aGames = SAA_Sys::getModel('game')->getGames($aGames, true , array('featured' => 'leftjoin'));

        if( $sAction == 'add' )
        {
            foreach($aGames as $aGame)
            {
                if( !isset($aGame['feature_id']) )
                {
                    $this->database()->update('game', array('featured' => 1), 'game_id = '. (int) $aGame['game_id']);

                    $this->database()->insert('game_featured', array('game_id' => (int) $aGame['game_id'], 'ordering' => 0));
                }
            }
        }
        else
        {
            $sIn = '';

            foreach($aGames as $aGame)
            {
                if( isset($aGame['feature_id']) )
                {
                    $sIn .= (int) $aGame['game_id'] . ', ';

                    $this->database()->update('game', array('featured' => 0), 'game_id = '. (int) $aGame['game_id']);
                }
            }

            if( strlen($sIn) )
            {
                $sIn = rtrim(trim($sIn), ',');

                $this->database()->delete('game_featured', 'game_id IN('.$sIn.')');
            }
        }

        return true;
    }

    /**
     * Updates the total plays counter
     *
     * @param int $iGame
     */
    public function updateCounter($iGame)
    {
        $this->database()->updateCounter('game', 'total_plays', '+', '1', 'game_id = '.(int) $iGame, false);
    }

    /**
     * Removes a set of games from database
     *
     * @param string $sIn
     */
    private function _delete($sIn)
    {
        $oAchievement = SAA_Sys::getModel('game.achievement.process');

        $oAchievement->deleteForUsers($sIn, true); //Remove user's achievements
        $oAchievement->delete($sIn); //Remove achievements

        $this->database()->delete('game', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_category_map', 'game_id IN('.$sIn.')');
        //$this->database()->delete('game_challenge', 'game_id IN('.$sIn.')');
        //$this->database()->delete('game_challenge_cache', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_extra', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_favorite', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_featured', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_rate', 'item_id IN('.$sIn.')');
        $this->database()->delete('game_scoring', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_tag', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_text', 'game_id IN('.$sIn.')');
        $this->database()->delete('game_favorite', 'game_id IN('.$sIn.')');
		$this->database()->delete('language_game_phrase', 'game_id IN('.$sIn.')');
        $this->database()->delete('score', 'game_id IN('.$sIn.')');
        $this->database()->delete('score_hash', 'game_id IN('.$sIn.')');
        $this->database()->delete('score_high', 'game_id IN('.$sIn.')');
        $this->database()->delete('score_user', 'game_id IN('.$sIn.')');

        SAA_Sys::getModel('comment.process')->deleteAll($sIn, 'game'); //Remove comments
        SAA_Sys::getModel('feed.process')->deleteByParent('score', $sIn); //Remove feeds
    }
}
?>
