<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Scoring Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Scoring_Process extends SAA_Model
{
    /**
     * Adds scoring hash(s) to a given game
     *
     * @param array $aHash
     * @param int $iGame
     * @return mixed
     */
    public function add($aHash, $iGame)
    {
        if( isset($aHash['scoring_type']) )
        {
            $aHash = array($aHash['scoring_type'] => $aHash['scoring_hash']);
        }

        $aSql = array();

        foreach($aHash as $iType => $sHash)
        {
            $aSql[] = array(
                $iGame,
                $iType,
                $sHash
            );
        }

        if( $aSql )
        {
            $this->database()->insertMultiple('game_scoring', array(
                'game_id',
                'scoring_type',
                'scoring_hash'
            ), $aSql);
        }

        return true;
    }

    /**
     * Removes all scoring hash for a given game
     *
     * @param int $iGame
     */
    public function deleteAll($iGame)
    {
        $this->database()->delete('game_scoring', 'game_id = '. (int) $iGame);
    }

    /**
     * Updates the scoring hash(s) for a given game
     *
     * @param array $aHash
     * @param array $aExisting
     * @param int $iGame
     * @return bool
     */
    public function update($aHash, $aExisting, $iGame)
    {
        $aSql = array();

        foreach($aHash as $iType => $sHash)
        {
            $sHash = trim($sHash);

            if( isset($aExisting['scoring_info'][$iType]) )
            {
                if( $aExisting['scoring_info'][$iType] == $sHash )
                {
                    continue;
                }

                $this->database()->update('game_scoring', array('scoring_hash' => $sHash), 'game_id = '. (int) $iGame .' AND scoring_type = '. (int) $iType);
            }
            else
            {
                $aSql[] = array(
                    $iGame,
                    $iType,
                    $sHash
                );
            }
        }

        if( $aSql )
        {
            $this->database()->insertMultiple('game_scoring', array(
                'game_id',
                'scoring_type',
                'scoring_hash'
            ), $aSql);
        }

        //remove old hashes
        foreach($aExisting['scoring_info'] as $iType => $sHash)
        {
            if( isset($aHash[$iType]) )
            {
                continue;
            }

            $this->database()->delete('game_scoring', 'game_id = '. (int) $iGame. ' AND scoring_type = '.(int) $iType);
        }

        return true;
    }
}
?>
