<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Search Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Search extends SAA_Model
{
    private $_bIsAdmin = false;

    private $_aJoin = array();
    private $_sSelect = '';

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $this->clearJoin();

        return $this;
    }

    /**
     * Empty the join array
     */
    private function clearJoin()
    {
        $this->_aJoin = array(
            'count' => array(),
            'rows' => array()
        );

        return $this;
    }

    /**
     * Empty the select appender
     */
    private function clearSelect()
    {
        $this->_sSelect = '';

        return $this;
    }

    /**
     * Gets games using a given criteria
     *
     * @param bool $bIsAdmin
     * @param array $aForm
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    public function execute($bIsAdmin, $aForm, $iPg, $iPgSize)
    {
        $oUrl = SAA_Sys::getLib('url');
        $oLink = SAA_Sys::getModel('game.link');

        $iLanguage = SAA_Sys::getLib('language')->getDefault();

        if( $bIsAdmin )
        {
            $this->_bIsAdmin = true;
        }

        if( $this->_bIsAdmin )
        {
            $aWhere = $this->prepareAdmin($aForm);
        }
        else
        {
            $aWhere = $this->preparePublic($aForm);
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        $this->getJoin();

        $iGames = $this->database()
            ->select('COUNT(*)')
            ->from('game', 'g')
            ->join('language_game_phrase', 'lgp', 'lgp.game_id = g.game_id AND lgp.var_name = "title" AND lgp.language_id = ' . (int) $iLanguage)
            ->get('column');

        $this->getSelect();

        if( $this->_bIsAdmin )
        {
            $this->database()->order('lgp.text');
        }
        else
        {
            $oUser = SAA_Sys::getModel('user');

            $this->database()
                    ->select('sh.high_score, '. $oUser->getUserFields('shu', 'high') .',')
                    ->leftjoin('score_high', 'sh', 'sh.game_id = g.game_id')
                    ->leftjoin('user', 'shu', 'shu.user_id = sh.user_id')
                    ->order(array('g.featured' => 'DESC', 'gf.ordering' => 'ASC', 'lgp.text' => 'ASC'));

            if( SAA_Sys::isUser() )
            {
                if( SAA_Sys::getModel('rating')->isEnabled('game') )
                {
                    SAA_Sys::getModel('rating')->getRowQuery('game_rate', 'r.user_id = '. SAA_Sys::getUserId() .' AND r.item_id = g.game_id');
                }

                $this->database()
                    ->select('su.user_score, su.time_scored as user_time_score,')
                    ->leftjoin('score_user', 'su', 'su.game_id = g.game_id AND su.user_id = '. SAA_Sys::getUserId());
            }
            else
            {
                if( SAA_Sys::getModel('rating')->isEnabled('game') )
                {
                    $this->database()->select('0 as rate_id,');
                }
            }
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        $this->getJoin(false);

        $aGames = $this->database()
            ->select('g.*')
            ->from('game', 'g')
            ->join('language_game_phrase', 'lgp', 'lgp.game_id = g.game_id AND lgp.var_name = "title" AND lgp.language_id = ' . (int) $iLanguage)
            ->limit($iPgSize, $iPg)
            ->get('rows');

        if( $aGames )
        {
            $aGames = SAA_Sys::getModel('language.game')->getTexts($aGames, $iLanguage);

            if( $this->_bIsAdmin )
            {
                $aGames = SAA_Sys::getModel('game.tag')->getForMultiple($aGames);
            }
            else
            {
                if( SAA_Sys::getSett('game.category_limit') >= 0 )
                {
                    $aGames = SAA_Sys::getModel('game.category')->getAll($aGames);
                }

                if( SAA_Sys::getSett('game.max_tags') >= 0 )
                {
                    $iLimit = SAA_Sys::getSett('game.max_tags');

                    $aGames = SAA_Sys::getModel('game.tag')->getForMultiple($aGames, $iLimit);
                }
            }

            foreach($aGames as $i => $aGame)
            {
                if( isset($aGame['high_score']) )
                {
                    $aGames[$i]['high_user'] = (SAA_Sys::getSett('user.enable_profile') ? '<a href="'.$oUrl->makeUrl(array('controller' => 'profile', 'vars' => array('user' => $aGame['high_user_id']))).'">'.$aGame['high_user_name'].'</a>' : $aGame['high_user_name']);
                }

                $aGames[$i]['img_loc'] = $oLink->expand($aGame['img_loc'], SAA_APP_GAME_IMAGE_URL);
            }
        }

        $this->clearSelect()
            ->clearJoin();

        return array($iGames, $aGames);
    }

    /**
     * Adds all joins to database call
     *
     * @param bool $bCounter
     */
    private function getJoin($bCount = true)
    {
        $sKey = ($bCount ? 'count' : 'rows');

        foreach($this->_aJoin[$sKey] as $aJoin)
        {
            switch($aJoin['type'])
            {
                case 'left':
                    $sMethod = 'leftjoin';
                    break;
                case 'right':
                    $sMethod = 'rightjoin';
                    break;
                default:
                    $sMethod = 'join';
            }

            $this->database()->$sMethod($aJoin['table'], $aJoin['alias'], $aJoin['conditional']);
        }
    }

    /**
     * Appends select to database call
     * 
     */
    private function getSelect()
    {
        if( $this->_sSelect )
        {
            $sSelect = trim(trim($this->_sSelect), ',');

            $this->database()->select($sSelect . ', ');
        }

        return $this;
    }

    /**
     * Process search filter from admin
     *
     * @param array $aForm
     * @return array
     */
    private function prepareAdmin($aForm)
    {
        $aWhere = array();

        if( isset($aForm['game_type']) )
        {
            $sWhere = 'g.type_id IN(';

            foreach($aForm['game_type'] as $iType)
            {
                $sWhere .= $this->database()->esc($iType). ', ';
            }

            $aWhere[] = rtrim(trim($sWhere), ',').') AND';
        }

        if( strlen($aForm['category_id']) )
        {
            $this->setJoin('left', 'game_category_cache', 'gcc', 'gcc.game_id = g.game_id');

            $aWhere[] = 'gcc.category_id = '. (int) $aForm['category_id'] .' AND';
        }

        if( isset($aForm['scoring']) )
        {
            $aWhere[] = 'g.scoring = '.($aForm['scoring'] == 'yes' ? 1 : 0).' AND';
        }

        if( strlen($aForm['title']) )
        {
            $aWhere[] = 'lgp.text LIKE "%'.$this->database()->esc($this->parse()->rareToEntities($aForm['title'])).'%" AND';
        }

        if( strlen($aForm['tags']) )
        {
            $aTags = implode($aForm['tags'], ',');

            $sJoin = 'g.game_id = gt.game_id AND gt.tag IN(';

            foreach($aTags as $sTag)
            {
                $sJoin .= '"'.$this->database()->esc($this->parse()->rareToEntities($sTag)).'", ';
            }

            $sJoin = rtrim(trim($sJoin), ',').')';

            $this->setJoin('join', 'game_tag', 'gt', $sJoin);

        }

        $this->setSelect('gf.feature_id');

        if( isset($aForm['is_featured']) )
        {
            $sTbl = (isset($aForm['is_featured']) ? 'join' : 'leftjoin');

            $this->setJoin('join', 'game_featured', 'gf', 'gf.game_id = g.game_id');

        }
        else
        {
            $this->setJoin('left', 'game_featured', 'gf', 'gf.game_id = g.game_id', 'rows');
        }

        return $aWhere;
    }

    /**
     * Process search filter from public
     *
     * @param array $aForm
     * @return array
     */
    private function preparePublic($aForm)
    {
        $aWhere = array();

        $aDisableType = array();

        if( !SAA_Sys::getSett('mochi.is_enabled') )
        {
            $aType = SAA_Sys::getModel('game.type')->get('MochiMedia', false);

            $aDisableType[] = (int) $aType['game_type_id'];
        }

        if( $aDisableType )
        {
            $sWhere = 'g.type_id IN(';

            foreach($aDisableType as $iType)
            {
                $sWhere .= $iType . ', ';
            }

            $aWhere[] = rtrim(trim($sWhere), ',').') AND';
        }

        if( isset($aForm['category']) && strlen($aForm['category']) )
        {
            $this->setJoin('left', 'game_category_cache', 'gcc', 'gcc.game_id = g.game_id');

            $aCategory = SAA_Sys::getLib('category')->get($aForm['category'], false);

            if( $aCategory )
            {
                $aWhere[] = 'gcc.category_id = '. (int) $aCategory['category_id'] .' AND';
            }
        }

        if( !SAA_Sys::getSett('global.show_non_scoring_games') )
        {
            $aWhere[] = 'g.scoring = 1 AND';
        }

        if( isset($aForm['title']) && strlen($aForm['title']) )
        {
            $aWhere[] = 'lgp.var_name = "title" AND lgp.text LIKE "%'.$this->database()->esc($this->parse()->rareToEntities($aForm['title'])).'%" AND';
        }

        if( isset($aForm['tag']) && strlen(trim($aForm['tag'])) )
        {
            $this->setJoin('join', 'game_tag', 'gt', 'g.game_id = gt.game_id AND gt.tag = "'.$this->database()->esc($this->parse()->rareToEntities($aForm['tag'])).'"');
        }

        if( isset($aForm['is_featured']) && $aForm['is_featured'] == 1 )
        {
            $this->setJoin('join', 'game_featured', 'gf', 'gf.game_id = g.game_id');
        }
        else
        {
            $this->setJoin('left', 'game_featured', 'gf', 'gf.game_id = g.game_id', 'rows');
        }

        if( isset($aForm['favorite_user_id']) )
        {
            $this->setSelect('gfav.item_id as is_user_id_favorite,')
                        ->setJoin('join', 'game_favorite', 'gfav2', 'gfav2.game_id = g.game_id AND gfav2.user_id = '. (int) $aForm['favorite_user_id']);
        }

        if( SAA_Sys::isUser() )
        {
            if( SAA_Sys::getSett('favorite.is_enabled') )
            {
                $sType = (isset($aForm['is_favorite']) && $aForm['is_favorite'] == 1 ? 'join' : 'left');

                $this->setSelect('gfav.item_id as is_favorite,')
                        ->setJoin($sType, 'game_favorite', 'gfav', 'gfav.game_id = g.game_id AND gfav.user_id = '. SAA_Sys::getUserId());
            }
            else
            {
                $this->setSelect('0 as is_favorite,');
            }
        }

        return $aWhere;
    }

    /**
     * Appends join
     *
     * @param string $sType
     * @param string $sTable
     * @param string $sAlias
     * @param string $sConditional
     */
    private function setJoin($sType, $sTable, $sAlias, $sConditional, $sQueryType = 'both')
    {
        if( $sQueryType == 'count' || $sQueryType == 'both' )
        {
            $this->_aJoin['count'][] = array(
                'type' => $sType,
                'table' => $sTable,
                'alias' => $sAlias,
                'conditional' => $sConditional
            );
        }

        if( $sQueryType == 'rows' || $sQueryType == 'both' )
        {
            $this->_aJoin['rows'][] = array(
                'type' => $sType,
                'table' => $sTable,
                'alias' => $sAlias,
                'conditional' => $sConditional
            );
        }

        return $this;
    }

    /**
     * Appends select
     *
     * @param string $sSelect
     */
    private function setSelect($sSelect)
    {
        $this->_sSelect = $sSelect .',';

        return $this;
    }
}
?>