<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Tag Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Tag_Process extends SAA_Model
{
    /**
     * Adds tag(s) to a given game
     *
     * @param string $mTags
     * @param int $iGame
     * @return mixed
     */
    public function add($mTags, $iGame)
    {
        if( is_string($mTags) )
        {
            $mTags = (array) $mTags;
        }

        $aSql = array();

        foreach($mTags as $sTag)
        {
            $aSql[] = array(
                $iGame,
                $this->parse()->rareToEntities($sTag)
            );
        }

        if( $aSql )
        {
            $this->database()->insertMultiple('game_tag', array(
                'game_id',
                'tag'
            ), $aSql);
        }

        return true;
    }

    /**
     * Removes all tags for a given game
     *
     * @param int $iGame
     */
    public function deleteAll($iGame)
    {
        $this->database()->delete('game_tag', 'game_id = '. (int) $iGame);
    }

    /**
     * Updates the tags for a given game
     *
     * @param string $sTags
     * @param array $aExisting
     * @param int $iGame
     * @return bool
     */
    public function update($sTags, $aExisting, $iGame)
    {
        $aTags = explode(',', $sTags);
		$aTags = array_map('trim', $aTags);
		$aTags = array_map(array($this->parse(), 'rareToEntities'), $aTags);

        $aSql = array();

        foreach($aTags as $iTag => $sTag)
        {
            if( !strlen($sTag) || in_array($sTag, $aExisting) )
            {
                continue;
            }

            $aSql[] = array(
                $iGame,
                $sTag
            );
        }

        if( $aSql )
        {
            $this->database()->insertMultiple('game_tag', array(
                'game_id',
                'tag'
            ), $aSql);
        }

        //delete old tags
        foreach($aExisting as $sTag)
        {
            if( in_array($sTag, $aTags) )
            {
                continue;
            }

            $this->database()->delete('game_tag', 'game_id = '. (int) $iGame. ' AND tag = "'.$this->database()->esc($sTag).'"');
        }

        return true;
    }
}
?>
