<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Type Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Type extends SAA_Model
{
    private $_aTypesById = array();
    private $_aTypesByTitle = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $sCache = 'game_type';

        if( !($aCache = $this->cache()->get($sCache)) )
        {
            $aCache = $this->_buildCache();

            $this->cache()->create($sCache, $aCache);
        }

        list($this->_aTypesById, $this->_aTypesByTitle) = $aCache;
    }

    /**
     * Gets a game type
     *
     * @param mixed $mId key
     * @param bool $bById when true, gets by id
     * @return mixed
     */
    public function get($mId, $bById = true)
    {
        $sArray = ($bById ? '_aTypesById' : '_aTypesByTitle');

        return (isset($this->{$sArray}[$mId]) ? $this->{$sArray}[$mId] : false);
    }

    /**
     * Gets all game types
     *
     * @param bool $bById when true, array keys are ids
     * @return array
     */
    public function getAll($bById = true)
    {
        return ($bById ? $this->_aTypesById : $this->_aTypesByTitle);
    }

    /**
     * Gets the total number of games for a given game type
     *
     * @param mixed $mId
     * @param bool $bById when set to true, mId is id
     * @return int
     */
    public function getCnt($mId, $bById = true)
    {
        if( !$bById )
        {
            $aType = $this->get($mId, $bById);

            $mId = $aType['game_type_id'];
        }

        return $this->database()
            ->select('COUNT(*)')
            ->from('game')
            ->where('type_id = '. (int) $mId)
            ->get('column');
    }

    /**
     * Builds the cache data
     *
     * @return array
     */
    private function _buildCache()
    {
        $aRows = $this->database()
            ->select('*')
            ->from('game_type')
            ->get('rows');

        $aIds = array();
        $aTitles = array();

        foreach($aRows as $aRow)
        {
            $aIds[$aRow['game_type_id']] = $aRow;
            $aTitles[$aRow['game_type_title']] = $aRow;
        }

        return array($aIds, $aTitles);
    }

    /*
     * Preforms all callbacks to game type model classes
     */
    public function  __call($sMethod, $aArgs)
    {
        if( $sMethod == 'getCategoryMaps' )
        {
            $sKey = (isset($aArgs[0]) ? $aArgs[0] : 'id');

            $aMaps = array();

            foreach($this->_aTypesById as $aType)
            {
                $aMaps[($sKey == 'title' ? $aType['game_type_title'] : $aType['game_type_id'])] = SAA_Sys::getModel('game.type.'. strtolower($aType['game_type_title']))->getCategoryMaps();
            }

            return $aMaps;
        }
    }
}
?>
