<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Game Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Language_Game_Process extends SAA_Model
{
    private $_aCols = array(
        'language_id',
        'game_id',
        'var_name',
        'text',
        'text_default',
        'time_added'
    );

    /**
     * Class Construtor
     */
    public function  __construct()
    {

    }

    /**
     * Add a game phrase to all existing languages
     *
     * @param array $aVal
     * @return mixed
     */
    public function add($aVal)
    {
        foreach($aVal['text'] as $iLanguage => $sText)
        {
            $sText = $this->parse()->rareToEntities($sText);

            $aSql = array(
                'language_id' => $iLanguage,
                'game_id' => $aVal['game_id'],
                'var_name' => $aVal['var_name'],
                'text' => $sText,
                'text_default' => $sText,
                'time_added' => SAA_TIME
            );

            $iId = $this->database()->insert('language_game_phrase', $aSql);
        }

        if( !$iId )
        {
            return false;
        }

        return true;
    }

    /**
     * Add missing game phrases to a language using the default language
     *
     * @param int $iLanguage
     * @return array
     */
    public function addMissing($iLanguage)
    {
        $iDefault = SAA_Sys::getLib('language')->getDefault();

        $aPhrases = $this->database()
            ->select('l.game_id, l.var_name, l.text, l.text_default')
            ->from('language_game_phrase', 'l')
            ->leftjoin('language_game_phrase', 'l2', 'l2.game_id = l.game_id AND l2.var_name = l.var_name AND l2.language_id = '. (int) $iLanguage)
            ->where('l.language_id = '. (int) $iDefault .' AND l2.phrase_id IS NULL')
            ->limit(50)
            ->get('rows');

        $aSql = array();

        foreach($aPhrases as $aPhrase)
        {
            $aSql[] = array(
                $iLanguage,
                $aPhrase['game_id'],
                $aPhrase['var_name'],
                $aPhrase['text'],
                $aPhrase['text_default'],
                SAA_TIME,
            );
        }

        if( count($aSql) )
        {
            $this->database()->insertMultiple('language_game_phrase', $this->_aCols, $aSql);
        }

        $iLeft = $this->database()
            ->select('COUNT(*)')
            ->from('language_game_phrase', 'l')
            ->leftjoin('language_game_phrase', 'l2', 'l2.game_id = l.game_id AND l2.var_name = l.var_name AND l2.language_id = '. (int) $iLanguage)
            ->where('l.language_id = '. (int) $iDefault .' AND l2.phrase_id IS NULL')
            ->get('column');

        return $iLeft;
    }

    /**
     * Changes the data for a given phrase over a given languages
     *
     * @param array $aVals updated phrase text with language id as key
     * @param string $sVar var name
     * @return bool
     */
    public function edit($aVal, $sVar, $iGame)
    {
        $sVar = $this->database()->esc($sVar);

        foreach($aVal as $iLanguage => $sText)
        {
            $aSql = array(
                'text' => $this->parse()->rareToEntities($sText)
            );

            $this->database()->update('language_game_phrase', $aSql, 'game_id = '.(int) $iGame .' AND var_name = "'. $sVar .'" AND language_id = '. (int) $iLanguage);
        }

        return true;
    }

    /**
     * Removes a phrase from all languages
     *
     * @param int $iGame
     * @param string $sVar
     * @return bool
     */
    public function delete($iGame, $sVar)
    {
        $this->database()->delete('language_game_phrase', 'game_id = '. (int) $iGame .' AND var_name = "'.$this->database()->esc($sVar).'"');

        return true;
    }

    /**
     * Revert game phrase to defaulted value
     *
     * @param int $iPhrase
     * @return bool
     */
    public function revert($iPhrase)
    {
        $this->database()->update('language_game_phrase', array('text' => $aPhrase['text_default']), 'phrase_id = '. (int) $iPhrase);

        return true;
    }
}
?>
