<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Phrase Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Language_Phrase extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Get phrase by id
     *
     * @param int $iPhraseId
     * @return array
     */
    public function get($iPhraseId)
    {
        return $this->database()
            ->select('*')
            ->from('language_phrase')
            ->where('phrase_id = '. (int) $iPhraseId)
            ->get('row');
    }

    /**
     * Get phrases by phrase_var
     *
     * @param string $sVar
     * @return array
     */
    public function getByVar($sVar)
    {
        $aPhrases = array();

        list($sSection, $sVar) = explode('.', $sVar);

        $aRows = $this->database()
            ->select('*')
            ->from('language_phrase')
            ->where('var_name = "'.$this->database()->esc($sVar).'" AND section_id = "'.$this->database()->esc($sSection).'"')
            ->get('rows');

        foreach($aRows as $aRow)
        {
            $aPhrases[$aRow['language_id']] = $aRow;
        }

        return $aPhrases;
    }

    /**
     * Get all phrases by language id
     *
     * @param int $iLanguage
     * @return array
     */
    public function getAllById($iLanguage)
    {
        return $this->database()
            ->select('*')
            ->from('language_phrase')
            ->where('language_id = '. (int) $iLanguage)
            ->get('rows');
    }

    /**
     * Gets phrases that match the provided search criteria
     *
     * @param array $aVal
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    public function search($aVal, $iPg, $iPgSize)
    {
        $aWhere = array();
        $aLanguages = SAA_Sys::getLib('language')->getLanguages();

        $aVal = $this->parse()->rareToEntities($aVal);

        if( isset($aVal['section_id']) && strlen($aVal['section_id']) )
        {
            $aWhere[] = 'lp.section_id = "'.$this->database()->esc($aVal['section_id']).'" AND';
        }

        if( isset($aVal['var_name']) && strlen($aVal['var_name']) )
        {
            $aWhere[] = 'lp.var_name LIKE "%'.$this->database()->esc($aVal['var_name']).'%" AND';
        }

        if( isset($aVal['search_text']) && strlen($aVal['search_text']) )
        {
            $aWhere[] = 'lp.text LIKE "%'.$this->database()->esc($aVal['search_text']).'%" AND';
        }

        if( count($aWhere) )
        {
            $this->database()->where($aWhere);
        }

        $iPhrases = $this->database()
            ->select('COUNT(*)')
            ->from('language_phrase', 'lp')
            ->join('language_section', 'ls', 'ls.section_name = lp.section_id AND ls.is_required = 0')
            ->get('column');

        if( count($aWhere) )
        {
            $this->database()->where($aWhere);
        }

        $aPhrases = $this->database()
            ->select('lp.*')
            ->from('language_phrase', 'lp')
            ->join('language_section', 'ls', 'ls.section_name = lp.section_id AND ls.is_required = 0')
            ->order(array(
                'section_id' => 'ASC',
                'var_name' => 'ASC',
            ))
            ->limit($iPgSize, $iPg)
            ->get('rows');

        foreach($aPhrases as $i=>$aPhrase)
        {
            $aPhrases[$i]['phrase_var'] = $aPhrase['section_id'] . '.' . $aPhrase['var_name'];
            $aPhrases[$i]['language_text'] = $aLanguages[$aPhrase['language_id']]['title'];
        }

       return array($iPhrases, $aPhrases);
    }
}
?>
