<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Language_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Adds a language while using other language for the phrases
     *
     * @param array $aVal
     * @return mixed
     */
    public function add($aVal)
    {
        if( !strlen(trim($aVal['copy'])) )
        {
            SAA_Error::add('Please select a language from which to copy the phrases.');
        }
        elseif( !SAA_Sys::getModel('language')->get($aVal['copy']) )
        {
            SAA_Error::add('The language where you are trying copy the phrases from does not exist.');
        }

        if( !strlen(trim($aVal['title'])) )
        {
            SAA_Error::add('Please enter a title.');
        }

        if( !strlen(trim($aVal['language_abrv'])) )
        {
            SAA_Error::add('Please enter a language abbreviation.');
        }
        elseif( strlen(trim($aVal['language_abrv'])) > 2 )
        {
            SAA_Error::add('Language abbreviation cannot be more than 2 characters.');
        }

        if( !strlen(trim($aVal['created_by'])) )
        {
            SAA_Error::add('Please enter the creator.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        if( $aVal['direction'] != 'ltr' )
        {
            $aVal['direction'] = 'rtl';
        }

        $aSql = array(
            'title' => $this->parse()->removeTags($aVal['title']),
            'language_abrv' => $this->parse()->removeTags($aVal['language_abrv']),
            'direction' => $aVal['direction'],
            'created_by' => $this->parse()->cleanTags($aVal['created_by'], 'global.allowed_tags'),
        );

        $iId = $this->database()->insert('language', $aSql);

        if( !$iId )
        {
            return false;
        }

        $aPhrases = SAA_Sys::getModel('language.phrase')->getAllById($aVal['copy']);

        $aSql = array();

        $aCol = array(
            'language_id',
            'section_id',
            'var_name',
            'text',
            'text_default',
            'time_added'
        );

        foreach($aPhrases as $aPhrase)
        {
            $aSql[] = array(
                $iId,
                $aPhrase['section_id'],
                $aPhrase['var_name'],
                $aPhrase['text'],
                $aPhrase['text'],
                SAA_TIME
            );
        }

        if( !($this->database()->insertMultiple('language_phrase', $aCol, $aSql)) )
        {
            $this->database()->delete('language', 'language_id = '. (int) $iId);

            return false;
        }

        //insert game phrases
        $sSql = 'INSERT INTO '. $this->database()->table('language_game_phrase') . ' (language_id, game_id, var_name, text, text_default, time_added)'.
                ' SELECT '.$iId.', lgp.game_id, lgp.var_name, lgp.text, lgp.text_default, '. SAA_TIME.' FROM '. $this->database()->table('language_game_phrase') . ' AS lgp WHERE language_id = '. (int) $aVal['copy'];

        $this->database()->query($sSql);

        $this->cache()->remove('languages');
        SAA_Sys::getModel('language.phrase.process')->clearCache();

        return true;
    }

    /**
     * Removes a language and all related phrases
     *
     * @param int $iId
     * @return bool
     */
    public function delete($iId)
    {
        $aLanguage = SAA_Sys::getModel('language')->get($iId);

        if( !$aLanguage || $aLanguage['is_default'] )
        {
            return false;
        }

        $this->database()->delete('language', 'language_id = '. (int) $iId);
        $this->database()->delete('language_phrase', 'language_id = '. (int) $iId);
        $this->database()->delete('language_game_phrase', 'language_id = '. (int) $iId);

        $this->cache()->remove('languages');
        SAA_Sys::getModel('language.phrase.process')->clearCache();

        return true;
    }

    /**
     * Updates a given language
     *
     * @param array $aVal
     * @param array $aLanguage
     * @return bool
     */
    public function edit($aVal, $aLanguage)
    {
        if( !strlen(trim($aVal['title'])) )
        {
            SAA_Error::add('Please enter a title.');
        }

        if( !strlen(trim($aVal['language_abrv'])) )
        {
            SAA_Error::add('Please enter a language abbreviation.');
        }
        elseif( strlen(trim($aVal['language_abrv'])) > 2 )
        {
            SAA_Error::add('Language abbreviation cannot be more than 2 characters.');
        }

        if( !strlen(trim($aVal['created_by'])) )
        {
            SAA_Error::add('Please enter the creator.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        if( $aVal['direction'] != 'ltr' )
        {
            $aVal['direction'] = 'rtl';
        }

        $aSql = array(
            'title' => $this->parse()->removeTags($aVal['title']),
            'language_abrv' => $this->parse()->removeTags($aVal['language_abrv']),
            'direction' => $aVal['direction'],
            'created_by' => $this->parse()->cleanTags($aVal['created_by'], 'global.allowed_tags'),
        );

        $this->database()->update('language', $aSql, 'language_id = '. (int) $aLanguage['language_id']);

        $this->cache()->remove('languages');

        return true;
    }

    /**
     * Adds a language using import data
     *
     * @param array $aXml
     * @return mixed
     */
    public function import($aXml)
    {
        $aVal = $aXml['language']['info'];

        if( !strlen(trim($aVal['title'])) )
        {
            return SAA_Error::add('The language does not have a title.');
        }
        elseif( !strlen(trim($aVal['language_abrv'])) )
        {
            return SAA_Error::add('The language does not have an abbreviation.');
        }
        elseif( strlen(trim($aVal['language_abrv'])) > 2 )
        {
            return SAA_Error::add('Language abbreviation cannot be more than 2 characters.');
        }

        $aSql = array(
            'title' => $this->parse()->removeTags($aVal['title']),
            'language_abrv' => $this->parse()->removeTags($aVal['language_abrv']),
            'direction' => (isset($aVal['direction']) ? $aVal['direction'] : ''),
            'created_by' => (isset($aVal['creator']) ? $this->parse()->cleanTags($aVal['creator'], 'global.allowed_tags') : ''),
        );

        $iId = $this->database()->insert('language', $aSql);

        if( !$iId )
        {
            return SAA_Error::add('Request cannot be processed.');
        }

        if( isset($aXml['language']['phrases']) && count($aXml['language']['phrases']) )
        {
            $aSql = array();

            $aCol = array(
                'language_id',
                'section_id',
                'var_name',
                'text',
                'text_default',
                'time_added'
            );

            $aPhrases = $aXml['language']['phrases']['phrase'];

            if( array_key_exists('@value', $aPhrases) )
            {
                $aPhrases = array($aPhrases);
            }

            foreach($aPhrases as $aPhrase)
            {
                $aSql[] = array(
                    $iId,
                    $aPhrase['@attributes']['section_id'],
                    $aPhrase['@attributes']['var_name'],
                    $aPhrase['@value'],
                    $aPhrase['@value'],
                    $aPhrase['@attributes']['time_added'],
                );
            }

            if( count($aSql) )
            {
                $this->database()->insertMultiple('language_phrase', $aCol, $aSql);
            }
        }

        if( isset($aXml['language']['sections']) && count($aXml['language']['sections']) )
        {
            $aCheck = SAA_Sys::getModel('language.section')->getAll('name');

            $aSql = array();

            $aCol = array(
                'section_name',
                'section_title'
            );

            $aSections = $aXml['language']['sections']['section'];

            if( array_key_exists('@value', $aSections) )
            {
                $aSections = array($aSections);
            }

            foreach($aSections as $aSection)
            {
                
                if( !array_key_exists($aSection['@attributes']['name'], $aCheck) )
                {
                    $aSql[] = array(
                        $aSection['@attributes']['name'],
                        $aSection['@value'],
                    );
                }
            }

            if( count($aSql) )
            {
                $this->database()->insertMultiple('language_section', $aCol, $aSql);
            }
        }

        $this->cache()->remove('languages');

        return $iId;
    }

    /**
     * Makes the given language default
     *
     * @param int $iLanguage
     * @return bool
     */
    public function setDefault($iLanguage)
    {
        $this->database()->update('language', array('is_default' => 0), 'language_id != '. (int) $iLanguage);
        $this->database()->update('language', array('is_default' => 1), 'language_id = '. (int) $iLanguage);

        $this->cache()->remove('languages');

        return true;
    }

    /**
     * Turns a language on/off
     *
     * @param int $iLanguage
     * @param int $iToggle
     * @return bool
     */
    public function toggleLanguage($iLanguage, $iToggle)
    {
        $this->database()->update('language', array('is_enabled' => $iToggle), 'language_id = '. (int) $iLanguage);

        $this->cache()->remove('languages');

        return true;
    }
}
?>
