<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Section Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Language_Section_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Add a section
     *
     * @param array $aVal array
     * @return bool
     */
    public function add($aVal)
    {
        if( !isset($aVal['section_title']) || !strlen($aVal['section_title']) )
        {
            SAA_Error::add('Please provide a title.');
        }

        if( !isset($aVal['section_name']) || !strlen($aVal['section_name']) )
        {
            SAA_Error::add('Please provide a name.');
        }
        else
        {
            $sName = $this->parse()->toAlphaNumeric($aVal['section_name'], 20, '_', '_');

            if( SAA_Sys::getModel('language.section')->get($sName, false) )
            {
                SAA_Error::add('Name is already in use. Please use a different name.');
            }
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $aSql = array(
            'section_name' => $sName,
            'section_title' => $this->parse()->removeTags($aVal['section_title'])
        );

        $iId = $this->database()->insert('language_section', $aSql);

        if( !$iId )
        {
            return false;
        }

        return true;
    }

    /**
     * Changes the data for a given section
     *
     * @param array $aVals updated phrase data
     * @return bool
     */
    public function edit($aVal, $aSection)
    {
        if( !isset($aVal['section_title']) || !strlen($aVal['section_title']) )
        {
            SAA_Error::add('Please provide a title.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $aSql = array(
            'section_title' => $this->parse()->removeTags($aVal['section_title'])
        );

        $this->database()->update('language_section', $aSql, 'section_id = '. (int) $aVal['section_id']);

        return true;
    }

    /**
     * Removes a language section and the corresponding phrases
     *
     * @param int $iId section_id
     * @return bool
     */
    public function delete($iId)
    {
        $aSection = SAA_Sys::getModel('language.section')->get($iId);

        if( !$aSection )
        {
            return false;
        }

        $this->database()->delete('language_section', 'section_id = '. (int) $iId);
        $this->database()->delete('language_phrase', 'section_id = "'. $this->database()->esc($aSection['section_name']) .'"');

        //clears the language phrases cache
        SAA_Sys::getModel('language.phrase.process')->clearCache();

        return true;
    }
}
?>
