<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Section Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Language_Section extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Gets the number of phrases using the section
     *
     * @param mixed $mSection
     * @param bool $bById
     * @return int
     */
    public function getPhraseNum($mSection, $bById = true)
    {
        if( $bById )
        {
            $aSection = $this->get($mSection);
            $mSection = $aSection['section_name'];
        }

        return $this->database()
            ->select('COUNT(*)')
            ->from('language_phrase')
            ->where('section_id = "'.$this->database()->esc($mSection).'"')
            ->get('column');
    }

    /**
     * Gets a given language section by either id or name
     *
     * @param mixed $mId
     * @param bool $bById
     * @return array
     */
    public function get($mId, $bById = true)
    {
        return $this->database()
            ->select('*')
            ->from('language_section')
            ->where($bById ? 'section_id = '. (int) $mId : 'section_name = "'.$this->database()->esc($mId).'"')
            ->get('row');
    }

    /**
     * Gets all language sections with a option to set the array key
     *
     * @param string $mKey
     * @return array
     */
    public function getAll($mKey = '')
    {
        $aSections = array();

        if( $mKey )
        {
            $this->database()->key(($mKey == 'id' ? 'section_id' : 'section_name'));
        }

        $aRows = $this->database()
            ->select('*')
            ->from('language_section')
            ->order('section_name')
            ->get('rows');

        return $aRows;
    }

    /**
     * Gets the langauge sections used by a given language
     *
     * @param int $iLanguage language id
     * @param bool $bExcludeRequired when true, exclude required language sections
     * @return array
     */
    public function getByLanguage($iLanguage, $bExcludeRequired = false)
    {
        return $this->database()
            ->select('ls.*')
            ->from('language_section', 'ls')
            ->join('language_phrase', 'lp', 'lp.section_id = ls.section_name')
            ->where('lp.language_id = '. (int) $iLanguage .($bExcludeRequired ? ' AND ls.is_required = 0' : ''))
            ->group('ls.section_id')
            ->get('rows');
    }

    /**
     * Get all for drop down
     *
     * @return array
     */
    public function getAllForSelect()
    {
        $aSections = array(
            '' => ''
        );

        $aRows = $this->getAll();

        foreach($aRows as $aRow)
        {
            if( $aRow['is_required'] )
            {
                continue;
            }

            $aSections[$aRow['section_name']] = $aRow['section_title'];
        }

        return $aSections;
    }
}
?>
