<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Menu Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Menu extends SAA_Model
{
    private $_aMenus = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $sCache = 'menu' . (SAA_Sys::isUser() ? '' : '_guest');

        if( !($this->_aMenus = $this->cache()->get($sCache)) )
        {
            $this->buildSorted();

            $this->cache()->create($sCache, $this->_aMenus);
        }
    }

    /**
     * Builds menu
     */
    public function buildSorted()
    {
        if( SAA_Sys::isUser() )
        {
            $this->database()->where('type_id != 2');
        }
        else
        {
            $this->database()->where('type_id != 1');
        }

        $this->_aMenus = $this->getAll();
    }

    /**
     * Get a given menu item
     *
     * @param int $iMenu
     * @return array
     */
    public function get($iMenu)
    {
        $aItems = $this->getAll(false);

        if( !isset($aItems[$iMenu]) )
        {
            return false;
        }

        $aItem = $aItems[$iMenu];

        $aItem['children'] = $this->_sort($aItems, $iMenu);

        return $aItem;
    }

    /**
     * Get all menu items and organizes them
     *
     * @param bool $bSort
     * @return array
     */
    public function getAll($bSort = true)
    {
        $aRows = $this->database()
            ->select('*')
            ->from('menu')
            ->order('ordering')
            ->key('menu_id')
            ->get('rows');

        return ($bSort ? $this->_sort($aRows, 0) : $aRows);
    }

    /**
     * Gets organized menu items
     *
     * @return array
     */
    public function getSorted()
    {
        return $this->_aMenus;
    }

    /**
     * Builds organized array
     *
     * @param array $aRows
     * @param int $iParent
     * @return array
     */
    public function _sort($aRows, $iParent = null)
    {
        $aChildren = array();

        foreach($aRows as $aRow)
        {
            if( $iParent == $aRow['parent_id'] )
            {
                $bHasChildren = false;

                foreach($aRows as $aCheck)
                {
                    if( $aCheck['parent_id'] == $aRow['menu_id'] )
                    {
                        $bHasChildren = true;
                        break;
                    }
                }

                if( $bHasChildren )
                {
                    $aRow['children'] = $this->_sort($aRows, $aRow['menu_id']);
                }

                $aChildren[] = $aRow;
            }
        }

        return $aChildren;
    }
}
?>
