<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Menu Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Menu_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Adds a menu item
     *
     * @param array $aVal
     * @return mixed
     */
    public function add($aVal)
    {
        reset($aVal['text']);
        $sRaw = current($aVal['text']);

        $aPhrase = array(
            'section_id' => 'menu',
            'text' => $aVal['text'],
            'var_name' => $sRaw
        );

        if( !($mPhrase = SAA_Sys::getModel('language.phrase.process')->add($aPhrase)) )
        {
            return false;
        }

        $aSql = array(
            'parent_id' => (int) (isset($aVal['parent_id']) ? $aVal['parent_id'] : 0),
            'type_id' => (int) $aVal['type_id'],
            'menu_name' => 'menu.'. $mPhrase['var_name'],
            'href_link' => $aVal['href_link'],
            'is_controller' => (SAA_Sys::getLib('validator')->checkRegex('url', $aVal['href_link']) ? 0 : 1)
        );

        $iId = $this->database()->insert('menu', $aSql);

        if( !$iId )
        {
            return false;
        }

        $this->clearCache();

        return $iId;
    }

    /**
     * Removes all menu cache
     */
    public function clearCache()
    {
        $this->cache()->remove(array(
            'menu',
            'menu_guest'
        ));
    }

    /**
     * Removes menu item and all of its children
     *
     * @param int $iId
     */
    public function delete($iId)
    {
        $aItems = SAA_Sys::getModel('menu')->get($iId);

        $this->_delete($aItems['menu_id'], $aItems['menu_name']);

        $this->clearCache();

        if( !isset($aItems['children']) )
        {
            return true;
        }

        $this->_deleteLoop($aItems['children']);

        return true;
    }

    /**
     * Updates menu item data
     * 
     * @param array $aVal
     * @param array $aMenu 
     */
    public function edit($aVals, $aMenu)
    {
        //if the language phrases are not updated, return error
        if( !($mPhrase = SAA_Sys::getModel('language.phrase.process')->editAll($aVals['text'], $aMenu['menu_name'])) )
        {
            return false;
        }

        $aSql = array(
            'parent_id' => $aVals['parent_id'],
            'href_link' => $aVals['href_link'],
            'type_id' => $aVals['type_id'],
            'is_controller' => (SAA_Sys::getLib('validator')->checkRegex('url', $aVals['href_link']) ? 0 : 1)
        );

        $this->database()->update('menu', $aSql, 'menu_id = '. (int) $aMenu['menu_id']);

        $this->clearCache();

        return true;
    }

    /**
     * Updates the order of the menu items
     *
     * @param array $aOrder
     * @return true
     */
    public function editOrder($aOrder)
    {
        foreach($aOrder as $iOrder => $iId)
        {
            $this->database()->update('menu', array('ordering' => (int) $iOrder), 'menu_id = '. (int) $iId);
        }

        $this->clearCache();

        return true;
    }

    /**
     * Delete menu item from database
     *
     * @param int $iId
     * @param string $sPhrase
     */
    private function _delete($iId, $sPhrase)
    {
        SAA_Sys::getModel('language.phrase.process')->delete($sPhrase);

        $this->database()->delete('menu', 'menu_id = '. (int) $iId);
    }

    /**
     * Loop through children and delete them
     *
     * @param array $aItems
     */
    private function _deleteLoop($aItems)
    {
        foreach($aItems as $aItem)
        {
            $this->_delete($aItem['menu_id'], $aItem['menu_name']);

            if( isset($aItem['children']) )
            {
                $this->_deleteLoop($aItem['children']);
            }
        }
    }
}
?>
