<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Rating Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Rating_Process extends SAA_Model
{
    private $_aTable = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Adds a rating to a given item
     *
     * @param string $sType
     * @param int $iItem
     * @param int $iRating
     * @return mixed
     */
    public function add($sType, $iItem, $iRating)
    {
        if( !SAA_Sys::isUser() )
        {
            return false;
        }

        $this->setTable($sType);

        if( $iRating == 0 )
        {
            return $this->clear($iItem);
        }

        $bIsRated = $this->database()->select('rate_id')
			->from($this->_aTable['table_item'])
			->where('user_id = '. SAA_Sys::getUserId() .' AND item_id = '. (int) $iItem)
			->get('column');

        if( $bIsRated )
        {
            $this->database()->update($this->_aTable['table_item'], array(
                    'rating' => (int) $iRating,
                    'time_stamp' => SAA_TIME
                ), 'user_id = '. SAA_Sys::getUserId() .' AND item_id = '. (int) $iItem
            );
        }
        else
        {
            $iId = $this->database()->insert($this->_aTable['table_item'], array(
                    'item_id' => $iItem,
                    'user_id' => SAA_Sys::getUserId(),
                    'rating' => (int) $iRating,
                    'time_stamp' => SAA_TIME
                )
            );

            if( !$iId )
            {
                return false;
            }
        }

        return $this->updateItemRow($iItem);
    }

    /**
     * Removes the user's rating for a given item
     *
     * @param int $iItem
     * @param int $iUser
     * @return array
     */
    public function clear($iItem, $iUser = 0)
    {
        $this->database()->delete($this->_aTable['table_item'], 'user_id = '. SAA_Sys::getUserId() .' AND item_id = '. (int) $iItem);

        return $this->updateItemRow($iItem);
    }

    /**
     * Removes all the user's rating for the whole set of an item type
     *
     * @param int $iUser
     * @param int $iPage
     * @param string $sType
     * @return bool
     */
    public function deleteUser($iUser, $iPage, $sType)
    {
        $this->setTable($sType);

        $iOffset = 50;

        $aRates = $this->database()
            ->select('ti.item_id')
            ->from($this->_aTable['table_item'], 'ti')
            ->where('ti.user_id = '. (int) $iUser)
            ->limit($iOffset, $iPage)
            ->key('item_id')
            ->get('rows');

        if( !$aRates )
        {
            return false;
        }

        $aAverages = $this->database()
                ->select('COUNT(*) AS count, AVG(rating) AS average_rating, item_id')
                ->from($this->_aTable['table_item'])
                ->where('item_id IN ('. implode(', ', array_keys($aRates)).')')
                ->group('item_id')
                ->get('rows');

        foreach($aAverages as $aAverage)
        {
            $this->database()->update($this->_aTable['table'], array(
                    'total_rating' => round($aAverage['average_rating']),
                    'total_votes' => $aAverage['count']
                ), $this->_aTable['table_unique'] . ' = ' . (int) $aAverage['item_id']
            );
        }

        return true;
    }

    /**
     * Sets the table values
     * 
     * @param string $sType 
     */
    public function setTable($sType)
    {
        $this->_aTable = SAA_Sys::getModel('rating')->getDb($sType);

        return true;
    }

    /**
     * Updates the total_rating and total_votes for a given item
     *
     * @param int $iItem
     * @return array
     */
    public function updateItemRow($iItem)
    {
        $aAverage = $this->database()
                ->select('COUNT(*) AS count, AVG(rating) AS average_rating')
                ->from($this->_aTable['table_item'])
                ->where('item_id = '. (int) $iItem)
                ->get('row');

        $this->database()->update($this->_aTable['table'], array(
                'total_rating' => round($aAverage['average_rating']),
                'total_votes' => $aAverage['count']
            ), $this->_aTable['table_unique'] . ' = ' . (int) $iItem
        );

        return array(round($aAverage['average_rating']), $aAverage['count']);
    }
}
?>