<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Report Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Report_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Adds a report
     *
     * @param array $aVal
     * @return bool
     */
    public function add($aVal)
    {
        if( !isset($aVal['model_id']) || !isset($aVal['item_id']) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('report.there_was_an_error_in_processing_your_request'));
        }
        elseif( !($aItem = SAA_Sys::getModel('report')->getItem($aVal['model_id'], $aVal['item_id'])) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('report.item_cannot_be_found'));
        }

        if( !isset($aVal['reason']) || !strlen($aVal['reason']) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('report.please_provide_a_reason_for_this_report'));
        }

        if( !isset($aVal['type_id']) || count($aVal['type_id']) == 0 )
        {
            SAA_Error::add(SAA_Sys::getPhrase('report.please_select_the_infraction'));
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        $iId = $this->database()->insert('report', array(
            'item_id' => (int) $aVal['item_id'],
            'model_id' => $aVal['model_id'],
            'user_id' => SAA_Sys::getUserId(),
            'time_reported' => SAA_TIME
        ));

        if( !$iId )
        {
            return false;
        }

        $this->database()->insert('report_text', array(
            'report_id' => $iId,
            'reason' => $this->parse()->cleanTags($aVal['reason'], 'global.allowed_tags')
        ));

        $aSql = array();

        foreach($aVal['type_id'] as $sType)
        {
            $aSql[] = array(
                $iId,
                $sType
            );
        }

        $this->database()->insertMultiple('report_item', array(
                'report_id',
                'type_id'
            ), $aSql);

        return true;
    }

    /**
     * Removes report(s)
     *
     * @param array $aReports
     * @return bool
     */
    public function delete($aReports)
    {
        $sIn = '';

        foreach($aReports as $iReport)
        {
            $sIn .= (int) $iReport . ', ';
        }

        if( strlen($sIn) )
        {
            $sIn = rtrim(trim($sIn), ',');

            $this->database()->delete('report', 'report_id IN('.$sIn.')');
            $this->database()->delete('report_text', 'report_id IN('.$sIn.')');
            $this->database()->delete('report_item', 'report_id IN('.$sIn.')');
        }

        return true;
    }

    /**
     * Removes reports for a given item with provided page number
     *
     * @param string $sModel
     * @param int $iItem
     * @param int $iPage
     */
    public function deleteBy($sModel, $iItem, $iPage)
    {
        $iOffset = 50;

        $aReports = $this->database()
            ->select('report_id')
            ->from('report')
            ->where('item_id = '. (int) $iItem . ' AND model_id = "'.$this->database()->esc($sModel).'"')
            ->limit($iOffset, $iPage)
            ->key('report_id')
            ->get('rows');

        if( !$aReports )
        {
            return false;
        }

        $sIn = implode(', ', array_keys($aReports));

        $this->database()->delete('report', 'report_id IN('.$sIn.')');
        $this->database()->delete('report_text', 'report_id IN('.$sIn.')');
        $this->database()->delete('report_item', 'report_id IN('.$sIn.')');

        return true;
    }
}
?>