<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Report Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Report extends SAA_Model
{
    /**
     * Gets a report
     *
     * @param int $iId
     * @return array
     */
    public function get($iId)
    {
        $aRow = $this->database()
            ->select('r.*, rt.*, u.user_name')
            ->from('report', 'r')
            ->join('report_text', 'rt', 'r.report_id = rt.report_id')
            ->leftjoin('user', 'u', 'u.user_id = r.user_id')
            ->where('r.report_id = '. (int) $iId)
            ->get('row');

        if( !$aRow )
        {
            return false;
        }

        if( !($aItem = $this->getItem($aRow['model_id'], $aRow['item_id'])) )
        {
            return false;
        }

        $aRow['item'] = $aItem;

        $aRow['type'] = $this->database()
                ->select('*')
                ->from('report_item')
                ->where('report_id = '. (int) $iId)
                ->get('rows');

        return $aRow;
    }

    /**
     * Get the different type of model_id used
     *
     * @return array
     */
    public function getModels()
    {
        return $this->database()
            ->select('model_id')
            ->from('report')
            ->group('model_id')
            ->get('rows');
    }

    /**
     * Searches reports using the given parameters
     *
     * @param array $aParam
     * @param int $iPg
     * @param int $iPgSize
     * @return array
     */
    public function search($aParam, $iPg, $iPgSize)
    {
        $aWhere = array();

        if( isset($aParam['model_id']) && strlen($aParam['model_id']) )
        {
            $aWhere[] = 'model_id = "'.$this->database()->esc($aParam['model_id']).'" AND';
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        $aRows = $this->database()
            ->select('*')
            ->from('report')
            ->limit($iPgSize, $iPg)
            ->get('rows');

        foreach($aRows as $i => $aRow)
        {
            if( !($aItem = $this->getItem($aRow['model_id'], $aRow['item_id'])) )
            {
                unset($aRows[$i]);

                continue;
            }

            $aRows[$i]['item'] = $aItem;
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        $iCnt = $this->database()
            ->select('COUNT(*)')
            ->from('report')
            ->get('column');

        return array($iCnt, $aRows);
    }

    /**
     *
     * @param string $sModel
     * @param int $iItem
     * @return array
     */
    public function getItem($sModel, $iItem)
    {
        return $this->_getItem($sModel, $iItem);
    }

    /**
     * Callback Methods
     *
     * @param string $sMethod
     * @param array $aArgs
     * @return mixed
     */
    public function  __call($sMethod, $aArgs)
    {
        if( $sMethod == '_getItem' )
        {
            $sMethod = $aArgs[0] . '.callback';
            return SAA_Sys::getModel($sMethod)->getReport($aArgs[1]);
        }
    }
}
?>