<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Score Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Score_Process extends SAA_Model
{
    /**
     * Class Contructor
     */
    public function  __construct()
    {

    }

    /**
     * Adds a score
     *
     * @param array $aVals score data
     * @param array $aUser user data
     * @param array $aGame game data
     * @return bool
     */
    public function add($aVals, $aUser, $aGame)
    {
        //record score
        $iId = $this->database()->insert('score', array(
            'game_id' => $aGame['game_id'],
            'user_id' => $aUser['user_id'],
            'score' => $aVals['score'],
            'time_scored' => SAA_TIME
        ));

        if( !$iId )
        {
            return false;
        }

        //update total plays and total scores for the given game
        $this->database()->update('game', array(
            'total_plays' => 'total_plays + 1',
            'total_scores' => 'total_scores + 1'
        ), 'game_id = '. $aGame['game_id'], false);

        $oScore = SAA_Sys::getModel('score');

        //Check to see if scoring order is right
        /*$aVals['sortOrder'] = ($aVals['sortOrder'] == 'desc' ? 0 : 1);

        if( $aGame['scoring_order'] != $aVals['sortOrder'] )
        {
            //
        }*/

        //if the score prefix is new
        if( $aVals['scoreLabel'] != $aGame['scoring_prefix'] && strtolower($aVals['scoreLabel']) != 'score' )
        {
            $this->updatePrefix($aGame['game_id'], $aVals['scoreLabel']);

            $aGame['scoring_prefix'] = $aVals['scoreLabel'];
        }

        //Add Activity Points
        $iPoints = SAA_Sys::getSett('score.activity_points');

        if( $iPoints )
        {
            $this->database()->updateCounter('user', 'total_points', '+', $iPoints, 'user_id = '. (int) $aUser['user_id']);
        }

        $aHighScore = $oScore->getTopScore($aGame['game_id']);
        $bIsHighScore = false;

        //determine if the score qualifies as the highest score
        if( $aHighScore ) //if a high score already exists
        {
            $sFunction = 'update';

            //if the score is equal to or better than the highest score
            if( $aGame['scoring_order'] && $aHighScore['high_score'] >= $aVals['score'] || !$aGame['scoring_order'] && $aHighScore['high_score'] <= $aVals['score'] )
            {
                //if there is a tie with the highest score and score
                if( $aHighScore['high_score'] == $aVals['score'] )
                {
                    if( SAA_Sys::getSett('score.tie_priority') ) //most recent
                    {
                        $bIsHighScore = true;
                    }
                }
                else
                {
                    $bIsHighScore = true;
                }
            }
        }
        else //if score is first high score
        {
            $sFunction = 'insert';

            $bIsHighScore = true;
        }

        //if is highest score
        if( $bIsHighScore )
        {
            if( $sFunction == 'insert' )
            {
                $iHigh = $this->database()->insert('score_high', array(
                    'game_id' => $aGame['game_id'],
                    'user_id' => $aUser['user_id'],
                    'high_score' => $aVals['score'],
                    'time_scored' => SAA_TIME
                ));

                SAA_Sys::getModel('feed.process')->add('score', $aUser['user_id'], $iHigh, $aGame['game_id'], serialize(array('score_type' => 1)));
            }
            else
            {
                $this->database()->update('score_high', array(
                    'user_id' => $aUser['user_id'],
                    'high_score' => $aVals['score'],
                    'time_scored' => SAA_TIME
                ), 'game_id = '. (int) $aGame['game_id']);

                $this->database()->updateCounter('user', 'total_champs', '-', 1, 'user_id = '. (int) $aHighScore['user_id']);

                SAA_Sys::getModel('feed.process')->add('score', $aUser['user_id'], $aHighScore['score_id'], $aGame['game_id'], serialize(array('score_type' => 0, 'old_user_id' => $aHighScore['user_id'])));
            }

            $this->database()->updateCounter('user', 'total_champs', '+', 1, 'user_id = '. (int) $aUser['user_id']);
        }


        $aUserHighScore = $oScore->getUserHighScore($aGame['game_id'], $aUser['user_id']);
        $bIsHighScore = false;

        //determine if score qualifies as the user's highest score
        if( $aUserHighScore )
        {
            if( $aGame['scoring_order'] && $aUserHighScore['user_score'] >= $aVals['score'] || !$aGame['scoring_order'] && $aUserHighScore['user_score'] <= $aVals['score'] )
            {
                $bIsHighScore = true;
            }
        }
        else
        {
            $bIsHighScore = true;
        }

        //if is user's highest score
        if( $bIsHighScore )
        {
            if( $aUserHighScore )
            {
                $this->database()->update('score_user', array(
                    'user_score' => $aVals['score'],
                    'time_scored' => SAA_TIME
                ), 'score_id = '. (int) $aUserHighScore['score_id']);
            }
            else
            {
                $this->database()->insert('score_user', array(
                    'game_id' => $aGame['game_id'],
                    'user_id' => $aUser['user_id'],
                    'user_score' => $aVals['score'],
                    'time_scored' => SAA_TIME
                ));
            }
        }

        return true;
    }

    /**
     * Deletes a given user's scores from a given set of games (defined by page and offset)
     *
     * @param int $iUser
     * @param int $iPage
     * @return bool
     */
    public function deleteUser($iUser, $iPage)
    {
        $oFeed = SAA_Sys::getModel('feed.process');

        $iOffset = 50;

        $aScores = $this->database()
            ->select('COUNT(*) AS minus, s.game_id, sh.score_id AS high_scorer, g.scoring_order')
            ->from('score', 's')
            ->join('game', 'g', 'g.game_id = s.game_id')
            ->leftjoin('score_high', 'sh', 'sh.game_id = s.game_id AND sh.user_id = s.user_id')
            ->where('s.user_id = '. (int) $iUser)
            ->group('s.game_id')
            ->limit($iOffset, $iPage)
            ->get('rows');

        if( !$aScores )
        {
            return false;
        }

        $bOrder = (SAA_Sys::getSett('score.tie_priority') == 'Most Recent' ? true : false);

        foreach($aScores as $aScore)
        {
            if( $aScore['high_scorer'] )
            {
                $aOrder = array();

                $aOrder['score'] = ($aScore['scoring_order'] ? 'ASC' : 'DESC');
                $aOrder['time_scored'] = ($bOrder ? 'DESC' : 'ASC');

                $aHighest = $this->database()
                    ->select('*')
                    ->from('score')
                    ->where('game_id = '. (int) $aScore['game_id']. ' AND user_id != '. $iUser)
                    ->order($aOrder)
                    ->limit(2)
                    ->get('rows');

                $this->database()->delete('score_high', 'game_id = '. (int) $aScore['game_id'] . ' AND user_id = '. $iUser);

                $aBest = $aHighest[0];

                $iId = $this->database()->insert('score_high', array(
                    'game_id' => $aScore['game_id'],
                    'user_id' => $aBest['user_id'],
                    'high_score' => $aBest['score'],
                    'time_scored' => $aBest['time_scored']
                ));

                $this->database()->updateCounter('user', 'total_champs', '+', 1, 'user_id = '. (int) $aBest['user_id']);

                if( isset($aHighest[1]) )
                {
                    $mFeed = serialize(array('score_type' => 0, 'old_user_id' => $aHighest[1]['user_id']));
                }
                else
                {
                    $mFeed = serialize(array('score_type' => 1));
                }

                $oFeed->add('score', $aBest['user_id'], $iId, $aScore['game_id'], $mFeed, $aBest['time_scored']);
            }

            $this->database()->updateCounter('game', 'total_scores', '-', $aScore['minus'], 'game_id = '. (int) $aScore['game_id']);
        }

        return true;
    }

    /**
     * Removes scores and related feeds using various criteria
     *
     * @param array $aVal
     * @return bool
     */
    public function reset($aVal)
    {
        $aWhere = array();

        if( $aVal['from'] )
        {
            $aWhere[] = 'time_scored >= '. strtotime($aVal['from']);
        }

        if( $aVal['to'] )
        {
            $aWhere[] = 'time_scored <= '. strtotime($aVal['to']);
        }

        if( $aVal['users'] )
        {
            $sUsers = '';
            $aUsers = explode(',', $aVal['users']);

            foreach($aUsers as $sUser)
            {
                $sUsers .= '"'.$this->database()->esc(trim($sUser)).'", ';
            }

            $sUsers = rtrim(trim($sUsers), ',');

            if( $sUsers )
            {
                $sIn = '';

                $aUsers = $this->database()
                    ->select('user_id')
                    ->from('user')
                    ->where('user_name IN ('.$sUsers.')')
                    ->get('rows');

                foreach($aUsers as $aUser)
                {
                    $sIn .= $aUser['user_id'].', ';
                }

                $sIn = rtrim(trim($sIn), ',');

                if( $sIn )
                {
                    $aWhere[] = 'user_id IN ('.$sIn.')';
                }
            }
        }

        if( isset($aVal['game_id']) )
        {
            $aWhere[] = 'game_id = '. (int) $aVal['game_id'];
        }

        if( $aWhere )
        {
            $this->database()->delete('score', implode(' AND ', $aWhere));

            SAA_Sys::getModel('feed.process')->deleteWhere('score', $aWhere, array('time_scored', 'game_id'), array('time_stamp', 'parent_item_id'));
        }
        else
        {
            $this->database()->truncateTable('score');
        }

        return true;
    }

    /**
     * Reset the scores, and related data for a given game
     * 
     * @param int $iGame
     * @return bool 
     */
    public function resetGame($iGame)
    {
        $this->database()->delete('score', 'game_id = '. (int) $iGame);
        $this->database()->delete('score_user', 'game_id = '. (int) $iGame);

        $aHigh = SAA_Sys::getModel('score')->getTopScore($iGame);

        $this->database()->delete('score_high', 'game_id = '. (int) $iGame);

        $this->database()->update('game', array('total_scores' => 0), 'game_id = '. (int) $iGame);

        if( $aHigh )
        {
            $this->database()->updateCounter('user', 'total_champs', '-', 1, 'user_id = '. (int) $aHigh['user_id']);
        }

        SAA_Sys::getModel('feed.process')->deleteByParent('score', $iGame);

        return false;
    }

    /**
     * Rebuilds the high score table
     *
     * @param int $iPg
     * @param int $iPgSize
     */
    public function rebuildHigh($iPg, $iPgSize)
    {
        $oFeed = SAA_Sys::getModel('feed.process');

        $aGames = $this->database()
            ->select('game_id, scoring_order')
            ->from('game')
            ->limit($iPgSize, $iPg)
            ->get('rows');

        $bOrder = (SAA_Sys::getSett('score.tie_priority') == 'Most Recent' ? true : false);

        foreach($aGames as $aGame)
        {
            $aOrder = array();

            $aOrder['score'] = ($aGame['scoring_order'] ? 'ASC' : 'DESC');
            $aOrder['time_scored'] = ($bOrder ? 'DESC' : 'ASC');

            $aScores = $this->database()
                    ->select('*')
                    ->from('score')
                    ->where('game_id = '. (int) $aGame['game_id'])
                    ->order($aOrder)
                    ->limit(2)
                    ->get('rows');

            if( !$aScores )
            {
                continue;
            }

            $aBest = $aScores[0];

            $iId = $this->database()->insert('score_high', array(
                'game_id' => $aGame['game_id'],
                'user_id' => $aBest['user_id'],
                'high_score' => $aBest['score'],
                'time_scored' => $aBest['time_scored']
            ));

            $this->database()->updateCounter('user', 'total_champs', '+', 1, 'user_id = '. (int) $aBest['user_id']);

            if( isset($aScores[1]) )
            {
                $mFeed = serialize(array('score_type' => 0, 'old_user_id' => $aScores[1]['user_id']));
            }
            else
            {
                $mFeed = serialize(array('score_type' => 1));
            }

            $oFeed->add('score', $aBest['user_id'], $iId, $aGame['game_id'], $mFeed, $aBest['time_scored']);
        }
    }

    /**
     * Rebuilds the user score table
     *
     * @param int $iGamePg
     * @param int $iPg
     * @param int $iPgSize
     */
    public function rebuildUser($iGamePg, $iPg, $iPgSize)
    {
        $iAdded = 0;

        $aGame = $this->database()
            ->select('game_id, scoring_order')
            ->from('game')
            ->limit(1, $iGamePg)
            ->get('row');

        if( !$aGame )
        {
            return false;
        }

        $aUsers = $this->database()
            ->select('u.user_id')
            ->from('user', 'u')
            ->join('score', 's', 's.user_id = u.user_id AND s.game_id = '. (int) $aGame['game_id'])
            ->group('u.user_id')
            ->limit($iPgSize, $iPg)
            ->get('rows');

        if( !$aUsers )
        {
            return 'game';
        }

        foreach($aUsers as $aUser)
        {
            if( $aGame['scoring_order'] )
            {
                $this->database()->select('MIN(score)');
            }
            else
            {
                $this->database()->select('MAX(score)');
            }

            $iScore = $this->database()
                    ->from('score')
                    ->where('game_id = '. (int) $aGame['game_id'] .' AND user_id = '. $aUser['user_id'])
                    ->get('column');

            if( SAA_Sys::getSett('score.tie_priority') == 'Most Recent' )
            {
                $this->database()->order('time_scored', 'DESC');
            }
            else
            {
                $this->database()->order('time_scored', 'ASC');
            }

            $aScore = $this->database()
                    ->select('*')
                    ->from('score')
                    ->where('game_id = '. (int) $aGame['game_id'] .' AND score = '. (int) $iScore .' AND user_id = '. (int) $aUser['user_id'])
                    ->limit(1)
                    ->get('row');

            if( !$aScore )
            {
                continue;
            }

            $iId = $this->database()->insert('score_user', array(
                'game_id' => $aGame['game_id'],
                'user_id' => $aScore['user_id'],
                'user_score' => $aScore['score'],
                'time_scored' => $aScore['time_scored']
            ));

            $iAdded++;
        }

        return $iAdded;
    }

    /**
     * Truncates the score_high and score_user table, remove score feeds, as well as set user table's total_champs to zero
     */
    public function truncateDependent()
    {
        $this->database()->truncateTable('score_high');
        $this->database()->truncateTable('score_user');

        SAA_Sys::getModel('feed.process')->deleteByWhere('score', null);

        $this->database()->update('user', array('total_champs' => 0));

        return true;
    }

    /**
     * Updates a scoring prefix for a given game
     *
     * @param int $iGame
     * @param string $sPrefix
     * @return bool
     */
    public function updatePrefix($iGame, $sPrefix)
    {
        return $this->database()->update('game', array('scoring_prefix' => $sPrefix), 'game_id = '. (int) $iGame);
    }
}
?>