<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Score Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Score extends SAA_Model
{
    /**
     * Get the recent scores
     *
     * @param array $aGame game array
     * @param int $iLimit
     * @return array
     */
    public function getRecentScores($aGame, $iLimit)
    {
        return $this->database()
                ->select('s.score, '.SAA_Sys::getModel('user')->getUserFields('u'))
                ->from('score', 's')
                ->join('user', 'u', 'u.user_id = s.user_id')
                ->where('s.game_id = '. (int) $aGame['game_id'])
                ->order('time_scored', 'DESC')
                ->limit($iLimit)
                ->get('rows');
    }

    /**
     * Gets the scoring prefix
     *
     * @param int $iGame game id
     * @return string
     */
    public function getScoringPrefix($iGame)
    {
        return $this->database()
                ->select('scoring_prefix')
                ->from('game')
                ->where('game_id = '. (int) $iGame)
                ->get('column');
    }

    /**
     * Gets the top score for the given game
     *
     * @param int $iGame
     * @return array
     */
    public function getTopScore($iGame)
    {
        return $this->database()
            ->select('*')
            ->from('score_high')
            ->where('game_id = ' . (int) $iGame)
            ->get('row');
    }

    /**
     * Get the top scores
     *
     * @param array $aGame game array
     * @param int $iLimit
     * @return array
     */
    public function getTopScores($aGame, $iLimit)
    {
        if( SAA_Sys::getSett('score.per_user') == 'One Score' )
        {
            $this->database()->select('MAX(s.score) as score,')
                ->group('s.user_id');
        }
        else
        {
            $this->database()->select('s.score,');
        }

        return $this->database()
                ->select(SAA_Sys::getModel('user')->getUserFields('u'))
                ->from('score', 's')
                ->join('user', 'u', 'u.user_id = s.user_id')
                ->where('s.game_id = '. (int) $aGame['game_id'])
                ->order('score', ($aGame['scoring_order'] ? 'ASC' : 'DESC'))
                ->limit($iLimit)
                ->get('rows');
    }

    /**
     * Gets the user's top score for a given game
     *
     * @param int $iGame
     * @param int $iUser user id
     * @return array
     */
    public function getUserHighScore($iGame, $iUser)
    {
        return $this->database()
            ->select('*')
            ->from('score_user')
            ->where('user_id = '. (int) $iUser . ' AND game_id = '. (int) $iGame)
            ->get('row');
    }

    /**
     * Get the user's top scores
     *
     * @param array $aGame game array
     * @param int $iUser user id
     * @param int $iLimit
     * @return array
     */
    public function getUserScores($aGame, $iUser, $iLimit)
    {
        return $this->database()
                ->select('s.score')
                ->from('score', 's')
                ->where('s.user_id = '.(int) $iUser.' AND s.game_id = '. (int) $aGame['game_id'])
                ->order('score', ($aGame['scoring_order'] ? 'ASC' : 'DESC'))
                ->limit($iLimit)
                ->get('rows');
    }

    /**
     * Gets the total number of scores
     *
     * @return string
     */
    public function totalScores()
    {
        return $this->database()
            ->select('COUNT(*)')
            ->from('score')
            ->get('column');
    }
}
?>