<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Setting Controller Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Setting_Controller extends SAA_Model
{
    public function  __construct()
    {

    }

    /**
     * Get the list of settings used by a given controller
     *
     * @param string $sController
     * @return array
     */
    public function getBy($sController)
    {
        return $this->database()
                ->select('s.setting_id, s.setting_title, s.var_name, s.group_id, sc.ordering')
                ->from('setting_controller', 'sc')
                ->join('setting', 's', 's.setting_id = sc.setting_id')
                ->where('sc.controller_id = "'.$this->database()->esc($sController).'"')
                ->order('sc.ordering', 'ASC')
                ->get('rows');
    }

    /*
     * Get all controllers used by settings
     *
     * @return array
     */
    public function getAll()
    {
        $sCache = 'settings_controller';

        if( !($aControllers = $this->cache()->get($sCache)) )
        {
            $aControllers = SAA_Sys::getModel('admin.controller')->getControllers();

            $aUsed = $this->database()
                ->select('*')
                ->from('setting_controller')
                ->group('controller_id')
                ->key('controller_id')
                ->get('rows');

            foreach($aControllers as $i => $sController)
            {
                if( !isset($aUsed[$sController]) )
                {
                    unset($aControllers[$i]);
                }
            }
            
            $this->cache()->create($sCache, $aControllers);
        }

        return $aControllers;
    }

    /*
     * Check controller to see if setting already exists
     *
     * @param string sController controller
     * @param int iSetting setting id
     *
     * @return bool
     */
    public function hasSetting($sController, $iSetting)
    {
        $aRow = $this->database()
            ->select('*')
            ->from('setting_controller')
            ->where('setting_id = '. (int) $iSetting . ' AND controller_id = "'.$this->database()->esc($sController).'"')
            ->get('row');

        return ($aRow ? true : false);
    }

    /*
     * Checks to see if a controller is valid
     *
     * @param string sController controller
     *
     * @return bool
     */
    public function is($sController)
    {
        $aControllers = $this->getAll();

        return (in_array($sController, $aControllers) ? true : false);
    }
}
?>
