<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Setting Controller Process Class
 * @license GoLevel LLC
 */

Class SAA_Model_Setting_Controller_Process extends SAA_Model
{
    /**
     * Class Constructors
     */
    public function  __construct()
    {

    }

    /*
     * Add setting to controller
     *
     * @param array aVal
     *
     * @return bool
     */
    public function add($aVal)
    {
        $aSetting = SAA_Sys::getModel('setting')->get($aVal['setting_id'], false);

        if( !$aSetting )
        {
            SAA_Error::add('Setting you are trying to add to the selected controller cannot be found.');
        }
        elseif( SAA_Sys::getModel('setting.controller')->hasSetting($aVal['controller_id'], $aSetting['setting_id']) )
        {
            SAA_Error::add('Setting is already exists for the selected controller.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $aSql = array(
            'setting_id' => (int) $aSetting['setting_id'],
            'controller_id' => $aVal['controller_id'],
            'ordering' => $this->database()->getExtremeVal('setting_controller', 'ordering', 'controller_id = "'.$this->database()->esc($aVal['controller_id']).'"') + 1
        );

        $this->database()->insert('setting_controller', $aSql);

        $this->clearCache();

        return true;
    }

    /*
     * Removes caching data related to setting controllers
     */
    public function clearCache()
    {
        $this->cache()->remove('settings_controller');

        return true;
    }

    /*
     * Removes the setting from controller
     */
    public function delete($iSetting, $sController)
    {
        $this->database()->delete('setting_controller', 'setting_id = '. (int) $iSetting . ' AND controller_id = "'.$this->database()->esc($sController).'"');

        return true;
    }

    /**
     * Sorts the ordering of the setting
     *
     * @param array $aVal
     * @param string $sController
     *
     * @return bool
     */
    public function saveOrder($aVal, $sController)
    {
        foreach($aVal['setting'] as $iSetting => $iOrder)
        {
            $this->database()->update('setting_controller', array('ordering' => $iOrder), 'setting_id = '. (int) $iSetting . ' AND controller_id = "'.$this->database()->esc($sController).'"');
        }

        $this->clearCache();

        return true;
    }
}
?>
