<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Setting Process
 * @license GoLevel LLC
 */

Class SAA_Model_Setting_Process extends SAA_Model
{
    /**
     * Class Constructors
     */
    public function  __construct()
    {

    }

    /*
     * Add a setting
     *
     * @param array aVal
     * @return bool
     */
    public function add($aVal)
    {
        if( in_array($aVal['type_id'], array('array1', 'select')) )
        {
            $aValues = $aVal['value'];

            foreach($aValues as $i => $sString)
            {
                if( !strlen($sString) )
                {
                    unset($aValues[$i]);
                }
            }

            if( $aVal['type_id'] == 'array1' )
            {
                $aVal['value'] = serialize(array('type' => 1, 'values' => $aValues));

                $aVal['type_id'] = 'array';
            }
            else
            {
                if( !count($aValues) )
                {
                    return SAA_Error::add('Options were not filled.');
                }

                $aVal['value'] = serialize(array(
                    'selected' => $aValues[0],
                    'options' => array_combine($aValues, $aValues)
                ));
            }
        }
        elseif( $aVal['type_id'] == 'array2' )
        {
            $aValues = $aVal['value'];

            $iItems = count($aValues['key']);

            for($i=0; $i<$iItems; $i++)
            {
                if( !strlen($aValues['key'][$i]) && !strlen($aValues['value'][$i]) )
                {
                    unset($aValues['key'][$i], $aValues['value'][$i]);
                }
            }

            $aVal['value'] = serialize(array(
                'type' => 2,
                'values' => array_combine($aValues['key'], $aValues['value'])
            ));

            $aVal['type_id'] = 'array';
        }
        elseif( $aVal['type_id'] == 'int' )
        {
            $aVal['value'] = (int) $aVal['value'];
        }

        $aVal['var_name'] = strtolower($aVal['var_name']);

        if( !strlen(trim($aVal['var_name'])) )
        {
            SAA_Error::add('Please provide a var name.');
        }

        if( ($aCheck = SAA_Sys::getModel('setting')->get($aVal['group_id'] . '.' . $aVal['var_name'], false)) )
        {
            SAA_Error::add('Setting already exists.');
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        $aSql = array(
            'type_id' => $aVal['type_id'],
            'group_id' => $aVal['group_id'],
            'setting_title' => $aVal['setting_title'],
            'setting_explain' => $aVal['setting_explain'],
            'var_name' => $aVal['var_name'],
            'value_current' => $aVal['value'],
            'value_original' => $aVal['value'],
            'ordering' => $this->database()->getExtremeVal('setting', 'ordering', 'group_id = "'.$this->database()->esc($aVal['group_id']).'"') + 1
        );

        $iId = $this->database()->insert('setting', $aSql);

        if( !$iId )
        {
            return false;
        }

        if( strlen(trim($aVal['controller'])) )
        {
            SAA_Sys::getModel('setting.controller.process')->add(array(
                'setting_id' => $aVal['group_id'] .'.'. $aVal['var_name'],
                'controller_id' => $aVal['controller']
            ));
        }

        $this->cache()->removeLike('settings');

        return true;
    }

    /*
     * Updates all settings
     *
     * @param array aVals
     * @return bool
     */
    public function edit($aVals)
    {
        $aSettings = SAA_Sys::getModel('setting')->getAll();

        foreach($aVals as $iSetting => $mValue)
	{
            $mCurrent = $aSettings[$iSetting]['value_current'];

            if( is_array($mCurrent) )
            {
                if( isset($mCurrent['selected']) )
                {
                    $mValue = array(
                        'selected' => (string) $mValue,
                        'options' => $mCurrent['options']
                    );
                }
                else
                {
                    if( $mCurrent['type'] == 2 )
                    {
                        $iItems = count($mValue['key']);

                        for($i=0; $i<$iItems; $i++)
                        {
                            if( !strlen($mValue['key'][$i]) && !strlen($mValue['value'][$i]) )
                            {
                                unset($mValue['key'][$i], $mValue['value'][$i]);
                            }
                        }

                        $mValue = array(
                            'type' => 2,
                            'values' => array_combine($mValue['key'], $mValue['value'])
                        );
                    }
                    else
                    {
                        foreach($mValue as $i => $sString)
                        {
                            if( !strlen($sString) )
                            {
                                unset($mValue[$i]);
                            }
                        }

                        $mValue = array(
                            'type' => 1,
                            'values' => array_values($mValue)
                        );
                    }
                }

                $mValue = serialize($mValue);
            }
            elseif( $aSettings[$iSetting]['type_id'] == 'bool' )
            {
                $mValue = ((bool) $mValue ? 'true' : 'false');
            }

            $aSql = array(
                'value_current' => $mValue
            );

            $this->database()->update('setting', $aSql, 'setting_id = '. (int) $iSetting);
        }

        $this->cache()->removeLike('settings');

        return true;
    }

    /*
     * Sorts the ordering of the setting
     */
    public function saveOrder($aVal)
    {
        foreach($aVal['setting'] as $iSetting => $iOrder)
        {
            $this->database()->update('setting', array('ordering' => $iOrder), 'setting_id = '. (int) $iSetting);
        }

        $this->cache()->removeLike('settings');

        return true;
    }
}
?>
