<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Setting Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Setting extends SAA_Model
{
    private $_bIsByController = false;

    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /*
     * Get a given setting by id or group_id.var_name
     *
     * @param string sSetting
     * @param bool bById if true get by id, or group_id.var_name
     * 
     * @return mixed
     */
    public function get($sSetting, $bById = true)
    {
        if( $bById )
        {
            $this->database()->where('setting_id = '. (int) $sSetting);
        }
        else
        {
            list($sGroup, $sVar) = explode('.', $sSetting);

            $this->database()->where('var_name = "'.$this->database()->esc($sVar).'" AND group_id = "'.$this->database()->esc($sGroup).'"');
        }

        $aSetting = $this->database()
            ->select('*')
            ->from('setting')
            ->get('row');

        if( !$aSetting )
        {
            return false;
        }

        $aSetting['value_current'] =  $this->getValue($aSetting['type_id'], $aSetting['value_current']);

        return $aSetting;
    }

    /*
     * Get all settings
     *
     * @param bool bSimple if true get small array, or full array
     *
     * @return array
     */
    public function getAll($bSimple = false)
    {
        if( $this->_bIsByController )
        {
            $this->database()->order('sc.ordering');
        }
        else
        {
            $this->database()->order('s.ordering');
        }

        $aSettings = $this->database()
            ->select('s.*')
            ->from('setting', 's')
            ->get('rows');

        $this->_bIsByController = false;

        if( $bSimple )
        {
            $aSimple = array();

            foreach($aSettings as $aSetting)
            {
                $aSimple[$aSetting['group_id'].'.'.$aSetting['var_name']] = $this->getValue($aSetting['type_id'], $aSetting['value_current']);

                if( $aSetting['type_id'] == 'array' )
                {
                    $sVar = $aSetting['group_id'].'.'.$aSetting['var_name'];
                    $aOption = $aSimple[$sVar];

                    $aSimple[$sVar] = $aOption['values'];
                }
                elseif( $aSetting['type_id'] == 'select' )
                {
                    $sVar = $aSetting['group_id'].'.'.$aSetting['var_name'];
                    $aOption = $aSimple[$sVar];

                    $aSimple[$sVar] = $aOption['options'][$aOption['selected']];
                }
            }

            return $aSimple;
        }
        else
        {
            $aTemps = $aSettings;
            $aSettings = array();

            foreach($aTemps as $aTemp)
            {
                $aSettings[$aTemp['setting_id']] = $aTemp;
            }
        }

        foreach($aSettings as $iSetting => $aSetting)
        {
            $aSettings[$iSetting]['value_current'] = $this->getValue($aSetting['type_id'], $aSetting['value_current']);
        }

        return $aSettings;
    }

    /*
     * Get settings by controller
     *
     * @param string sController controller
     *
     * @return array with 0 key being the groups and 1 key being the settings
     */
    public function getAllByController($sController)
    {
        $this->_bIsByController = true;

        $this->database()->join('setting_controller', 'sc', 'sc.setting_id = s.setting_id AND sc.controller_id = "'.$this->database()->esc($sController).'"');

        $aJs = array();
        $aSettings = $this->getAll();

        foreach($aSettings as $aSetting)
        {
            $aJs[$aSetting['group_id']] = $aSetting['group_id'];
        }

        return array($aJs, $aSettings);
    }

    /*
     * Get settings by group
     *
     * @param array aGroup setting group
     *
     * @return array
     */
    public function getAllByGroup($aGroup)
    {
        $this->database()->where('s.group_id = "'.$this->database()->esc($aGroup['group_id']).'"');

        return $this->getAll();
    }

    /*
     * Convert a value to cast type
     *
     * @param string sType type
     * @param string sRaw string to convert
     *
     * @return mixed
     */
    public function getValue($sType, $sRaw)
    {
        $mValue = '';

        switch($sType)
        {
            case 'array':
            case 'select':
                $mValue = unserialize($sRaw);
                break;
            case 'bool':
                $mValue = ($sRaw == 'false' ? false : true);
                break;
            case 'int':
                $mValue = (int) $sRaw;
                break;
            case 'string_long':
            default:
                $mValue = (string) $sRaw;
        }

        return $mValue;
    }
}
?>
