<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Theme Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Theme_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Adds a theme
     *
     * @param array $aVal
     * @return mixed
     */
    public function add($aVal)
    {
        if( !strlen($aVal['title']) )
        {
            SAA_Error::add('Provide a title.');
        }

        $aFolders = SAA_Sys::getModel('theme')->findThemes();

        if( !isset($aFolders[$aVal['folder_loc']]) )
        {
            SAA_Error::add('The folder no longer exists.');
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        $aSql = array(
            'title' => $this->parse()->removeTags($aVal['title']),
            'is_default' => 0,
            'folder_loc' => $aVal['folder_loc'],
            'is_active' => $aVal['is_active']
        );

        $iId = $this->database()->insert('theme', $aSql);

        if( !$iId )
        {
            return false;
        }

        $this->clearCache();

        return $iId;
    }

    /**
     * Removes all theme cache
     */
    public function clearCache()
    {
        $this->cache()->remove(array(
            'themes'
        ));
    }

    /**
     * Removes a theme
     *
     * @param int $iId
     * @return bool
     */
    public function delete($iId)
    {
        $this->database()->delete('theme', 'theme_id = '. (int) $iId);

        $this->clearCache();

        return true;
    }

    /**
     * Updates a theme row
     * 
     * @param array $aVal
     * @param array $aTheme
     */
    public function edit($aVal, $aTheme)
    {
        $aSql = array(
            'title' => $this->parse()->removeTags($aVal['title']),
            'is_default' => 0,
            'folder_loc' => $aVal['folder_loc'],
            'is_active' => $aVal['is_active']
        );

        $this->database()->update('theme', $aSql, 'theme_id = '. (int) $aTheme['theme_id']);

        $this->clearCache();

        return true;
    }

    /**
     * Restores the default theme
     */
    public function restore()
    {
        $aThemes = SAA_Sys::getModel('theme')->getAll(false);

        $mId = false;

        foreach($aThemes as $iTheme => $aTheme)
        {
            if( $aTheme['folder_loc'] == 'default' )
            {
                $mId = $iTheme;
            }
        }

        if( $mId === false )
        {
            $mId = $this->database()->insert('theme', array(
                'title' => 'Default',
                'is_default' => 0,
                'folder_loc' => 'default',
                'is_active' => 1
            ));
        }

        $this->setDefault($mId);
    }

    /**
     * Makes the given theme default
     *
     * @param int $iId
     * @return bool
     */
    public function setDefault($iId)
    {
        $this->database()->update('theme', array('is_default' => 0), 'theme_id != '. (int) $iId);
        $this->database()->update('theme', array('is_default' => 1), 'theme_id = '. (int) $iId);

        $this->clearCache();

        return true;
    }

    /**
     * Turns a theme on/off
     *
     * @param int $iId
     * @param int $iToggle
     * @return bool
     */
    public function toggle($iId, $iToggle)
    {
        $this->database()->update('theme', array('is_active' => $iToggle), 'theme_id = '. (int) $iId);

        $this->clearCache();

        return true;
    }
}
?>
