<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name User Process Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_User_Process extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Activates user(s)
     *
     * @param mixed $mUser
     * @return true
     */
    public function activate($mUser)
    {
        if( !is_array($mUser) )
        {
            $mUser = (array) $mUser;
        }

        foreach($mUser as $iUser)
        {
            $this->database()->update('user', array('is_verified' => 1), 'user_id = '. (int) $iUser);
        }

        return true;
    }

    /**
     * Actives a user
     *
     * @param string $sEmail
     * @param string $sHash
     * @return bool
     */
    public function activeAccount($sEmail, $sHash)
    {
        $sEmail = $this->splitString($sEmail);
        $sHash = $this->splitString($sHash);

        $aRow = $this->database()
            ->select('*')
            ->from('user')
            ->where('email = "' . $this->database()->esc($sEmail) . '"')
            ->get('row');

        if( !$aRow )
        {
            SAA_Error::add(SAA_Sys::getPhrase('user.the_account_you_are_looking_for_does_not_exist'));
        }
        elseif( $aRow['is_verified'] )
        {
            SAA_Error::add(SAA_Sys::getPhrase('user.your_account_has_already_been_verified'));
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        $this->activate($aRow['user_id']);

        return SAA_Sys::getModel('user.session')->createSession($aRow);
    }

    /**
     * Insert user using facebook information
     */
    public function addFromFacebook()
    {
        $aFacebook = SAA_Sys::getLib('api.facebook')->getUser();

        if( !$aFacebook )
        {
            return false;
        }

        $sSalt = $this->parse()->salt(5);
        $sPassword = $this->parse()->salt(10);

        return $this->database()->insert('user', array(
            'facebook_id' => $aFacebook['id'],
            'user_name' => $aFacebook['name'],
            'email' => $aFacebook['email'],
            'password' => $sPassword,
            'salt' => $sSalt,
            'gender' => ($aFacebook['gender'] == 'female' ? 1 : 0),
            'date_added' => SAA_TIME,
            'last_login' => SAA_TIME,
            'lang_id' => SAA_Sys::getModel('language')->getDefaultId(),
            'theme_id' => SAA_Sys::getModel('theme')->getDefaultId(),
        ));
    }

    /**
     * Deactivates user(s)
     *
     * @param mixed $mUser
     * @return true
     */
    public function deactivate($mUser)
    {
        if( !is_array($mUser) )
        {
            $mUser = (array) $mUser;
        }

        foreach($mUser as $iUser)
        {
            $this->database()->update('user', array('is_verified' => 0), 'user_id = '. (int) $iUser);
        }

        return true;
    }

    /**
     * Deletes a given set of user(s)
     * 
     * @param array $aVars
     */
    public function delete($aVars)
    {
        $iUser = $aVars['current'];

        if( $aVars['stage'] == 'initial' )
        {
            $this->database()->delete('score_user', 'user_id = '. (int) $iUser);
            $this->database()->delete('user_award', 'user_id = '. (int) $iUser);
            $this->database()->delete('user_forgotten', 'user_id = '. (int) $iUser);
            $this->database()->delete('user_session', 'user_id = '. (int) $iUser);
            $this->database()->delete('user_session_force', 'user_id = '. (int) $iUser);

            return array(
                'stage' => 'score',
                'page' => 1,
                'current' => $iUser,
                'remaining' => $aVars['remaining']
            );
        }
        elseif( $aVars['stage'] == 'score' )
        {
            $iPage = $aVars['page'];

            $oFeed = SAA_Sys::getModel('feed.process');

            if( SAA_Sys::getModel('score.process')->deleteUser($iUser, $iPage) )
            {
                return array(
                    'stage' => 'score',
                    'page' => ($iPage + 1),
                    'current' => $iUser,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                $this->database()->delete('score', 'user_id = '. (int) $iUser);

                SAA_Sys::getModel('feed.process')->deleteByUser($iUser);

                return array(
                    'stage' => 'rate',
                    'page' => 1,
                    'current' => $iUser,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'rate' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('rating.process')->deleteUser($iUser, $iPage, 'game') )
            {
                return array(
                    'stage' => 'rate',
                    'page' => ($iPage + 1),
                    'current' => $iUser,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                $this->database()->delete('game_rate', 'user_id = '. (int) $iUser);

                return array(
                    'stage' => 'comment',
                    'page' => 1,
                    'current' => $iUser,
                    'remaining' => $aVars['remaining']
                );
            }
        }
        elseif( $aVars['stage'] == 'comment' )
        {
            $iPage = $aVars['page'];

            if( SAA_Sys::getModel('comment.process')->deleteUser($iUser, $iPage) )
            {
                return array(
                    'stage' => 'comment',
                    'page' => ($iPage + 1),
                    'current' => $iUser,
                    'remaining' => $aVars['remaining']
                );
            }
            else
            {
                if( $aVars['remaining'] )
                {
                    return array(
                        'stage' => 'initial',
                        'current' => array_shift($aVars['remaining']),
                        'remaining' => $aVars['remaining']
                    );
                }
                else
                {
                    $this->database()->delete('user', 'user_id = '. (int) $iUser);

                    return array(
                        'stage' => 'finished'
                    );
                }
            }
        }

        return false;
    }

    /**
     * Insert user traditional
     *
     * @param array $aVal
     * @return mixed
     */
    public function addFromRegister($aVal)
    {
        $aVal['email'] = trim($aVal['email']);

        if( !strlen($aVal['email']) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.please_provide_an_email_address'));
        }
        elseif( !SAA_Sys::getLib('validator')->checkRegex('email', $aVal['email']) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.please_provide_a_valid_email_address'));
        }
        elseif( ($aExists = SAA_Sys::getModel('user')->get($aVal['email'], 'email')) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.email_address_is_already_in_use'));
        }

        $aVal['name'] = trim($aVal['name']);
        $aVal['name'] = $this->parse()->removeTags(html_entity_decode($aVal['name']), false);

        if( !strlen($aVal['name']) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.please_provide_a_user_name'));
        }
        elseif( $this->parse()->cleanChrs($aVal['name'], SAA_Sys::getSett('user.banned_chars')) != $aVal['name'] )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.user_name_must_consist_of_letters_numbers_or_the_character'));
        }
        elseif( ($aExists = SAA_Sys::getModel('user')->get($aVal['name'], 'user_name')) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.user_name_is_already_in_use'));
        }

        if( !strlen($aVal['password']) )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.please_provide_a_password'));
        }
        elseif( strlen($aVal['password']) < 5 )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.password_must_be_greater_than_5_characters'));
        }
        elseif( $aVal['password'] != $aVal['confirm'] )
        {
            SAA_Error::add(SAA_Sys::getPhrase('create.password_does_not_match_confirmation'));
        }

        $aSecurity = SAA_Sys::getSett('user.sign_up_questions');

        if( $aSecurity )
        {
            if( !isset($aVal['security']) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('create.security_question_has_not_been_completed'));
            }
            elseif( !isset($aSecurity[ $aVal['security']['key'] ]) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('create.there_is_an_error_answering_the_security_question'));
            }
            else
            {
                $sAnswer = trim($aVal['security']['value']);
                $sAnswers = $aSecurity[ $aVal['security']['key'] ];

                if( strpos($sAnswers, $sAnswer) === false )
                {
                    SAA_Error::add(SAA_Sys::getPhrase('create.the_security_question_has_been_answered_incorrectly'));
                }
            }
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        $sSalt = $this->parse()->salt(5);

		$aTheme = SAA_Sys::getLib('theme')->getDefault();

        $aSql = array(
            'user_name' => $this->parse()->rareToEntities($aVal['name'], 255),
            'email' => $aVal['email'],
            'password' => md5(md5($aVal['password']) . $sSalt),
            'salt' => $sSalt,
            'date_added' => SAA_TIME,
            'lang_id' => SAA_Sys::getLib('language')->getDefault(),
            'theme_id' => $aTheme['theme_id'],
        );

        if( SAA_Sys::getSett('user.validate_email') )
        {
            $aSql['is_verified'] = 0;
        }

        $iId = $this->database()->insert('user', $aSql);

        if( !$iId )
        {
            return false;
        }

        $aSql['user_id'] = $iId;

        if( SAA_Sys::getSett('user.validate_email') )
        {
            $this->emailVerification($aSql);
        }
        else
        {
            SAA_Sys::getModel('user.session')->createSession($aSql);
        }

        return $iId;
    }

    /**
     * Attempts to log the user in
     *
     * @param array $aVal
     * @param int $iTimes
     * @return bool
     */
    public function login($aVal, $iTimes)
    {
        if( SAA_Sys::getSett('user.login_method') == 'User Name' )
        {
            $this->database()->where('user_name = "'.$this->database()->esc($aVal['name']).'" AND');
        }
        elseif( SAA_Sys::getSett('user.login_method') == 'Email' )
        {
            $this->database()->where('email = "'.$this->database()->esc($aVal['email']).'" AND');
        }
        else
        {
            $this->database()->where('(user_name = "'.$this->database()->esc($aVal['name']).'" || email = "'.$this->database()->esc($aVal['name']).'") AND');
        }

        $aRow = $this->database()
            ->select('*')
            ->from('user')
            ->get('row');

        if( !$aRow )
        {
            if( SAA_Sys::getSett('user.login_method') == 'User Name' )
            {
                return SAA_Error::add(SAA_Sys::getPhrase('login.make_sure_your_user_name_is_correct'));
            }
            elseif( SAA_Sys::getSett('user.login_method') == 'Email' )
            {
                return SAA_Error::add(SAA_Sys::getPhrase('login.make_sure_your_email_is_correct'));
            }

            return SAA_Error::add(SAA_Sys::getPhrase('login.make_sure_your_email_or_user_name_is_correct'));
        }
        elseif( !$aRow['is_verified'] )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('login.your_account_must_be_activated_before_you_can_login_please_check_your_email'));
        }
        elseif( $aRow['password'] != md5(md5($aVal['password']) . $aRow['salt']) )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('login.the_password_entered_does_not_match_your_password'));
        }

        SAA_Sys::getModel('user.session')->createSession($aRow);

        return true;
    }

    public function edit($aVal)
    {
        $aSql = array();

        $aUser = SAA_Sys::getModel('user.session')->getUser();

        $aVal['email'] = trim($aVal['email']);

        if( $aVal['email'] != $aUser['email'] )
        {
            if( !strlen($aVal['email']) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.please_provide_an_email_address'));
            }
            elseif( !SAA_Sys::getLib('validator')->checkRegex('email', $aVal['email']) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.please_provide_a_valid_email_address'));
            }
            elseif( ($aExists = SAA_Sys::getModel('user')->get($aVal['email'], 'email')) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.email_address_is_already_in_use'));
            }

            $aSql['email'] = $aVal['email'];
        }

        $aVal['user_name'] = trim($aVal['user_name']);
        $aVal['user_name'] = $this->parse()->removeTags(html_entity_decode($aVal['user_name']), false);

        if( $aVal['user_name'] != $aUser['user_name'] )
        {
            if( !strlen($aVal['user_name']) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.please_provide_a_user_name'));
            }
            elseif( $this->parse()->cleanChrs($aVal['user_name'], SAA_Sys::getSett('user.banned_chars')) != $aVal['user_name'] )
            {
                SAA_Error::add(SAA_Sys::getPhrase('create.user_name_must_consist_of_letters_numbers_or_the_character')); //need to check non latin chars
            }
            elseif( ($aExists = SAA_Sys::getModel('user')->get($aVal['user_name'], 'user_name')) )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.user_name_is_already_in_use'));
            }

            $aSql['user_name'] = $aVal['user_name'];
        }

        if( strlen($aVal['password'] . $aVal['confirm']) )
        {
            if( strlen($aVal['password']) < 5 )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.password_must_be_greater_than_5_characters'));
            }
            elseif( $aVal['password'] != $aVal['confirm'] )
            {
                SAA_Error::add(SAA_Sys::getPhrase('account.password_does_not_match_confirmation'));
            }

            $aSql['salt'] = $this->parse()->salt(5);
            $aSql['password'] = md5(md5($aVal['password']) . $sSalt);
        }

        if( isset($aVal['lang_id']) )
        {
            $aSql['lang_id'] = (int) $aVal['lang_id'];
        }

        if( isset($aVal['theme_id']) )
        {
            $aSql['theme_id'] = (int) $aVal['theme_id'];
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        if( !$aUser['facebook_id'] && isset($_FILES['image']) )
        {
            $sImage = SAA_Sys::getLib('file')->upload('image', SAA_APP_USER_IMAGE, $aUser['user_id'], false, SAA_Sys::getSett('user.allowed_image_types'));

            if( $sImage )
            {
                $aSql['icon_loc'] = $sImage;
            }
        }

		if( $aSql )
		{
			$this->database()->update('user', $aSql, 'user_id = '. $aUser['user_id']);
		}

        return true;
    }

    /**
     * Admin version of updating user information
     *
     * @param array $aVal
     * @param array $aUser
     * @return bool
     */
    public function editByAdmin($aVal, $aUser)
    {
        $aSql = array();

        $aVal['email'] = trim($aVal['email']);

        if( !strlen($aVal['email']) )
        {
            SAA_Error::add('Please provide an email address.');
        }
        elseif( !SAA_Sys::getLib('validator')->checkRegex('email', $aVal['email']) )
        {
            SAA_Error::add('Please provide a valid email address.');
        }
        elseif( $aVal['email'] != $aUser['email'] )
        {
            if( ($aExists = SAA_Sys::getModel('user')->get($aVal['email'], 'email')) )
            {
                SAA_Error::add('Email address is already in use.');
            }
            else
            {
                $aSql['email'] = $aVal['email'];
            }
        }

        $aVal['user_name'] = trim($aVal['user_name']);
        $aVal['user_name'] = $this->parse()->removeTags(html_entity_decode($aVal['user_name']), false);

        if( !strlen($aVal['user_name']) )
        {
            SAA_Error::add('Please provide a user name.');
        }
        elseif( $this->parse()->cleanChrs($aVal['user_name'], SAA_Sys::getSett('user.banned_chars')) != $aVal['user_name'] )
        {
            SAA_Error::add('User name must not consist any of the following characters: '. SAA_Sys::getSett('user.banned_chars'));
        }
        elseif( $aVal['user_name'] != $aUser['user_name'] )
        {
            if( ($aExists = SAA_Sys::getModel('user')->get($aVal['user_name'], 'user_name')) )
            {
                SAA_Error::add('User name is already in use.');
            }
            else
            {
                $aSql['user_name'] = $this->parse()->rareToEntities($aVal['user_name'], 255);
            }
        }

        if( strlen($aVal['password']) > 0 )
        {
            if( strlen($aVal['password']) < 5 )
            {
                SAA_Error::add(SAA_Sys::getPhrase('create.password_must_be_greater_than_5_characters'));
            }
            elseif( $aVal['password'] != $aVal['password2'] )
            {
                SAA_Error::add(SAA_Sys::getPhrase('create.password_does_not_match_confirmation'));
            }
            elseif( !strlen($aVal['password']) )
            {
                $aSql['salt'] = $this->parse()->salt(5);
                $aSql['password'] = md5(md5($aVal['password']) . $aSql['salt']);
            }
        }

        if( SAA_Error::is() )
        {
            return false;
        }

        $aSql = array_merge($aSql, array(
            'is_verified' => $aVal['is_verified'],
            'lang_id' => $aVal['lang_id'],
            'theme_id' => $aVal['theme_id'],
            'total_points' => $aVal['total_points'],
            'total_medals' => $aVal['total_medals'],
            'total_champs' => $aVal['total_champs']
        ));

        $this->database()->update('user', $aSql, 'user_id = '. (int) $aUser['user_id']);

        return true;
    }

    /**
     * Send forgot password email
     *
     * @param string $sEmail
     * @return bool
     */
    public function emailPassword($sEmail)
    {
        $oUrl = SAA_Sys::getLib('url');

        $aUser = SAA_Sys::getModel('user')->get($sEmail, 'email');

        if( !$aUser )
        {
            return SAA_Error::add(SAA_Sys::getPhrase('user.we_cannot_find_any_accounts_associated_with_the_provided_email_address'));
        }

        $sHash = md5('emailPassword' . rand() . 'append' . md5(SAA_TIME . $aUser['user_id']));

        $this->database()->insert('user_forgotten', array('user_id' => $aUser['user_id'], 'user_hash' => $sHash, 'time_added' => SAA_TIME));

        $sLink = $oUrl->makeUrl(array(
            'controller' => 'user.forgotten',
            'vars' => array(
                'user' => $oUrl->encodeBase64($this->parse()->salt(4) . $sEmail . $this->parse()->salt(1)),
                'token' => $oUrl->encodeBase64($this->parse()->salt(4) . $sHash . $this->parse()->salt(1))
            )
        ));

        SAA_Sys::getLib('template')->assign(array(
            'aUser' => $aUser,
            'sEmailUrl' => $sLink,
            'sEmailHrs' => SAA_Sys::getSett('mail.forgotten_password_lifetime')
        ));

        return SAA_Sys::getLib('mail')
            ->from(SAA_Sys::getSett('mail.email'), SAA_Sys::getSett('mail.from'))
            ->to($sEmail)
            ->subject(SAA_Sys::getSett('mail.forgotten_password_subject'))
            ->message(SAA_Sys::getSett('mail.forgotten_password_body'))
            ->send();
    }

    /**
     * Send verify account email
     *
     * @param string $aUser
     * @return bool
     */
    public function emailVerification($aUser)
    {
        $oUrl = SAA_Sys::getLib('url');

        $sHash = md5('emailPassword' . rand() . 'append' . md5(SAA_TIME . $aUser['user_id']));

        $sLink = $oUrl->makeUrl(array(
            'controller' => 'user.validate',
            'vars' => array(
                'user' => $oUrl->encodeBase64($this->parse()->salt(4) . $aUser['email'] . $this->parse()->salt(1)),
                'token' => $oUrl->encodeBase64($this->parse()->salt(4) . $sHash . $this->parse()->salt(1))
            )
        ));

        SAA_Sys::getLib('template')->assign(array(
            'aUser' => $aUser,
            'url_link' => $sLink
        ));

        return SAA_Sys::getLib('mail')
            ->from(SAA_Sys::getSett('mail.email'), SAA_Sys::getSett('mail.from'))
            ->to($aUser['email'])
            ->subject(SAA_Sys::getSett('mail.verify_account_subject'))
            ->message(SAA_Sys::getSett('mail.verify_account_body'))
            ->send();
    }

    /**
     * Send verify account email to a given set of users
     *
     * @param array $aUsers
     * @return true
     */
    public function emailVerificationMass($aUsers)
    {
        $oUser = SAA_Sys::getModel('user');

        foreach($aUsers as $iUser)
        {
            $aUser = $oUser->get($iUser);

            if( !$aUser || $aUser['is_verified'] == 1 )
            {
                continue;
            }

            $this->emailVerification($aUser);
        }

        return true;
    }

    /**
     * Login using forgotten password email link
     *
     * @param string $sEmail
     * @param string $sHash
     * @return bool
     */
    public function forceLogin($sEmail, $sHash)
    {
        $oUser = SAA_Sys::getModel('user');

        $sEmail = $this->splitString($sEmail);
        $sHash = $this->splitString($sHash);

        $aRow = $this->database()
                ->select('u.*, uf.time_added as forgotten_time_stamp')
                ->from('user_forgotten', 'uf')
                ->join('user', 'u', 'u.user_id = uf.user_id AND u.email = "'.$this->database()->esc($sEmail).'"')
                ->where('uf.user_hash = "'.$this->database()->esc($sHash).'"')
                ->get('row');

        if( !$aRow )
        {
            SAA_Error::add(SAA_Sys::getPhrase('user.the_token_is_no_longer_valid'));
        }
        elseif( SAA_TIME > SAA_Sys::getSett('mail.forgotten_password_lifetime')*3600 + $aRow['forgotten_time_stamp'] )
        {
            SAA_Error::add(SAA_Sys::getPhrase('user.the_token_is_no_longer_valid'));
        }

        $this->database()->delete('user_forgotten', 'user_id = '. (int) $aRow['user_id']);

        if( SAA_Error::is() )
        {
            return false;
        }

        return SAA_Sys::getModel('user.session')->createSession($aRow);
    }

    /**
     * Salt a string
     *
     * @param string $sStr
     * @return string
     */
    public function splitString($sStr)
    {
        return substr($sStr, 4, -1);
    }

    /**
     * Removes all user's data
     *
     * @param array $iUser
     */
    private function _delete($iUser)
    {
        $this->database()->delete('feed', 'user_id = '. (int) $iUser);


        $this->database()->delete('user_award', 'user_id = '. (int) $iUser);
        $this->database()->delete('user_forgotten', 'user_id = '. (int) $iUser);
        $this->database()->delete('user_session', 'user_id = '. (int) $iUser);
        $this->database()->delete('user_session_force', 'user_id = '. (int) $iUser);


        //remove from scores tables
            //update total scores in game table
        //remove from game rate
            //update total votes and total rating
        //remove from game favorites
            //update total favorites
        //remove from friend and friend request
        //remove from comment
            //update total comments for respected tables
        //remove from message
        //remove from news
        //remove from challenges and challenge cache
        
    }
}
?>
