<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name User Session Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_User_Session extends SAA_Model
{
    private $_aUser = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        if( SAA_Sys::getSett('facebook.enable_login') )
        {
            if( !($aSession = SAA_Sys::getLib('session')->get('user')) )
            {
                $iFacebookId = SAA_Sys::getLib('api.facebook')->getUserId();
            }
        }
        else
        {
            $aSession = SAA_Sys::getLib('session')->get('user');
        }

        $sIp = SAA_Sys::getLib('browser')->getIp();

        /**
         * Attempt to authorize user from fingerprint
         */
        if( ($sForce = SAA_Sys::getLib('request')->get('sForceUserHash')) )
        {
            $aUser = $this->database()
                    ->select('u.*, us.user_hash, us.user_agent, usf.time_forced')
                    ->from('user_session_force', 'usf')
                    ->join('user_session', 'us', 'us.user_id = usf.user_id AND us.user_ip = "'.$this->database()->esc($sIp).'"')
                    ->join('user', 'u', 'u.user_id = us.user_id')
                    ->where('usf.force_hash = "'.$this->database()->esc($sForce).'"')
                    ->get('row');

            if( !$aUser || !$this->checkHash($this->createHash($aUser['password'], $aUser['salt']), $aUser['user_hash']) )
            {
                $this->logout();

                unset($aUser);
            }
            elseif( $aUser['time_forced'] < SAA_TIME - 86400 )
            {
                $this->database()->delete('user_session_force', 'force_hash = "'.$this->database()->esc($sForce).'"');

                $this->logout();

                unset($aUser);
            }
            else
            {
                unset($aUser['user_hash'], $aUser['user_agent'], $aUser['time_forced']);
            }
        }
        /**
         * Attempt to authorize user from session
         */
        elseif( $aSession )
        {
            $aUser = $this->database()
                    ->select('u.*, us.user_agent')
                    ->from('user_session', 'us')
                    ->join('user', 'u', 'u.user_id = us.user_id')
                    ->where('us.user_id = '. $aSession['user_id'] .' AND us.user_hash = "'.$this->database()->esc($aSession['salty_apple']).'" AND us.user_ip = "'.$this->database()->esc($sIp).'"')
                    ->get('row');

            //Something does not match
            if( !$aUser || $this->createHash($_SERVER['HTTP_USER_AGENT'], $aUser['salt']) != $aUser['user_agent'] || !$this->checkHash($this->createHash($aUser['password'], $aUser['salt']), $aSession['salty_apple']) )
            {
                $this->logout();

                unset($aUser);
            }
        }
        /**
         * Attempt to authorize user from facebook
         */
        elseif( SAA_Sys::getSett('facebook.enable_login') )
        {
            if( $iFacebookId )
            {
				$aUser = $this->database()
                    ->select('*')
                    ->from('user')
                    ->where('facebook_id = '. $iFacebookId)
                    ->get('row');

                //create session
                if( $aUser )
                {
                    $this->createSession($aUser);
                }
                //add new facebook user to arcade
                else
                {
                    if( SAA_Sys::getModel('user.process')->addFromFacebook() )
                    {
                        SAA_Sys::getLib('url')->reload();
                    }
                }
            }
        }

        if( isset($aUser) && $aUser )
        {
            if( !$aUser['is_verified'] )
            {
                $this->logout();
            }
            else
            {
                unset($aUser['password'], $aUser['salt']);

                $this->_aUser = $aUser;
            }
        }
    }

    /**
     * Verifies the hash matches the given fingerprint
     *
     * @param string $sHash
     * @param string $sFingerPrint
     * @return bool
     */
    public function checkHash($sHash, $sFingerPrint)
    {
        if( strlen($sFingerPrint) != 52 )
        {
            return false;
        }

        $sSeed = substr($sFingerPrint, 40, 12);

        if (sha1($sSeed . md5($sHash) . $sSeed) . $sSeed == $sFingerPrint)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Turns a password and a given salt into a hash
     *
     * @param string $sPassword
     * @param string $sSalt
     * @return string
     */
    public function createHash($sPassword, $sSalt = '')
    {
        if( !$sSalt )
        {
            $sSalt = SAA_Sys::getLib('parse.input')->salt(5);
        }

        return md5(md5($sPassword) . md5($sSalt));
    }

    /**
     * Creates a fingerprint for a given hash
     *
     * @param string $sHash
     * @return string
     */
    public function createFingerprint($sHash)
    {
        $sSeed = '';

        for ($i = 1; $i <= 12; $i++)
        {
            $sSeed .= substr('howc0meist4aTr', rand(0, 13), 1);
        }

        return sha1($sSeed . md5($sHash) . $sSeed) . $sSeed;
    }

    /**
     * Creates a force session hash
     * 
     * @return mixed
     */
    public function createForce()
    {
        if( !$this->isUser() )
        {
            return false;
        }

        $sHash = $this->createHash('salty' . rand() . 'apples');
        $sHash = $this->createFingerprint($sHash);

        $this->database()->insert('user_session_force', array(
            'user_id' => $this->getUserId(),
            'force_hash' => $this->createFingerprint($sHash),
            'time_forced' => SAA_TIME
        ));

        return $sHash;
    }

    /**
     * Creates a session for the given user
     *
     * @param array $aUser
     * @return bool
     */
    public function createSession($aUser)
    {
        $this->removeRows($aUser['user_id']);

        $sApple = $this->createFingerprint($this->createHash($aUser['password'], $aUser['salt']));

        $this->database()->insert('user_session', array(
            'user_id' => $aUser['user_id'],
            'user_hash' => $sApple,
            'user_agent' => $this->createHash($_SERVER['HTTP_USER_AGENT'], $aUser['salt']),
            'user_ip' => SAA_Sys::getLib('browser')->getIp(),
            'time_added' => SAA_TIME
        ));

        SAA_Sys::getLib('session')->set('user', array(
            'salty_apple' => $sApple,
            'user_id' => $aUser['user_id']
        ));

        return true;
    }

    /**
     * Gets a given set of values for the current user
     *
     * @param mixed $mKeys
     * @return mixed
     */
    public function get($mKeys)
    {
        $aVal = array();

        if( is_string($mKeys) )
        {
            return $this->_aUser[$mKeys];
        }

        foreach($mKeys as $sKey)
        {
            $aVal[$sKey] = $this->_aUser[$sKey];
        }

        return $aVal;
    }

    /**
     * Gets the session force hash
     *
     * @return mixed
     */
    public function getForce()
    {
        if( !$this->isUser() )
        {
            return false;
        }

        $aRow = $this->database()
            ->select('*')
            ->from('user_session_force')
            ->where('user_id = '. (int) $this->getUserId())
            ->get('row');

        if( $aRow && $aRow['time_forced'] < SAA_TIME - 86400 )
        {
            $this->database()->delete('user_session_force', 'user_id = '. (int) $aRow['user_id']);

            $aRow = false;
        }

        if( !$aRow )
        {
            return $this->createForce();
        }

        return $aRow['force_hash'];
    }

    /**
     * Gets the current user array
     *
     * @return array
     */
    public function getUser()
    {
        return $this->_aUser;
    }

    /**
     * Gets the current user id
     *
     * @return int
     */
    public function getUserId()
    {
        return ($this->isUser() ? $this->_aUser['user_id'] : 0);
    }

    /**
     * Determines whether or not the user is an admin
     *
     * @return bool
     */
    public function isAdmin()
    {
        if( !$this->isUser() )
        {
            return false;
        }

        return ($this->_aUser['is_admin'] ? true : false);
    }

    /**
     * Returns whether or not a user is logged in
     *
     * @return bool
     */
    public function isUser()
    {
        return ($this->_aUser ? true : false);
    }

    /**
     * Kills session and logs out user
     *
     * @return bool
     */
    public function logout()
    {
        $this->removeRows();

        if( SAA_Sys::getSett('facebook.enable_login') )
        {
            SAA_Sys::getLib('api.facebook')->logout();
        }

        SAA_Sys::getLib('session')->clear();

        return true;
    }

    /**
     * Removes user session data in database
     *
     * @param int $iUser
     */
    public function removeRows($iUser = false)
    {
        if( !$iUser && $this->isUser() )
        {
            $iUser = $this->getUserId();
        }

        if( $iUser )
        {
            $this->database()->delete('user_session', 'user_id = '. (int) $iUser);
            $this->database()->delete('user_session_force', 'user_id = '. (int) $iUser);
        }
    }
}
?>
