<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name User
 * @license GoLevel LLC
 */

Class SAA_Model_User extends SAA_Model
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Finds a set of like users from a given string
     *
     * @param string $sLike
     * @param int $iLimit
     * @return array
     */
    public function findLike($sLike, $iLimit)
    {
        $aUsers = array();

        $aRows = $this->database()
                ->select('user_id, user_name')
                ->from('user')
                ->where('user_name LIKE "'.$this->database()->esc($sLike).'%"')
                ->get('rows');

        if( !$aRows )
        {
            return array(
                'id' => '',
                'label' => '',
                'value' => ''
            );
        }

        foreach($aRows as $aRow)
        {
            $aUsers[] = array(
                'id' => $aRow['user_id'],
                'label' => $aRow['user_name'],
                'value' => $aRow['user_name']
            );
        }

        return $aUsers;
    }

    /**
     * Gets a given user provided a given column and value
     *
     * @param string $sValue
     * @param string $sType
     * @return mixed
     */
    public function get($sValue, $sType = 'user_id')
    {
        $aUser = $this->database()
            ->select('*')
            ->from('user')
            ->where($sType . ' = "' . $this->database()->esc($sValue) . '"')
            ->get('row');

        if( !$aUser )
        {
            return false;
        }

        unset($aUser['password']);

        return $aUser;
    }

    /**
     * Gets the user name (link) for a given user
     *
     * @param mixed $mUser
     * @return string
     */
    public function getName($mUser = null, $sAttr = '')
    {
        if( is_null($mUser) )
        {
            if( !SAA_Sys::isUser() )
            {
                return '';
            }

            $mUser = SAA_Sys::getModel('user.session')->get(array(
                'user_id', 'user_name'
            ));
        }
        elseif( is_numeric($mUser) )
        {
            $mUser = $this->get($mUser);
        }

        if( SAA_Sys::getSett('user.enable_profile') )
        {
            $sText = '<a '.$sAttr.' href="'.SAA_Sys::getLib('url')->makeUrl(array('controller' => 'profile', 'vars' => array('user' => $mUser['user_id']))).'">'.$mUser['user_name'].'</a>';
        }
        else
        {
            $sText = $mUser['user_name'];
        }

        return $sText;
    }

    /**
     * Gets the essential user columns with an optional prepended prefix
     *
     * @param string $sPrefix
     * @param string $sPrepend
     * @return string
     */
    public function getUserFields($sPrefix = '', $sPrepend = '')
    {
        if( $sPrefix )
        {
            $sPrefix .= '.';
        }

        if( $sPrepend )
        {
            return "{$sPrefix}user_id as {$sPrepend}_user_id, {$sPrefix}facebook_id as {$sPrepend}_facebook_id, {$sPrefix}user_name as {$sPrepend}_user_name, {$sPrefix}email as {$sPrepend}_email, {$sPrefix}gender as {$sPrepend}_gender, {$sPrefix}icon_loc as {$sPrepend}_icon_loc, {$sPrefix}total_champs as {$sPrepend}_total_champs, {$sPrefix}total_points as {$sPrepend}_total_points";
        }

        return "{$sPrefix}user_id, {$sPrefix}facebook_id, {$sPrefix}user_name, {$sPrefix}email, {$sPrefix}gender, {$sPrefix}icon_loc, {$sPrefix}total_champs, {$sPrefix}total_points";
    }

    public function searchAdmin($aForm, $iPg, $iPgSize)
    {
        $aWhere = array();

        if( isset($aForm['user_name']) && strlen($aForm['user_name']) )
        {
            $aWhere[] = 'u.user_name = "'.$this->database()->esc($aForm['user_name']).'" AND';
        }

        if( isset($aForm['email']) && strlen($aForm['email']) )
        {
            $aWhere[] = 'u.email = "'.$this->database()->esc($aForm['email']).'" AND';
        }

        if( isset($aForm['is_verified']) && strlen($aForm['is_verified']) )
        {
            $aWhere[] = 'u.is_verified = "'.(int) $aForm['is_verified'].'" AND';
        }

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        switch($aForm['sort_by'])
        {
            case 'date_added asc':
                $sOrder = 'u.date_added ASC';
                break;
            case 'date_added desc':
                $sOrder = 'u.date_added DESC';
                break;
            case 'last_login asc':
                $sOrder = 'u.last_login ASC';
                break;
            default:
                $sOrder = 'u.last_login DESC';
        }

        $aRows = $this->database()
            ->select('u.*')
            ->from('user', 'u')
            ->limit($iPgSize, $iPg)
            ->order($sOrder)
            ->get('rows');

        if( $aWhere )
        {
            $this->database()->where($aWhere);
        }

        $iCnt = $this->database()
            ->select('COUNT(*)')
            ->from('user', 'u')
            ->get('column');

        return array($iCnt, $aRows);
    }

    /**
     * Gets the users with the most high scores held
     *
     * @param int $iLimit
     * @return array
     */
    public function topPlayers($iLimit)
    {
        $aUsers = $this->database()
            ->select('*')
            ->from('user')
            ->order(array('total_champs' => 'DESC'))
            ->get('rows');

        return $aUsers;
    }

	/**
	 * Get the total number of users
	 *
	 * @return int
	 */
	public function total()
	{
		return $this->database()
            ->select('COUNT(*)')
            ->from('user', 'u')
            ->get('column');
	}
}
?>
