<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Include Tables
 * @license GoLevel LLC
 */

$_aTables = array();

//move feed text to a new 'feed_text' table?

$_aTables['ad'] = array(
    'fields' => array(
        'ad_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'ad_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '150'
        ),
        'type_id' => array(
            'type' => 'TINYINT',
            'constraint' => '1'
        ),
        'location' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'default' => 0
        ),
        'start_date' => array(
            'type' => 'INT',
            'constraint' => '11',
            'default' => 0
        ),
        'end_date' => array(
            'type' => 'INT',
            'constraint' => '11',
            'default' => 0
        ),
        'total_views' => array(
            'type' => 'INT',
            'constraint' => '11',
            'default' => 0
        ),
        'total_clicks' => array(
            'type' => 'INT',
            'constraint' => '11',
            'default' => 0
        ),
        'is_active' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'default' => 1
        ),
        'html_code' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'image_path' => array(
            'type' => 'VARCHAR',
            'constraint' => '75',
            'default' => null
        )
    ),
    'primary' => 'ad_id',
    'unique' => array(
        'ad_name'
    )
);

$_aTables['category'] = array(
    'fields' => array(
        'category_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'name' => array(
            'type' => 'VARCHAR',
            'constraint' => '250'
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => '250'
        ),
        'ordering' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        ),
        'time_stamp' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'category_id',
    'unique' => array(
        'slug'
    )
);

$_aTables['category_data'] = array(
    'fields' => array(
        'category_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        ),
        'map_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        )
    ),
    'key' => array(
        'category_id',
        'map_id'
    )
);

$_aTables['category_map'] = array(
    'fields' => array(
        'map_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'type_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'map_title' => array(
            'type' => 'VARCHAR',
            'constraint' => '250',
        ),
        'map_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '250',
        )
    ),
    'primary' => 'map_id'
);

$_aTables['comment'] = array(
    'fields' => array(
        'comment_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'parent_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'model_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '50'
        ),
        'item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'child_total' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'time_stamp' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'update_time' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'comment_id',
    'key' => array(
        'item_id' => array(
            'item_id',
            'model_id',
            'parent_id'
        ),
        'parent_id' => array(
            'parent_id',
            'model_id'
        ),
        'comment_id' => array(
            'comment_id',
            'model_id',
            'item_id'
        ),
        'comment_id_2' => array(
            'comment_id',
            'user_id'
        )
    )
);

$_aTables['comment_text'] = array(
    'fields' => array(
        'comment_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'text' => array(
            'type' => 'MEDIUMTEXT'
        )
    ),
    'key' => array(
        'comment_id'
    )
);

$_aTables['crumb'] = array(
    'fields' => array(
        'link_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'link_slug' => array(
            'type' => 'VARCHAR',
            'constraint' => '32'
        ),
        'link_content' => array(
            'type' => 'TEXT'
        ),
        'time_added' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'link_id',
    'unique' => array(
        'link_slug'
    )
);

$_aTables['feed'] = array(
    'fields' => array(
        'feed_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'type_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '75'
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'parent_item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'content' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'time_stamp' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'feed_id',
    'key' => array(
        'type_id'
    )
);

$_aTables['game'] = array(
    'fields' => array(
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'type_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'slug' => array(
            'type' => 'VARCHAR',
            'constraint' => '250'
        ),
        'featured' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'category_manual' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'total_rating' => array(
            'type' => 'DECIMAL',
            'constraint' => '3,2',
            'default' => '0.00'
        ),
        'total_votes' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'total_plays' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'total_favorites' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'total_scores' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'total_comments' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'file_loc' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'img_loc' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'px_width' => array(
            'type' => 'VARCHAR',
            'constraint' => '4'
        ),
        'px_height' => array(
            'type' => 'VARCHAR',
            'constraint' => '4'
        ),
        'scoring' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'scoring_prefix' => array(
            'type' => 'VARCHAR',
            'constraint' => '50'
        ),
        'scoring_order' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'time_added' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'primary' => 'game_id',
    'unique' => array(
        'slug'
    )
);

$_aTables['game_award'] = array(
    'fields' => array(
        'award_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'award_hash' => array(
            'type' => 'VARCHAR',
            'constraint' => '32'
        ),
        'type_id' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true
        ),
        'is_active' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 1
        ),
        'title' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'thumbnail' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        )
    ),
    'primary' => 'award_id',
    'key' => array(
        'game_id',
        'award_hash' => array(
            'game_id',
            'award_hash'
        )
    )
);

$_aTables['game_award_text'] = array(
    'fields' => array(
        'award_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'award_description' => array(
            'type' => 'TEXT'
        )
    ),
    'key' => array(
        'award_id'
    )
);

$_aTables['game_category'] = array(
    'fields' => array(
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'category_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        ),
        'is_excluded' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        )
    ),
    'key' => array(
        'category_id',
        'game_id',
        'game_id_2' => array(
            'game_id',
            'category_id'
        )
    )
);

$_aTables['game_category_cache'] = array(
    'fields' => array(
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'category_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        )
    ),
    'key' => array(
        'category_id',
        'game_id'
    )
);

$_aTables['game_category_map'] = array(
    'fields' => array(
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'category_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '20',
        ),
        'map_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        )
    ),
    'key' => array(
        'game_id',
        'map_id',
        'map_id_2' => array(
            'game_id',
            'map_id'
        )
    )
);

$_aTables['game_extra'] = array(
    'fields' => array(
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'source_type' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'source_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '50',
        ),
        'file_size' => array(
            'type' => 'VARCHAR',
            'constraint' => '12',
            'default' => 0
        ),
        'image_size' => array(
            'type' => 'VARCHAR',
            'constraint' => '12',
            'default' => 0
        ),
    ),
    'key' => array(
        'game_id',
        'source_id'
    )
);

$_aTables['game_favorite'] = array(
    'fields' => array(
        'item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'item_id',
    'key' => array(
        'user_id',
        'game_id' => array(
            'game_id',
            'user_id'
        )
    )
);

$_aTables['game_featured'] = array(
    'fields' => array(
        'feature_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'ordering' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        )
    ),
    'primary' => 'feature_id'
);

$_aTables['game_link'] = array(
    'fields' => array(
        'game_link_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_link' => array(
            'type' => 'TEXT'
        )
    ),
    'primary' => 'game_link_id'
);

$_aTables['game_rate'] = array(
    'fields' => array(
        'rate_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'item_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'rating' => array(
            'type' => 'DECIMAL',
            'constraint' => '3,2'
        ),
        'time_stamp' => array(
            'type' => 'INT',
            'constraint' => '10',
            'unsigned' => true
        )
    ),
    'primary' => 'rate_id',
    'key' => array(
        'has_rated' => array(
            'item_id',
            'user_id'
        )
    )
);

$_aTables['game_scoring'] = array(
    'fields' => array(
        'item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'scoring_type' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'scoring_hash' => array(
            'type' => 'VARCHAR',
            'constraint' => '32'
        )
    ),
    'primary' => 'item_id',
    'key' => array(
        'scoring_hash' => array(
            'scoring_hash',
            'scoring_type'
        ),
        'game_id',
        'game_id_2' => array(
            'game_id',
            'scoring_type'
        )
    )
);

$_aTables['game_tag'] = array(
    'fields' => array(
        'tag_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'tag' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        )
    ),
    'primary' => 'tag_id',
    'key' => array(
        'game_id',
        'game_id_2' => array(
            'game_id',
            'tag'
        )
    )
);

$_aTables['game_type'] = array(
    'fields' => array(
        'game_type_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_type_title' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        )
    ),
    'primary' => 'game_type_id'
);

$_aTables['language'] = array(
    'fields' => array(
        'language_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'title' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'language_abrv' => array(
            'type' => 'VARCHAR',
            'constraint' => '12',
        ),
        'direction' => array(
            'type' => 'VARCHAR',
            'constraint' => '3',
        ),
        'created_by' => array(
            'type' => 'VARCHAR',
            'constraint' => '255',
        ),
        'is_default' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => '0'
        ),
        'is_enabled' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => '1'
        )
    ),
    'primary' => 'language_id'
);

$_aTables['language_game_phrase'] = array(
    'fields' => array(
        'phrase_id' => array(
            'type' => 'INT',
            'constraint' => '10',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'language_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'var_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '250',
        ),
        'text' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'text_default' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'time_added' => array(
            'type' => 'INT',
            'constraint' => '10',
            'unsigned' => true
        )
    ),
    'primary' => 'phrase_id',
    'key' => array(
        'language_id',
        'game_phrase' => array(
            'game_id',
            'var_name',
            'language_id'
        ),
        'game_id',
        'game_id_2' => array(
            'game_id',
            'language_id'
        ),
        'game_id_3' => array(
            'game_id',
            'var_name'
        )
    )
);

$_aTables['language_phrase'] = array(
    'fields' => array(
        'phrase_id' => array(
            'type' => 'INT',
            'constraint' => '10',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'language_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'section_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '20',
            'default' => 'global'
        ),
        'var_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '250',
        ),
        'text' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'text_default' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'time_added' => array(
            'type' => 'INT',
            'constraint' => '10',
            'unsigned' => true
        )
    ),
    'primary' => 'phrase_id',
    'key' => array(
        'language_id',
        'section_id' => array(
            'section_id',
            'var_name'
        ),
        'section_id_2' => array(
            'section_id'
        ),
        'var_name' => array(
            'var_name',
            'section_id',
            'language_id'
        )
    )
);

$_aTables['language_section'] = array(
    'fields' => array(
        'section_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'section_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '20',
        ),
        'section_title' => array(
            'type' => 'VARCHAR',
            'constraint' => '255',
        ),
        'is_required' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'unsigned' => true,
            'default' => 0
        )
    ),
    'primary' => 'section_id',
    'unique' => array(
        'section_name'
    ),
    'key' => array(
        'section_name_2' => array(
            'section_name',
            'is_required'
        )
    )
);

$_aTables['menu'] = array(
    'fields' => array(
        'menu_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'parent_id' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true,
            'default' => 0
        ),
        'type_id' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
        ),
        'menu_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'href_link' => array(
            'type' => 'TEXT'
        ),
        'is_controller' => array(
            'type' => 'TINYINT',
            'constraint' => 1,
            'unsigned' => true
        ),
        'ordering' => array(
            'type' => 'SMALLINT',
            'constraint' => '5',
            'unsigned' => true
        ),
    ),
    'primary' => 'menu_id'
);

$_aTables['report'] = array(
    'fields' => array(
        'report_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'model_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '50'
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'time_reported' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'primary' => 'report_id'
);

$_aTables['report_item'] = array(
    'fields' => array(
        'report_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'type_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '20'
        )
    ),
    'key' => array(
        'report_id'
    )
);

$_aTables['report_text'] = array(
    'fields' => array(
        'report_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'reason' => array(
            'type' => 'TEXT'
        )
    ),
    'key' => array(
        'report_id'
    )
);

$_aTables['score'] = array(
    'fields' => array(
        'score_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'score' => array(
            'type' => 'INT',
            'constraint' => '12'
        ),
        'time_scored' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'score_id',
    'key' => array(
        'game_id',
        'user_id',
        'user_id_2' => array(
            'game_id',
            'user_id'
        )
    )
);

$_aTables['score_high'] = array(
    'fields' => array(
        'score_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'high_score' => array(
            'type' => 'INT',
            'constraint' => '12'
        ),
        'time_scored' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'score_id',
    'key' => array(
        'game_id'
    )
);

$_aTables['score_user'] = array(
    'fields' => array(
        'score_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'user_score' => array(
            'type' => 'INT',
            'constraint' => '12'
        ),
        'time_scored' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'score_id',
    'key' => array(
        'user_id',
        'user_id_2' => array(
            'game_id',
            'user_id'
        )
    )
);

$_aTables['setting'] = array(
    'fields' => array(
        'setting_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'type_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '20'
        ),
        'group_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '20'
        ),
        'setting_title' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'setting_explain' => array(
            'type' => 'TEXT'
        ),
        'var_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '250'
        ),
        'value_current' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'value_original' => array(
            'type' => 'MEDIUMTEXT'
        ),
        'ordering' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'default' => 0
        )
    ),
    'primary' => 'setting_id'
);

$_aTables['setting_controller'] = array(
    'fields' => array(
        'setting_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true
        ),
        'controller_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'ordering' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'default' => 0
        )
    ),
    'key' => array(
        'setting_id' => array(
            'setting_id',
            'controller_id'
        )
    )
);

$_aTables['setting_group'] = array(
    'fields' => array(
        'setting_group_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'group_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '30'
        ),
        'group_id' => array(
            'type' => 'VARCHAR',
            'constraint' => '20'
        ),
        'js_actions' => array(
            'type' => 'TEXT'
        ),
        'ordering' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'default' => 0
        )
    ),
    'primary' => 'setting_group_id',
    'unique' => array(
        'group_id'
    )
);

$_aTables['site_info'] = array(
    'fields' => array(
        'info_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'var_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '30'
        ),
        'info_value' => array(
            'type' => 'TEXT'
        ),
        'date_updated' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        )
    ),
    'primary' => 'info_id',
	'unique' => array(
        'var_name'
    )
);


$_aTables['theme'] = array(
    'fields' => array(
        'theme_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'title' => array(
            'type' => 'VARCHAR',
            'constraint' => '250'
        ),
        'is_default' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'folder_loc' => array(
            'type' => 'VARCHAR',
            'constraint' => '250'
        ),
        'is_active' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 1
        ),
    ),
    'primary' => 'theme_id'
);

$_aTables['user'] = array(
    'fields' => array(
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'facebook_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'default' => 0
        ),
        'is_admin' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'is_verified' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 1
        ),
        'user_name' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'email' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'password' => array(
            'type' => 'VARCHAR',
            'constraint' => '32'
        ),
        'salt' => array(
            'type' => 'VARCHAR',
            'constraint' => '5'
        ),
        'gender' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 0
        ),
        'icon_loc' => array(
            'type' => 'VARCHAR',
            'constraint' => '75'
        ),
        'date_added' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'last_login' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'lang_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'default' => 0
        ),
        'theme_id' => array(
            'type' => 'TINYINT',
            'constraint' => '3',
            'unsigned' => true,
            'default' => 0
        ),
        'total_achievements' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true,
            'default' => 0
        ),
        'total_champs' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true,
            'default' => 0
        ),
        'total_medals' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true,
            'default' => 0
        ),
        'total_points' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'primary' => 'user_id',
    'unique' => array(
        'user_name',
        'email'
    ),
);

$_aTables['user_award'] = array(
    'fields' => array(
        'item_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true,
            'auto_increment' => true
        ),
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'game_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true,
        ),
        'award_id' => array(
            'type' => 'MEDIUMINT',
            'constraint' => '8',
            'unsigned' => true
        ),
        'is_active' => array(
            'type' => 'TINYINT',
            'constraint' => '1',
            'unsigned' => true,
            'default' => 1
        ),
        'date_added' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'primary' => 'item_id',
    'keys' => array(
        'user_id',
        'user_id_2' => array(
            'user_id',
            'game_id'
        ),
        'user_id_3' => array(
            'user_id',
            'award_id'
        ),
        'game_id'
    )
);

$_aTables['user_forgotten'] = array(
    'fields' => array(
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'user_hash' => array(
            'type' => 'VARCHAR',
            'constraint' => '255'
        ),
        'time_added' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'unique' => array(
        'user_hash'
    ),
    'keys' => array(
        'user_id'
    )
);

$_aTables['user_session'] = array(
    'fields' => array(
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'user_hash' => array(
            'type' => 'VARCHAR',
            'constraint' => '52'
        ),
        'user_agent' => array(
            'type' => 'VARCHAR',
            'constraint' => '32'
        ),
        'user_ip' => array(
            'type' => 'VARCHAR',
            'constraint' => '15'
        ),
        'time_added' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'unique' => array(
        'user_id',
        'user_hash'
    ),
    'keys' => array(
        'user_id_2' => array(
            'user_id',
            'user_hash',
            'user_ip'
        )
    )
);

$_aTables['user_session_force'] = array(
    'fields' => array(
        'user_id' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
        'force_hash' => array(
            'type' => 'VARCHAR',
            'constraint' => '52'
        ),
        'time_forced' => array(
            'type' => 'INT',
            'constraint' => '12',
            'unsigned' => true
        ),
    ),
    'unique' => array(
        'force_hash'
    ),
    'keys' => array(
        'user_id'
    )
);

?>