<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Migrate Games Page
 * @license GoLevel LLC
 */

require_once(SAA_SYSTEM_INCLUDE . 'app.php');
require_once(SAA_DIR . 'system' . DS . 'library' . DS . 'model.class.php');

//Load settings from db then cache
SAA_Sys::getLib('setting')->build();

$sHeadTitle = 'Migrating Games';

$oDb = SAA_Sys::getLib('database');
$oFile = SAA_Sys::getLib('file');
$oParse = SAA_Sys::getLib('parse.input');

$iLimit = 15;
$iPage = (isset($_GET['page']) ? (int) $_GET['page'] : 0);
$bImport = (isset($_GET['import']) ? (int) $_GET['import'] : 0);

if( $bImport )
{
	//If there is no legacy game table, move to next stage
	if( !$oDb->togglePrefix(false)->tableExists('arcade_games') )
	{
		header('Location: ./update.php?'.$sPageNext);
        exit;
	}

    if( !$oDb->togglePrefix(true)->isColumn('game', 'old_id') )
    {
        $oDb->alterTable('add', 'game', array(
            'old_id INT( 12 ) UNSIGNED NOT NULL'
        ));
    }

    $aGames = $oDb->select('ag.*')
        ->togglePrefix(false)
        ->from('arcade_games', 'ag')
        ->togglePrefix(true)
        ->leftjoin('game', 'g', 'g.old_id = ag.game_id')
        ->where('g.game_id IS NULL')
        ->limit($iLimit, $iPage)
        ->get('rows');

    if( !$aGames )
    {
        header('Location: ./update.php?'.$sPageNext);
        exit;
    }

	if( !SAA_Sys::getSett('mochi.publisher_id') )
	{
		SAA_Sys::getLib('setting')->set('mochi.publisher_id', '1e7738e7c78a0639')
	}

    $oMochi = SAA_Sys::getModel('game.adder.mochi');

    $oGameTagProcess = SAA_Sys::getModel('game.tag.process');
    $oGameType = SAA_Sys::getModel('game.type.process');
    $oGameLink = SAA_Sys::getModel('game.link');

    $aGameType = SAA_Sys::getModel('game.type')->getAll(false);

    $oLangGameProcess = SAA_Sys::getModel('language.game.process');

    $aLanguages = SAA_Sys::getLib('language')->getLanguages();
    $aLanguages = array_keys($aLanguages);

    $sOldDirSwf = SAA_DIR . 'file' . DS . 'swf' . DS;
    $sOldDirImg = SAA_DIR . 'file' . DS . 'images' . DS;

    foreach($aGames as $aGame)
    {
        if( $aGame['type'] != 1 )
        {
            continue;
        }

        $sSlug = $oParse->toAlphaNumeric($aGame['title'], 250, '_-');
        $sSlug = $oParse->getUnique($sSlug, 'game', 'slug', 250);

        $aSql = array(
            'old_id' => $aGame['game_id'],
            'type_id' => $aGameType['MochiMedia']['game_type_id'],
            'slug' => $sSlug,
            'total_rating' => (isset($aGame['rating']) ? $aGame['rating'] : 0),
            'total_votes' => (isset($aGame['votes']) ? $aGame['votes'] : 0),
            'total_plays' => (int) $aGame['plays'],
            'px_width' => $aGame['width'],
            'px_height' => $aGame['height'],
            'scoring' => $aGame['has_scores'],
            'scoring_order' => $aGame['reverse'],
            'time_added' => $aGame['time_added']
        );

        $iGame = $oDb->insert('game', $aSql);

        if( !$iGame )
        {
            continue;
        }

        $aExtra = array(
            'game_id' => $iGame,
            'source_type' => $aGameType['MochiMedia']['game_type_id'],
            'source_id' => $aGame['unique']
        );

        $sDir = $oFile->buildDir($iGame);

        /**
         * Move swf file on local
         */
        if( strpos($aGame['file'], 'http') === false )
        {
            $sOldSwf = $sOldDirSwf . ltrim($aGame['file'], DS);

            if( $oFile->exists($sOldSwf) )
            {
                $sSwfFile = $oFile->buildName(SAA_APP_GAME_SWF . $sDir, 'swf');

                $oFile->createDir(SAA_APP_GAME_SWF . $sDir . $sSwfFile);

                if( $oFile->move($sOldSwf, SAA_APP_GAME_SWF . $sDir . $sSwfFile) )
                {
                    $aSql['file_loc'] = $sDir . $sSwfFile;
                    $aExtra['file_size'] = filesize(SAA_APP_GAME_SWF . $sDir . $sSwfFile);
                }
            }
        }
        /**
         * Shrink swf url
         */
        else
        {
            $aSql['file_loc'] = $oLink->shrink($aGame['file']);
        }

        /**
         * Move swf file on local
         */
        if( strpos($aGame['image'], 'http') === false )
        {
            $sOldImg = $sOldDirImg . ltrim($aGame['image'], DS);

            if( $oFile->exists($sOldImg) )
            {
                $sImgExt = $oFile->getExt($sOldImg);

                $sImgFile = $oFile->buildName(SAA_APP_GAME_IMAGE . $sDir, $sImgExt);

                $oFile->createDir(SAA_APP_GAME_IMAGE . $sDir . $sImgFile);

                if( $oFile->move($sOldImg, SAA_APP_GAME_IMAGE . $sDir . $sImgFile) )
                {
                    $aSql['img_loc'] = $sDir . $sImgFile;
                    $aExtra['image_size'] = filesize(SAA_APP_GAME_IMAGE . $sDir . $sImgFile);
                }
            }
        }
        /**
         * Shrink swf url
         */
        else
        {
            $aSql['img_loc'] = $oLink->shrink($aGame['image']);
        }

        $oDb->update('game', $aSql, 'game_id = '. (int) $iGame);

        $oDb->insert('game_extra', $aExtra);
        $oDb->insert('game_scoring', array(
            'game_id' => $iGame,
            'scoring_type' => $aGameType['MochiMedia']['game_type_id'],
            'scoring_hash' => $aGame['unique']
        ));

        if( ($aImport = $oMochi->importSingle($aGame['unique'])) )
        {
            $aText = array(
                'title' => $oParse->removeTags($aImport['name']),
                'description' => $oParse->cleanTags($aImport['description'], 'global.allowed_tags'),
                'direction' => $oParse->cleanTags($aImport['instructions'], 'global.allowed_tags')
            );

            $aCategories = $oMochi->getCategory($aImport);
            $aTags = $oMochi->getTags($aImport['tags']);

            $oGameTagProcess->add($aTags, $iGame);
        }
        else
        {
            $aText = array(
                'title' => $oParse->removeTags(html_entity_decode($aGame['title'])),
                'description' => $oParse->cleanTags(html_entity_decode($aGame['description']), 'global.allowed_tags'),
                'direction' => $oParse->cleanTags(html_entity_decode($aGame['directions']), 'global.allowed_tags')
            );

            if( $aGame['base_category'] )
            {
                $aCategories = explode(',', $aGame['base_category']);
                $aCategories = $oMochi->getCategory(array(
                    'category' => $aCategories
                ));
            }
        }

        if( isset($aCategories) )
        {
            foreach($aCategories as $aCategory)
            {
                $aSql = array_merge(array('game_id' => $iGame), $aCategory);

                $oDb->insert('game_category_map', $aSql);
            }

            unset($aCategories);
        }

        foreach($aText as $sVarName => $sText)
        {
            $aVal = array(
                'game_id' => $iGame,
                'var_name' => $sVarName,
                'text' => array_fill_keys($aLanguages, $sText)
            );

            $oLangGameProcess->add($aVal);
        }
    }

    $sHead = '<meta http-equiv="refresh" content="2;url=./update.php?'.$sPageCurrent.'&import=1&page='.($iPage+1).'">';
}
?>

<?php
/**
 * Header Include
 */
require_once(SAA_DIR . 'install' . DS . 'page' . DS . 'include' .  DS . 'header.php');

if( $bImport )
{
?>

<div class="goodBox">
    Importing Games.... Stage <?php echo $iPage; ?>
</div>

<?php
}
else
{
?>
<div class="goodBox">
    Migrate games from legacy table. Click start to begin.
</div>

<div class="spacer"></div>

<div class="p_t_8 m_b_4" align="right">
    <a href="./update.php?<?=$sPageNext?>" class="link_box" id="js_next_btn">Skip</a>
    <a href="./update.php?<?=$sPageCurrent?>&import=1" class="link_box" id="js_next_btn">Start</a>
</div>
<?php
}

/**
 * Footer Include
 */
require_once(SAA_DIR . 'install' . DS . 'page' . DS . 'include' .  DS . 'footer.php');
?>