<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Migrate Scores Page
 * @license GoLevel LLC
 */

require_once(SAA_SYSTEM_INCLUDE . 'app.php');
require_once(SAA_DIR . 'system' . DS . 'library' . DS . 'model.class.php');

//Load settings from db then cache
SAA_Sys::getLib('setting')->build();

$sHeadTitle = 'Migrating Games';

$oDb = SAA_Sys::getLib('database');

$iLimit = 50;
$iPage = (isset($_GET['page']) ? (int) $_GET['page'] : 0);
$bImport = (isset($_GET['import']) ? (int) $_GET['import'] : 0);

if( $bImport )
{
	//If there is no legacy score table, move to next stage
	if( !$oDb->togglePrefix(false)->tableExists('arcade_scores') )
	{
		header('Location: ./update.php?'.$sPageNext);
        exit;
	}
	//If there is no legacy game table, move to next stage
	elseif( !$oDb->tableExists('arcade_games') )
	{
		header('Location: ./update.php?'.$sPageNext);
        exit;
	}
	//If there is no legacy user table, move to next stage
	elseif( !$oDb->tableExists('arcade_users') )
	{
		header('Location: ./update.php?'.$sPageNext);
        exit;
	}

	if( !$oDb->togglePrefix(true)->isColumn('game', 'old_id') )
    {
        $oDb->alterTable('add', 'game', array(
            'old_id INT( 12 ) UNSIGNED NOT NULL'
        ));
    }

	if( !$oDb->isColumn('user', 'old_id') )
    {
        $oDb->alterTable('add', 'user', array(
            'old_id INT( 12 ) UNSIGNED NOT NULL'
        ));
    }

	if( !$oDb->isColumn('score', 'old_id') )
    {
        $oDb->alterTable('add', 'score', array(
            'old_id INT( 12 ) UNSIGNED NOT NULL'
        ));
    }

    $aOldScores = $oDb->select('old_scores.*')
        ->togglePrefix(false)
        ->from('arcade_scores', 'old_scores')
        ->togglePrefix(true)
        ->leftjoin('score', 'score', 'score.old_id = old_scores.score_id')
        ->where('score.score_id IS NULL')
        ->limit($iLimit, $iPage)
        ->get('rows');

    if( !$aOldScores )
    {
        header('Location: ./update.php?'.$sPageNext);
        exit;
    }

	/**
	 * Gather old game ids
	 */
	$aOldGameIds = array();

	foreach($aOldScores as $aOldScore)
    {
		$aOldGameIds[] = (int) $aOldScore['game_id'];
	}

	/**
	 * Gather old user ids
	 */
	$aOldUserIds = array();

	foreach($aOldScores as $aOldScore)
    {
		$aOldUserIds[] = (int) $aOldScore['user_id'];
	}

	/**
	 * Map old game ids to v2 game ids
	 */
	$aMapGames = array();

	$aGames = $oDb->select('game_id, old_id')
        ->from('game')
        ->where('old_id IN('.implode(',', $aOldGameIds).')')
        ->get('rows');

	foreach($aGames as $aGame)
	{
		$aMapGames[$aGame['old_id']] = $aGame['game_id'];
	}

	/**
	 * Map old user ids to v2 user ids
	 */
	$aMapUsers = array();

	$aUsers = $oDb->select('user_id, old_id')
        ->from('user')
        ->where('old_id IN('.implode(',', $aOldUserIds).')')
        ->get('rows');

	foreach($aUsers as $aUser)
	{
		$aMapUsers[$aUser['old_id']] = $aUser['user_id'];
	}

	$aSql = array();

	foreach($aOldScores as $aOldScore)
    {
		//Is the old game id reference to a v2 game id?
		if( !isset($aMapGames[$aOldScore['game_id']]) )
		{
			continue;
		}
		//Is the old user id reference to a v2 user id?
		elseif( !isset($aMapUsers[$aOldScore['user_id']]) )
		{
			continue;
		}

		$aSql[] = array(
			$aOldScore['score_id'],
            $aMapGames[$aOldScore['game_id']],
            $aMapUsers[$aOldScore['user_id']],
            $aOldScore['score'],
            $aOldScore['time']
        );
	}

	if( $aSql )
    {
        $oDb->insertMultiple('score', array(
            'old_id',
            'game_id',
            'user_id',
            'score',
            'time_scored'
        ), $aSql);
    }

    $sHead = '<meta http-equiv="refresh" content="2;url=./update.php?'.$sPageCurrent.'&import=1&page='.($iPage+1).'">';
}
?>

<?php
/**
 * Header Include
 */
require_once(SAA_DIR . 'install' . DS . 'page' . DS . 'include' .  DS . 'header.php');

if( $bImport )
{
?>

<div class="goodBox">
    Importing Scores.... Stage <?php echo $iPage; ?>
</div>

<?php
}
else
{
?>
<div class="goodBox">
    Migrate scores from legacy table. Click start to begin.
</div>

<div class="spacer"></div>

<div class="p_t_8 m_b_4" align="right">
    <a href="./update.php?<?=$sPageNext?>" class="link_box" id="js_next_btn">Skip</a>
    <a href="./update.php?<?=$sPageCurrent?>&import=1" class="link_box" id="js_next_btn">Start</a>
</div>
<?php
}

/**
 * Footer Include
 */
require_once(SAA_DIR . 'install' . DS . 'page' . DS . 'include' .  DS . 'footer.php');
?>