<?php

/**
 *
 * @author Hutton
 * @name install/index.php
 * @license GoLevel LLC
 */

// Make sure we are running PHP5
if (version_compare(phpversion(), '5', '<') === true)
{
    exit('SAA needs PHP 5 or newer.');
}

define('Trapper', true);

define('SAA_ON', TRUE);

define('DS', DIRECTORY_SEPARATOR); //Directory Seperator

define('SAA_DIR', dirname(dirname(__FILE__)) . DS); //Install Directory

define('SAA_IS_AJAX', false);

define('SAA_DEBUG', true);

//Load all constants
require_once(SAA_DIR . 'system' . DS . 'include' . DS . 'const.php');

//Load classes
require_once(SAA_DIR . 'system' . DS . 'library' . DS . 'debug.class.php');
require_once(SAA_DIR . 'system' . DS . 'library' . DS . 'error.class.php');
require_once(SAA_DIR . 'system' . DS . 'library' . DS . 'system.class.php');

//Error
error_reporting(E_ALL | E_STRICT);

//Error handler
set_error_handler(array('SAA_Debug', 'handle'));

//Set time zone
if( function_exists('date_default_timezone_set') )
{
	date_default_timezone_set('America/New_York');
}

//Page
$_sPage = (isset($_GET['c']) ? $_GET['c'] : 'index');

//Allowed Pages
$_aPages = array();

//Version
$_sUpdateVersion = (isset($_GET['version']) ? $_GET['version'] : '1.3');

//Load version history
require_once(SAA_DIR . 'install' . DS . 'include' . DS . 'versions.php');

//Set update variables
$_aUpdateVars = array();
$_aUpdateVars['skip_permissions'] = true;
$_aUpdateVars['skip_db_setup'] = true;

//From v1.x to v2.0.0
if( version_compare('2.0.0', $_sUpdateVersion) == 1 )
{
	$_aUpdateVars['skip_permissions'] = false;
	$_aUpdateVars['skip_db_setup'] = false;

	$_sTargetVersion = '2.0.0';
}
//To v2.0.1
elseif( version_compare('2.0.1', $_sUpdateVersion) == 1 )
{
	$_sTargetVersion = '2.0.1';
}
//To v2.0.2 
elseif( version_compare('2.0.2', $_sUpdateVersion) == 1 )
{
	$_sTargetVersion = '2.0.2';
}
//Up-to-date
else
{
	require_once(SAA_DIR . 'install' . DS . 'page' . DS . 'update' . DS . 'finished.php');
	exit;
}

//Load the step file
if( !file_exists(SAA_DIR . 'install' . DS . 'page' . DS . 'update' . DS . 'v' . $_sTargetVersion . DS . '_step.php') )
{
	exit('Step file cannot be found.');
}

include(SAA_DIR . 'install' . DS . 'page' . DS . 'update' . DS . 'v' . $_sTargetVersion . DS . '_step.php');

//Get the page and step key
$_iStepKey = -1;

foreach($_aPages as $i => $aPage)
{
	if( $aPage[0] == $_sPage )
	{
		$_iStepKey = $i;

		$sFile = $aPage[1];

		break;
	}
}

//If there is no step key aka no page
if( $_iStepKey == -1 )
{
	exit('Step cannot be found.');
}

//Load the page
if( !file_exists(SAA_DIR . 'install' . DS . 'page' . DS . 'update' . DS . $sFile . '.php') )
{
	exit('Page file cannot be found.');
}

$sPageCurrent = 'c='.$_sPage.'&version='.$_sUpdateVersion;
$sPageNext = pageNext();

require_once(SAA_DIR . 'install' . DS . 'page' . DS . 'update' . DS . $sFile . '.php');

function pageNext()
{
	global $_aPages;
	global $_aUpdateVars;
	global $_aVersions;
	global $_iStepKey;
	global $_sTargetVersion;
	global $_sUpdateVersion;

	$iStepKey = $_iStepKey;
	
	$sNextStep = '';
	$sNextVersion = '';

	do
	{
		//If there is a next step
		if( isset($_aPages[$iStepKey + 1]) )
		{
			//If the next page is a permission
			if( $_aPages[$iStepKey + 1][0] == 'permissions' || $_aPages[$iStepKey + 1][0] == 'permissions-revert' )
			{
				//If the permisssion pages are skipped
				if( $_aUpdateVars['skip_permissions'] )
				{
					$iStepKey++;

					continue;
				}
			}
			//If the next page the db setup
			elseif( $_aPages[$iStepKey + 1][0] == 'db-setup' )
			{
				//If the db-setup page is skipped
				if( $_aUpdateVars['skip_db_setup'] )
				{
					$iStepKey++;

					continue;
				}
			}

			$sNextStep = $_aPages[$iStepKey + 1][0];
		}
		//If there is no next step, set it as finished
		else
		{
			$sNextStep = 'finished';
		}
	}
	while( !$sNextStep );

	//If the next step is the finished step
	if( $sNextStep == 'finished' )
	{
		$mNewerVersion = false;

		foreach($_aVersions as $i => $sVersion)
		{
			//Are there newer versions needing to be loaded?
			if( version_compare($sVersion, $_sTargetVersion) == 1 )
			{
				$mNewerVersion = $i;

				break;
			}
		}

		//If there is a newer version
		if( $mNewerVersion )
		{
			$sNextStep = 'index';
			$sNextVersion = '&version='.$_aVersions[$mNewerVersion - 1].'&header=1';
		}
		else
		{
			$sNextVersion = '&version='.$_sUpdateVersion;
		}
	}
	else
	{
		$sNextVersion = '&version='.$_sUpdateVersion;
	}

	return 'c='.$sNextStep.$sNextVersion;
}
?>