<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Ajax Class
 * @license GoLevel LLC
 */

Class SAA_Ajax
{
    private static $_aCalls = array();

    private static $_aMessages = array();

    private static $_aErrorParam = array('display' => 'dialog', 'location' => '#ajax-dialog-message');

    private static $_aMessageParam = array('display' => 'dialog', 'location' => '#ajax-dialog-message', 'auto_close' => 3000);

    private $_oRequest;

    private $_aCallable = array(
        'addClass',
        'after',
        'append',
        'attr',
        'delay',
        'fadeOut',
        'hide',
        'html',        
        'remove',
        'removeAttr',
        'removeClass',
        'show',
        'slideDown',
        'trigger',
        'val'
    );

    public function __call($sMethod, $aArguments)
    {
        if( !in_array($sMethod, $this->_aCallable) )
        {
            return SAA_Debug::trigger('Not a safe jQuery function');
        }

        $this->_call('$(\'' . $aArguments[0] . '\').' . $sMethod . '(' . $this->_getArguments($aArguments) . ');');

        return $this;
    }

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $this->_oRequest = SAA_Sys::getLib('request');
    }

    /**
     * Adds jquery content
     *
     * @param string $sCall
     * @return SAA_Ajax
     */
    public function _call($sCall)
    {
        self::$_aCalls[] = $sCall;

        return $this;
    }

    /**
     * Escapes a string for javascript
     * 
     * @param string $sStr 
     * @return string 
     */
    public function _escape($sStr)
    {
        return str_replace("'", "\'", $sStr);
    }

    public function _load()
    {
        $sCall = $this->_oRequest->get('ajaxProcess');

        if( substr_count($sCall, '.') == 0 )
        {
            SAA_Debug::trigger('Ajax process is not valid.');
        }

        $iMiddle = strrpos($sCall, '.');
	$iEnd = strlen($sCall);

        $sClass = trim(substr($sCall, 0, $iMiddle));
	$sFunction = trim(substr($sCall, $iMiddle, $iEnd), '.');

        $sObj = SAA_Sys::loadClass(SAA_APP_AJAX_PROCESS, $sClass, 'Ajax');

        $oProcess = new $sObj;

        if( $this->_request()->get('json') )
        {
            echo json_encode($oProcess->$sFunction());

            return true;
        }

        $oProcess->$sFunction();

        if( !$this->_loadErrors() )
        {
            $this->_loadMessages();
        }

        $sXml = '';
        foreach (self::$_aCalls as $sCall)
        {
            $sXml .= $this->_clean($sCall);
        }
        echo $sXml;
    }

    /**
     * Loads an ajax controller
     *
     * @param string $sController controller name
     * @param array $aStored data to pass to controller
     * @return string
     */
    public function _loadController($sController, $aStored = array())
    {
	$sContoller = SAA_Sys::load($sController, $aStored);

        SAA_Error::clear();

        return $sContoller;
    }

    public function _loadErrors()
    {
        $aErrors = SAA_Error::get();

        if( !$aErrors )
        {
            return false;
        }

        $sCall = 'oAjaxErrors = SAA.error; oAjaxErrors.ini({type: \''.self::$_aErrorParam['display'].'\', location: \''.self::$_aErrorParam['location'].'\'});';

        foreach($aErrors as $sError)
        {
            $sCall .= 'oAjaxErrors.add(\''.  addslashes($sError).'\');';
        }

        $sCall .= 'oAjaxErrors.show().clear();';

        $this->_call($sCall);

        return true;
    }

    public function _loadMessages()
    {
        if( !self::$_aMessages )
        {
            return false;
        }

        $sCall = 'oAjaxMsg = SAA.message; oAjaxMsg.ini({type: \''.self::$_aMessageParam['display'].'\', location: \''.self::$_aMessageParam['location'].'\''.(self::$_aMessageParam['auto_close'] ? ', auto_close: 3000' : '').'});';

        foreach(self::$_aMessages as $sMessage)
        {
            $sCall .= 'oAjaxMsg.add(\''.  addslashes($sMessage).'\');';
        }

        $sCall .= 'oAjaxMsg.show();';

        $this->_call($sCall);
    }

    public function _request()
    {
        return $this->_oRequest;
    }

    public function _setErrorParam($aParam)
    {
        foreach($aParam as $sKey => $sVal)
        {
            self::$_aErrorParam[$sKey] = $sVal;
        }
    }

    public function _setMessageParam($aParam)
    {
        foreach($aParam as $sKey => $sVal)
        {
            self::$_aMessageParam[$sKey] = $sVal;
        }
    }

    /**
     *
     * @param string $sMsg content to add in the alert box
     * @return SAA_Ajax
     */
    public function alert($sMsg)
    {
        return $this->_call('alert(\''.  addslashes($sMsg).'\');');
    }

    /**
     *
     * @param string $sError error to add
     * @param array $aParam param array for messages
     * @return SAA_Ajax
     */
    public function error($sError, $aParam = array())
    {
        if( $aParam )
        {
            $this->_setErrorParam($aParam);
        }

        SAA_Error::add($sError);

        return $this;
    }

    public function isAdmin()
    {
        if( !SAA_Sys::isAdmin() )
        {
            $this->error('You do not have permission to preform this action.');

            return false;
        }

        return true;
    }

    /**
     *
     * @param string $sMsg message to add
     * @param array $aParam param array for messages
     * @return SAA_Ajax
     */
    public function message($sMsg, $aParam = array())
    {
        if( $aParam )
        {
            $this->_setMessageParam($aParam);
        }

        self::$_aMessages[] = $sMsg;

        return $this;
    }

    /**
     *
     * @param string $sStr converts the string to clean value
     * @return string
     */
    private function _clean($sStr)
    {
        return str_replace(array("\n", "\r"), '\\n', $sStr);
    }

    private function _getArguments($aArguments)
    {
        $sArgs = '';

        foreach($aArguments as $iKey=>$sArgument)
        {
            if( $iKey == 0 )
            {
                continue;
            }

            if( is_int($sArgument) || strpos($sArgument, 'function()') === 0 )
            {
                $sValue = $sArgument;
            }
            else
            {
                $sValue = '\'' . str_replace("'", "\'", $sArgument) . '\'';
            }

            if( is_bool($sArgument) )
            {
                $sValue = ($sArgument === true ? 'true' : 'false');
            }

            $sArgs .= $sValue . ', ';
        }
        return rtrim($sArgs, ', ');
    }
}
?>
