<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Api Facebook Library Class
 * @license GoLevel LLC
 */

Class SAA_Api_Facebook
{
    private $_oApi = null;

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        require_once(SAA_SYSTEM_3PARTY . 'Facebook' . DS . 'facebook.php');

        if( strlen(SAA_Sys::getSett('facebook.app_id')) && strlen(SAA_Sys::getSett('facebook.secret')) )
        {
            $this->_oApi = new Facebook(array(
                'appId'  => SAA_Sys::getSett('facebook.app_id'),
                'secret' => SAA_Sys::getSett('facebook.secret')
            ));
        }
    }

    /**
     * Preforms an api call to facebook
     *
     * @return mixed
     */
    public function call()
    {
        $aArgs = func_get_args();

        return call_user_func_array(array($this->_oApi, 'api'), $aArgs);
    }

    /**
     * Gets the current user's facebook id
     *
     * @return mixed
     */
    public function getUserId()
    {
        return $this->_oApi->getUser();
    }

    /**
     * Gets the current user's facebook data
     *
     * @return mixed
     */
    public function getUser()
    {
        $iUser = $this->getUserId();

        if( !$iUser )
        {
            return false;
        }

		$aUser = $this->_oApi->api('/me');

        return $aUser;
    }

    /**
     * Gets the facebook login url
     *
     * @return string
     */
    public function getLoginUrl()
    {
        return $this->_oApi->getLoginUrl(array('scope' => 'email'));
    }

    /**
     * Gets the facebook logout url
     *
     * @return string
     */
    public function getLogoutUrl()
    {
        return $this->_oApi->getLogoutUrl(array('next' => SAA_Sys::getLib('url')->makeSimple('login')));
    }

    /**
     * Removes the related sessions/cookies on local
     */
    public function logout()
    {
        $this->_oApi->destroySession();
    }
}
?>
