<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Cache Driver File Library Class
 * @license GoLevel LLC
 */

Class SAA_Cache_Driver_File
{
    /**
     * Class Constructor
     */
    public function __construct()
    {

    }

    /*
     * Creates the cache file
     *
     * Note: mExpire can use to the current time as a reference by using '+' prepended to the integer value of seconds.
     * Example: $mExpire = '+60' would have an expire time of 1 min after the current time. By default the mExpire will treat value as a time stamp.
     *
     * @param string sName name of the cache file
     * @param mixed mVal content to store
     * @param mixed mExpire value to expire the cache file
     *
     */
    public function create($sName, $mVal, $mExpire = null)
    {
	$this->remove($sName);

	$sText = '<?php defined(\'Trapper\') or exit(\'Trespasser!\'); ' . "\n";

        if( $mExpire )
        {
            if( strpos($mExpire, '+') !== false )
            {
                $mExpire = SAA_TIME + $mExpire;
            }

            $sText .= '$iExpire = '. (int) $mExpire .';';
        }

	$sText .= '$mVal = ' . var_export($mVal, true) . ';';
	$sText .= "\n" . ' ?>';

	return SAA_Sys::getLib('file')->write($this->_getLocation($sName), $sText, 0644);
    }

    /*
     * Gets data from cache file
     *
     * @param string sName name of the cache file
     *
     */
    public function get($sName)
    {
        $sLocation = $this->_getLocation($sName);

        if( !file_exists($sLocation) )
        {
            return false;
        }

	require($sLocation);

	if( !isset($mVal) )
	{
	    return false;
	}

        if( isset($iExpire) )
        {
            if( $iExpire < SAA_TIME )
            {
                $this->remove($sName);

                return false;
            }
        }

	return $mVal;
    }


    /**
     * Removes a file from the cache
     *
     * @param string $mName name of the cache file
     * @return bool
     */
    public function remove($mName)
    {
        $oFile = SAA_Sys::getLib('file');

        if( is_string($mName) )
        {
            $mName = (array) $mName;
        }

        foreach($mName as $sName)
        {
            $sFile = $this->_getLocation($sName);

            if( !file_exists($sFile) )
            {
                continue;
            }

            $oFile->delete($sFile);
        }

        return true;
    }

    /**
     * Removes all of the cache files
     */
    public function removeAll()
    {
        SAA_Sys::getLib('file')->cleanDir(SAA_APP_CACHE);
    }

    /**
     * Removes any files that have a given pattern
     * 
     * @param string $sLike pattern
     * @return int number of files removed
     */
    public function removeLike($sLike)
    {
        $oFile = SAA_Sys::getLib('file');

        $iRemove = 0;
        $aFiles = glob(SAA_APP_CACHE . $sLike . '*.php');

        foreach($aFiles as $sFile)
        {
            if( $oFile->delete($sFile) )
            {
                $iRemove++;
            }
        }

        return $iRemove;
    }

    /**
     * Get the actual name and location of the file
     *
     * @param string $sName name of the file
     * @return string
     */
    private function _getLocation($sName)
    {
	return SAA_APP_CACHE . $sName . '.php';
    }
}
?>
