<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Category Library Class
 * @license GoLevel LLC
 */

Class SAA_Category
{
    private $_aSlugs = array();
    private $_aCategories = array();
    private $_aMap = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $oCache = SAA_Sys::getLib('cache');

        if( ($aCategories = $oCache->get('category')) )
        {
            $this->_aSlugs = $aCategories['slug'];
            $this->_aCategories = $aCategories['categories'];
        }
        else
        {
            $aCategories = SAA_Sys::getModel('category')->getAll();

            $this->_getSlugs($aCategories);
            $this->_get($aCategories);

            $aCategories = array(
                'slug' => $this->_aSlugs,
                'categories' => $this->_aCategories
            );

            $oCache->create('category', $aCategories);
        }

        if( !($aMap = $oCache->get('category_map')) )
        {
            $this->_aMap = SAA_Sys::getModel('category.map')->getKeys();

            $oCache->create('category_map', $this->_aMap);
        }

        $this->_aMap = $aMap;
    }

    /**
     * Gets a category by id or slug
     *
     * @param string $mKey
     * @param bool $bId
     * @return mixed
     */
    public function get($mKey, $bId = true)
    {
        if( !$bId )
        {
            if( !array_key_exists($mKey, $this->_aSlugs) )
            {
                return false;
            }

            $mKey = $this->_aSlugs[$mKey];
        }

        return $this->_aCategories[$mKey];
    }

    /**
     * Gets all categories
     * 
     * @return array
     */
    public function getAll()
    {
       return $this->_aCategories;
    }

    /**
     * Get all categories with the category id as key and name as value
     *
     * @return array
     */
    public function getAllSimple()
    {
        $aCategories = array();

        foreach($this->_aCategories as $iId => $aCategory)
        {
            $aCategories[$aCategory['slug']] = SAA_Sys::getPhrase($aCategory['name']);
        }

        return $aCategories;
    }

    /*
     * Gets the category maps associated by category id
     *
     */
    public function getMap($iCategory)
    {
        if( !isset($this->_aMap[$iCategory]) )
        {
            return array();
        }

        return $this->_aMap[$iCategory];
    }

    /**
     * Builds $this->_aCategories array
     *
     * @param array $aCategories
     */
    private function _get($aCategories)
    {
        foreach($aCategories as $aCategory)
        {
            $this->_aCategories[$aCategory['category_id']] = $aCategory;
        }
    }

    /**
     * Builds $this->_aSlugs array
     *
     * @param array $aCategories 
     */
    private function _getSlugs($aCategories)
    {
        foreach($aCategories as $aCategory)
        {
            $this->_aSlugs[$aCategory['slug']] = $aCategory['category_id'];
        }
    }
}
?>
