<?php
defined('Trapper') or exit('Trespasser!');

/**
*
* @author Hutton
* @name Database Driver Mysql Libaray Class
* @license GoLevel LLC
*/

class SAA_Database_Driver_Mysql extends SAA_Database_Abstraction
{
	private $_rLink = null;
	private $_aTables = array();

	/**
	* Alters a given table
	*
	* @param string $sType
	* @param string $sTable
	* @param array $aFields
	* @param string $sAfter
	* @return resource
	*/
	public function alterTable($sType, $sTable, $aFields, $sAfter = '')
	{
		$sSql = 'ALTER TABLE '. $this->table($sTable) ." $sType ";

		$sSql .= implode(', ', $aFields);

		if( $sAfter != '' )
		{
			$sSql .= ' AFTER ' . $sAfter;
		}

		return $this->query($sSql);
	}

	/**
	* Connects to the mysql database
	*
	* @param string $sHost
	* @param string $sUser
	* @param string $sPass
	* @param string $sName
	* @param string $sPort
	* @return resource
	*/
	public function connect($sHost, $sUser, $sPass, $sName, $sPort = false)
	{
		if( $sPort )
		{
			$sHost = $sHost . ':' . $sPort;
		}

		$this->_rLink = @mysql_connect($sHost, $sUser, $sPass);

		if( !$this->_rLink )
		{
			return false;
		}

		return @mysql_select_db($sName, $this->_rLink);
	}

	public function createTable($sTable, $aFields, $sPrimary = '', $aKeys = array(), $aUniques = array(), $bIfExists = true)
	{
		$sSql = 'CREATE TABLE ';

		if( $bIfExists === TRUE )
		{
			$sSql .= 'IF NOT EXISTS ';
		}

		$sSql .= $this->table($sTable) . ' (';

		foreach($aFields as $sField => $aField)
		{
			$sSql .= $sField . ' ' . $aField['type'] . (!empty($aField['constraint']) ? ' (' . $aField['constraint'] . ')' : '').' '.(!empty($aField['unsigned']) ? ' unsigned ' : '') . (!empty($aField['null']) ? ' NULL ' : ' NOT NULL ') . (isset($aField['default']) ? ' default "'.$aField['default'].'"' : '') . (!empty($aField['auto_increment']) ? ' auto_increment' : '') . ', ';
		}

		if( $sPrimary )
		{
			$sSql .= "PRIMARY KEY (".$sPrimary."), ";
		}

		if( $aKeys )
		{
			foreach ($aKeys as $sKey => $mKey)
			{
				if( is_array($mKey) )
				{
					$sSql .= "KEY {$sKey} (". implode(',', $mKey) ."), ";
				}
				else
				{
					$sSql .= "KEY {$mKey} (" . $mKey . "), ";
				}
			}
		}

		if( $aUniques )
		{
			foreach ($aUniques as $sKey => $mKey)
			{
				if( is_array($mKey) )
				{
					$sSql .= "UNIQUE {$sKey} (". implode(',', $mKey) ."), ";
				}
				else
				{
					$sSql .= "UNIQUE {$mKey} (" . $mKey . "), ";
				}
			}
		}

		$sSql = rtrim(trim($sSql), ',');

		$sSql .= "\n) ENGINE=MyISAM DEFAULT CHARSET=latin1;";

		return $this->query($sSql);
	}

	/*
	* Drops table from database
	*
	* @param string sTable table name
	*
	* @return mixed
	*/
	public function dropTable($sTable)
	{
		return $this->query("DROP TABLE IF EXISTS " . $this->table($sTable));
	}

	/*
	* Escapes value(s)
	*
	* @param mixed mParam string/array of values
	*
	* @return mixed
	*/
	public function esc($mParam)
	{
		if( is_array($mParam) )
		{
			return array_map(array($this, 'esc'), $mParam);
		}

		if( get_magic_quotes_gpc() )
		{
			$mParam = stripslashes($mParam);
		}

		$mParam = mysql_real_escape_string($mParam);

		return $mParam;
	}

	/*
	* Gets the extreme value
	*
	* @param string sTable table
	* @param string sCol column name
	* @param string sCondition where conditionals
	* @param bool bHighest if set to true then get highest value, else set to false get lowest value
	*
	* @return string
	*/
	public function getExtremeVal($sTable, $sCol, $sCondition = '1', $bHighest = true)
	{
		$sCol = ($bHighest ? 'MAX('.$sCol.')' : 'MIN('.$sCol.')');

		return $this->_getField('SELECT '. $sCol .' FROM ' . $this->table($sTable) . ' WHERE ' . $sCondition);
	}

	/*
	* Returns the id of the last inserted row
	*/
	public function getLastId()
	{
		return @mysql_insert_id($this->_rLink);
	}

	/**
	* Gets the list of tables
	* 
	* @return array
	*/
	public function getTables()
	{
		$this->_aTables = $this->_getRows('SHOW TABLE STATUS', true);

		return $this->_aTables;
	}

	/*
	* Insert row into table
	*
	* @param string sTable table
	* @param array aValues the keys are column name and values are data
	* @param bool bEsc escape values
	*
	* @return int
	*/
	public function insert($sTable, $aValues = array(), $bEsc = true)
	{
		$sValues = '';

		$sFields = implode(', ', array_keys($aValues));

		if( $bEsc )
		{
			foreach ($aValues as $mValue)
			{
				if( is_null($mValue) )
				{
					$sValues .= "NULL, ";
				}
				else
				{
					$sValues .= "'" . $this->esc($mValue) . "', ";
				}
			}
		}
		else
		{
			foreach ($aValues as $mValue)
			{
				if( is_null($mValue) )
				{
					$sValues .= "NULL, ";
				}
				else
				{
					$sValues .= "'" . $mValue . "', ";
				}
			}
		}

		$sValues = rtrim(trim($sValues), ',');

		if( ($rResult = $this->query('INSERT INTO ' . $this->table($sTable) . ' (' . $sFields . ') VALUES (' . $sValues . ')')) )
		{
			return $this->getLastId();
		}

		return 0;
	}

	/*
	* Insert multiple rows into table
	*
	* @param string sTable table
	* @param array aCols values are column names
	* @param array aValues child arrays are in the same key order as aCols
	* @param bool bEsc escape values
	*
	* @return bool
	*/
	public function insertMultiple($sTable, $aCols, $aValues, $bEsc = true)
	{
		$sValues = '';

		$sFields = implode(', ', $aCols);

		if( $bEsc )
		{
			foreach ($aValues as $aValue)
			{
				$sValue = '';
				$sValues .= '(';

				foreach($aValue as $mValue)
				{
					if( is_null($mValue) )
					{
						$sValue .= "NULL, ";
					}
					else
					{
						$sValue .= "'" . $this->esc($mValue) . "', ";
					}
				}

				$sValues .= rtrim($sValue, ', ') . '), ';
			}
		}
		else
		{
			foreach ($aValues as $aValue)
			{
				$sValue = '';
				$sValues .= '(';

				foreach($aValue as $mValue)
				{
					if( is_null($mValue) )
					{
						$sValue .= "NULL, ";
					}
					else
					{
						$sValue .= "'" . $mValue . "', ";
					}
				}

				$sValues .= rtrim($sValue, ', ') . '), ';
			}
		}

		$sValues = rtrim(trim($sValues), ',');

		if( ($rResult = $this->query('INSERT INTO ' . $this->table($sTable) . ' (' . $sFields . ') VALUES ' . $sValues)) )
		{
			return true;
		}

		return false;
	}

	/**
	*
	* @param string $sTable
	* @param string $sCol
	* @return bool
	*/
	public function isColumn($sTable, $sCol)
	{
		$aRows = $this->getRows('SHOW COLUMNS FROM '.$this->table($sTable));
		foreach($aRows as $aRow)
		{
			if( strtolower($aRow['Field']) == strtolower($sCol) )
			{
				return true;
			}
		}

		return false;
	}

	/*
	* Query database
	*
	* @param string sSql query
	*
	* @return mixed
	*/
	public function query($sSql)
	{
		$rResult = @mysql_query($sSql, $this->_rLink);

		if( !$rResult )
		{
			SAA_Debug::trigger('Query Error:' . $this->_sqlError(), (SAA_DEBUG ? E_USER_ERROR : E_USER_WARNING));
		}

		return $rResult;
	}

	/*
	* Rename table
	*
	* @param string sTable old name
	* @param string sTableNew new name
	*
	* @return mixed
	*/
	public function renameTable($sTable, $sTableNew)
	{
		return $this->query('ALTER TABLE '. $this->table($sTable) ." RENAME TO ". $this->table($sTableNew));
	}

	/**
	* Determines if a given table exists.
	*
	* @param string $sTable
	* @return bool
	*/
	public function tableExists($sTable)
	{
		$aTables = ($this->_aTables ? $this->_aTables : $this->getTables());

		foreach ($aTables as $aTable)
		{
			if ($aTable['Name'] == $this->table($sTable))
			{
				return true;
			}
		}

		return false;
	}

	/*
	* Truncate table
	*
	* @param string sTable table
	*
	* @return mixed
	*/
	public function truncateTable($sTable)
	{
		return $this->query('TRUNCATE TABLE '. $this->table($sTable));
	}

	/*
	* Update rows in table
	*
	* @param string sTable table
	* @param array aValues the keys are column name and values are data
	* @param string sCondition where conditionals
	* @param bool bEsc escape values
	*
	* @return mixed
	*/
	public function update($sTable, $aValues = array(), $sCondition = null, $bEsc = true)
	{
		$sSets = '';

		foreach($aValues as $sCol => $sValue)
		{
			$sSets .= "{$sCol} = " . (is_null($sValue) ? 'NULL' : ($bEsc ? "'" . $this->esc($sValue) . "'" : $sValue)) . ", ";
		}

		$sSets[strlen($sSets)-2] = '  ';

		return $this->query('UPDATE ' . $this->table($sTable) . ' SET ' . $sSets . ($sCondition ? ' WHERE ' . $sCondition : ''));
	}

	/*
	* Update counters for given rows using conditionals
	*
	* @param string sTable table
	* @param string sCol column to update counter
	* @param string sMath arithmetic
	* @param string sValue value
	* @param string sCondition where conditionals
	*
	* @return mixed
	*/
	public function updateCounter($sTable, $sCol, $sMath, $sValue, $sCondition)
	{
		return $this->query('UPDATE ' . $this->table($sTable) . ' SET ' . $sCol . ' = '. $sCol . ' ' . $sMath . ' ' . $sValue . ' WHERE ' . $sCondition);
	}

	/*
	* Get a column value from a row
	*
	* @param string sSql query
	*
	* @return string
	*/
	protected function _getField($sSql)
	{
		$sColumn = '';

		if( ($aRow = $this->_getRow($sSql, false, $this->_rLink)) )
		{
			$sColumn = $aRow[0];
		}

		return $sColumn;
	}

	/*
	* Get a row
	*
	* @param string sSql query
	* @param bool bAssoc whether or not the array is associative
	*
	* @return array
	*/
	protected function _getRow($sSql, $bAssoc)
	{
		$hRes = $this->query($sSql, $this->_rLink);

		$aResult =  mysql_fetch_array($hRes, ($bAssoc ? MYSQL_ASSOC : MYSQL_NUM));

		return $aResult ? $aResult : array();
	}

	/*
	* Get rows
	*
	* @param string sSql query
	* @param bool aAssoc whether or the row arrays are associative
	* @param string sKey use a column value as key
	*
	* @return array
	*/
	protected function _getRows($sSql, $bAssoc = true, $sKey = null)
	{
		$bAssoc = ($bAssoc ? MYSQL_ASSOC : MYSQL_NUM);

		$aRows = array();

		$this->rQuery = $this->query($sSql, $this->_rLink);

		if( !is_null($sKey) )
		{
			while( $aRow = mysql_fetch_array($this->rQuery, $bAssoc) )
			{
				$aRows[$aRow[$sKey]] = $aRow;
			}
		}
		else
		{
			while( $aRow = mysql_fetch_array($this->rQuery, $bAssoc) )
			{
				$aRows[] = $aRow;
			}
		}

		return $aRows;
	}

	/*
	* Return query error
	*
	* @return string
	*/
	private function _sqlError()
	{
		return mysql_error();
	}
}
?>