<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Debug Libaray Class
 * @license GoLevel LLC
 */

Final Class SAA_Debug
{
    /**
     * Pretty up the arguments
     *
     * @param array $aArgs
     * @return string
     */
    public static function getArguments($aArgs)
    {
	foreach ($aArgs as $k=>$mArg)
	{
	    if (is_numeric($mArg))
	    {
		$aArgs[$k] = '<span style="color:#7700AA">'.$mArg.'</span>';
	    }
	    elseif(is_bool($mArg))
	    {
		$aArgs[$k] = '<span style="color:#222288;">'.($mArg ? 'TRUE' : 'FALSE').'</span>';
	    }
	    elseif(is_null($mArg))
	    {
		$aArgs[$k] = '<span style="color:#222288;">NULL</span>';
	    }
	    elseif(is_array($mArg))
	    {
		$aArgs[$k] = 'Array('.count($mArg).')';
	    }
	    elseif (is_string($mArg) && ! (('"' == substr($mArg,0,1)) && ('"' == substr($mArg,-1))))
	    {
		$aArgs[$k] = '<span style="color:#0000FF">"'.$mArg.'"</span>';
	    }
	    elseif(is_object($mArg))
	    {
		unset($aArgs[$k]);
		$aArgs[$k] = '{' . ucfirst(get_class($mArg)) . '}';
	    }
	}
	return implode(', ', $aArgs);
    }

    /**
     * Handles debug errors
     *
     * @param int $iErrNum
     * @param string $sStr
     * @param string $sFile
     * @param int $iLine
     */
    public static function handle($iErrNum, $sStr, $sFile, $iLine)
    {
	$sTxt = "<div style='background-color: #eed8d9; border: 2px solid #cccccc; border-collapse: collapse; margin: 1em; padding: 1em; float: left;'><h4 style='background-color: transparent; color: #006; font-weight: bold;'><strong>Error:</strong> $sStr</h4>";

	//Backtrace the Error
	$aDebug = debug_backtrace();
	$aDebug = array_reverse($aDebug);

	foreach($aDebug as $iCnt=>$aItem)
	{
	    if( !isset($aItem['file']) || $aItem['function'] == 'trigger_error' || $aItem['function'] == 'trigger' )
		    continue;

	    $sArgs = '';
	    if (isset($aItem['args']) && is_array($aItem['args']) )
	    {
		$sArgs = SAA_Debug::getArguments($aItem['args']);
	    }

	    $sEntry = '<b>Function:</b> '.(isset($aItem['class']) ? $aItem['class'] : '').(isset($aItem['type']) ? $aItem['type'] : ''). $aItem['function'] .'('. $sArgs .')<br/>';
	    $sTxt .= "<div><b>Line:</b> {$aItem['line']}<br/>{$sEntry}<b>File:</b> {$aItem['file']}</div><hr/>";
	}

	echo $sTxt."</div><div style='clear: both;'></div>";
	die();
    }

    /**
     * Restores error handler to either default or SAA_Debug::handle
     *
     * @param bool $bTurnOn
     */
    public static function restoreHandler($bTurnOn = false)
    {
	if( !$bTurnOn )
	{
	    restore_error_handler();
	}
	else
	{
	    set_error_handler(array('SAA_Debug', 'handle'));
	}
    }

    /**
     * Turn debug on/off
     *
     * @param string $sSwitch
     */
    public static function toggle($sSwitch = 'off')
    {
	if( $sSwitch == 'off' )
	{
	    error_reporting(0);
	}
	else
	{
	    error_reporting((SAA_DEBUG ? E_ALL | E_STRICT : 0));
	}
    }

    /**
     * Trigger an error
     *
     * @param string $sMsg
     * @param string $sErrorCode
     */
    public static function trigger($sMsg, $sErrorCode = E_USER_WARNING)
    {
	trigger_error(strip_tags($sMsg), $sErrorCode);

	if ($sErrorCode == E_USER_ERROR)
	{
	    exit;
	}

	return false;
    }
}
?>
