<?php
defined('Trapper') or exit('Trespasser!');

/**
*
* @author Hutton
* @name File Libaray Class
* @license GoLevel LLC
*/

Class SAA_File
{
	/**
	* Generates a directory location based on integer
	*
	* @param int $iNum
	* @return string
	*/
	public function buildDir($iNum)
	{
		$iNum = (int) $iNum;

		if( !$iNum )
		{
			$iNum = rand();
		}

		$iParent = (string) floor($iNum/10000) + 1;
		$iChild = (string) (floor($iNum/100) + 1) % 100;

		$sParent = '';
		for($i =0, $iCnt = strlen($iParent); $i < $iCnt; $i++)
		{
			$sParent .= chr(97 + $iParent[$i]);
		}

		$sChild = '';
		for($i =0, $iCnt = strlen($iChild); $i < $iCnt; $i++)
		{
			$sChild .= chr(97 + $iChild[$i]);
		}

		return $sParent . DS . $sChild . DS;
	}

	/**
	* Generates an available file name within a given directory
	*
	* @param string $sDir
	* @param string $sExt
	* @param int $iLen
	* @return string
	*/
	public function buildName($sDir, $sExt, $iLen = 5)
	{
		$oParseInput = SAA_Sys::getLib('parse.input');

		do
		{
			$sFile = $oParseInput->salt($iLen, true). '.' . $sExt;
		}
		while( file_exists($sDir . $sFile) );

		return $sFile;
	}

	/**
	* Removes files from a directory with options to remove the directory and/or children directory/files
	*
	* @param string $sDir
	* @param bool $bDelete
	* @param bool $bDeleteChildren
	*/
	public function cleanDir($sDir , $bDelete = false, $bDeleteChildren = false)
	{
		if( is_dir($sDir) )
		{
			if( ($rHandle = opendir($sDir)) )
			{
				while(($sFile = readdir($rHandle)) !== false)
				{
					if($sFile != '.' && $sFile != '..')
					{
						if(is_dir($sDir . DS . $sFile))
						{
							$this->cleanDir($sDir . DS . $sFile, $bDeleteChildren, $bDeleteChildren);
						}
						else
						{
							$this->delete($sDir . DS . $sFile);
						}
					}
				}
			}

			closedir($rHandle);

			if( $bDelete )
			{
				rmdir($sDir);
			}
		}
	}

	/**
	* Creates any children directories
	* 
	* @param string $sDir
	* @return bool
	*/
	public function createDir($sDir)
	{
		return $this->_createDir($sDir);
	}

	/**
	* Removes a given file
	*
	* @param string $sFile
	* @return bool
	*/
	public function delete($sFile)
	{
		return (@unlink($sFile) ? true : false);
	}

	/**
	* Downloads a given file to user's browser
	*
	* @param string $sFile the file to send to browser
	* @param string $sPretty the filename the user will see
	* @param string $sMimeType file string
	* @param bool $bDelete
	* @return string
	*/
	public function downloadLocal($sFile, $sPretty, $sMimeType = '', $bDelete = false)
	{
		if( ini_get('zlib.output_compression') )
		{
			ini_set('zlib.output_compression', 'Off');
		}

		if( !$sMimeType )
		{
			$sMimeType = $this->mimeType($sFile);
		}

		// Start sending headers
		header("Pragma: public"); // required
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: private", false); // required for certain browsers
		header("Content-Transfer-Encoding: binary");
		header("Content-Type: " . $sMimeType);
		header("Content-Length: " . filesize($sFile));
		header("Content-Disposition: attachment; filename=\"" . $sPretty . "\";" );

		// Send data
		readfile($sFile);

		if( $bDelete )
		{
			$this->delete($sFile);
		}

		exit;
	}

	/**
	* Gets a given file from a remote location with options to download (and option delete from local on download)
	*
	* @param string $sUrl
	* @param string $sDestination
	* @param bool $bDownload
	* @param bool $bDelete
	* @return bool
	*/
	public function downloadRemote($sUrl, $sDestination, $bDownload = false, $bDelete = false)
	{
		$this->_createDir($sDestination);

		$fp = fopen($sDestination, 'w+');

		$ch = curl_init($sUrl);

		curl_setopt($ch, CURLOPT_TIMEOUT, 50);
		curl_setopt($ch, CURLOPT_FILE, $fp);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);

		$bReturn = curl_exec($ch);

		curl_close($ch);

		fclose($fp);

		if( $bDownload )
		{
			$sName = basename($sDestination);

			$this->downloadLocal($sFile, $sName, false, $bDelete);
		}

		return $bReturn;
	}

	/**
	* Determines if the file exists
	* 
	* @param string $sFile
	* @return bool
	*/
	public function exists($sFile)
	{
		return (file_exists($sFile) ? true : false);
	}

	/**
	* Gets extension using the file's name
	*
	* @param string $sFile file string
	* @return string
	*/
	public function getExt($sFile)
	{
		return strtolower(substr(strrchr($sFile, '.'), 1));
	}

	/**
	* Gets a given set of files within a directory with an option to filter by file extension
	*
	* @param string $sDirectory
	* @param string $sType
	* @return array
	*/
	public function getFiles($sDirectory, $sType = '')
	{
		$aFiles = array();

		if( $rDir = @opendir($sDirectory) )
		{
			if( $sType )
			{
				$sType = strtolower($sType);
			}

			while( false !== ($sFile = readdir($rDir)) )
			{
				if( $sFile == '.' || $sFile == '..' || $sFile == 'index.html' || is_dir($sDirectory . DS . $sFile) )
				{
					continue;
				}

				if( $sType )
				{
					if( $sType != $this->getExt($sFile) )
					{
						continue;
					}
				}

				$aFiles[] = $sFile;
			}

			closedir($rDir);

			return $aFiles;
		}

		return false;
	}

	/**
	* Gets all files within a given folder
	*
	* @staticvar array $aFiles
	* @param string $sDirectory
	* @param string $sType
	* @param bool $bRecursive
	* @return array
	*/
	public function getFilesAll($sDirectory, $sType = '', $bRecursive = false)
	{
		static $aFiles = array();

		if( !$bRecursive )
		{
			$aFiles = array();
		}

		if( $sType )
		{
			$sType = strtolower(ltrim($sType, '.'));
		}

		$rDir = opendir($sDirectory);
		while( $sFile = readdir($rDir) )
		{
			if( $sFile == '.' || $sFile == '..' )
			{
				continue;
			}

			if( $sType && !is_dir($sDirectory . DS . $sFile) )
			{
				if( $sType != $this->getExt($sFile) )
				{
					continue;
				}
			}

			$sNewDir = rtrim($sDirectory, DS) . DS . $sFile;

			if (is_dir($sNewDir))
			{
				$this->getFilesAll($sNewDir, $sType, true);
			}
			else
			{
				$aFiles[] = $sNewDir;
			}
		}
		closedir($rDir);

		return $aFiles;
	}

	/**
	* Gets the folders that are within a directory
	*
	* @param string $sDirectory
	* @return mixed
	*/
	public function getFolders($sDirectory)
	{
		$aFolder = array();

		if( $rDir = @opendir($sDirectory) )
		{
			while( false !== ($sFile = readdir($rDir)) )
			{
				if( $sFile == '.' || $sFile == '..' || !is_dir($sDirectory . DS . $sFile) )
				{
					continue;
				}

				$aFolder[] = $sFile;
			}

			closedir($rDir);

			return $aFolder;
		}

		return false;
	}

	/**
	* Gets the octal value of the folder/file
	* 
	* @param string $sFile
	* @return mixed
	*/
	public function getPermission($sFile)
	{
		if( !$this->exists($sFile) )
		{
			return false;
		}

		return substr(sprintf('%o', fileperms($sFile)), -4);
	}

	/**
	* Gets an item from $_FILES
	*
	* @param mixed $mItem the form item name. For form items like aFile[small], use mItem as array('aFile', 'small')
	* @param bool $bShowErrorType
	* @return mixed
	*/
	public function getUpload($mItem, $bShowErrorType = true)
	{
		if( is_array($mItem) )
		{
			list($sParent, $sName) = $mItem;

			if( !isset($_FILES[$sParent]) || !isset($_FILES[$sParent]['name'][$sName]) )
			{
				return SAA_Error::add('Form item '.$sParent.'['.$sName.'] cannot be found.');
			}

			$aUpload = array(
			'name' => $_FILES[$sParent]['name'][$sName],
			'type' => $_FILES[$sParent]['type'][$sName],
			'location' => $_FILES[$sParent]['tmp_name'][$sName],
			'error' => $_FILES[$sParent]['error'][$sName],
			'size' => $_FILES[$sParent]['size'][$sName],
			);
		}
		else
		{
			if( !isset($_FILES[$mItem]) )
			{
				return SAA_Error::add('Form item '.$mItem.' cannot be found.');
			}

			$aUpload = array(
			'name' => $_FILES[$mItem]['name'],
			'type' => $_FILES[$mItem]['type'],
			'location' => $_FILES[$mItem]['tmp_name'],
			'error' => $_FILES[$mItem]['error'],
			'size' => $_FILES[$mItem]['size'],
			);
		}

		if( $aUpload['error'] > 0 )
		{
			if( !$bShowErrorType )
			{
				return false;
			}

			switch($aUpload['error'])
			{
			case 1:
				$sMsg = 'The file is too large.';
				break;
			case 2:
				$sMsg = 'The file is too large as specified in the form.';
				break;
			case 3:
				$sMsg = 'The file was only partially uploaded.';
				break;
			case 4:
				$sMsg = 'No file was uploaded.';
				break;
			case 6:
				$sMsg = 'Missing a temporary folder.';
				break;
			case 7:
				$sMsg = 'Failed to write file to disk.';
				break;
			case 8:
				$sMsg = 'Bad file extension.';
				break;
			default:
				$sMsg = 'Error in uploading the file.';
			}

			return SAA_Error::add($sMsg);
		}

		return $aUpload;
	}

	/**
	* Moves a file from one directory to another location
	* 
	* @param string $sOld
	* @param string $sNew
	* @return bool 
	*/
	public function move($sOld, $sNew)
	{
		return rename($sOld, $sNew);
	}

	/**
	* Move a given uploaded file to a provided destination
	*
	* @param array $aUpload
	* @param string $sDestination
	* @return bool
	*/
	public function moveUpload($aUpload, $sDestination)
	{
		$this->_createDir($sDestination);

		if( !@move_uploaded_file($aUpload['location'], $sDestination) )
		{
			return false;
		}

		return true;
	}

	/**
	* Gets the mime type for a given file
	*
	* @param string $sFile
	* @return string
	*/
	public function mimeType($sFile)
	{
		if( function_exists('finfo_file') )
		{
			$rFile = finfo_open(FILEINFO_MIME_TYPE);
			$sMime = finfo_file($rFile, $sFile);
		}
		elseif( function_exists('mime_content_type') )
		{
			$sMime = mime_content_type($sFile);
		}
		else
		{
			$sMime = $this->mimeTypeByExt($sFile);
		}

		return $sMime;
	}

	/**
	* Gets mime type using the file's extension
	*
	* @param string $sFile file string
	* @param bool $bExt
	* @param bool $bList
	* @return mixed
	*/
	public function mimeTypeByExt($sFile, $bExt = false, $bList = false)
	{
		$sExt = ($bExt ? $sFile : $this->getExt($sFile));

		$aMimes = $this->getMimes();

		if( array_key_exists($sExt, $aMimes) )
		{
			if( is_array($aMimes[$sExt]) && !$bList )
			{
				return current($aMimes[$sExt]);
			}
			else
			{
				return $aMimes[$sExt];
			}
		}
		else
		{
			return false;
		}
	}

	public function update($sFile, $aFind = array(), $aReplace = array(), $sContent = '')
	{
		if( !file_exists($sFile) )
		{
			return false;
		}

		if( !strlen($sContent) )
		{
			$sContent = file_get_contents($sFile);
		}

		$sContent = preg_replace($aFind, $aReplace, $sContent);

		if( $rFile = @fopen($sFile, 'w') )
		{
			fwrite($rFile, $sContent);

			fclose($rFile);

			return true;
		}

		return false;
	}

	/**
	* Gets an uploaded file and moves the given file to a provided destination. 
	*
	*
	* @param mixed $mItem
	* @param string $sDirectory
	* @param mixed $mSubDirectory
	* @param string $sFile
	* @param array/string $mAllowed file extension
	* @param bool $bBuildDir
	* @return bool
	*/
	public function upload($mItem, $sDirectory, $mSubDirectory, $sFile = '', $mAllowed = array(), $bBuildDir = true)
	{
		if( !($aItem = $this->getUpload($mItem)) )
		{
			return false;
		}

		if( $mAllowed )
		{
			if( is_string($mAllowed) )
			{
				$mAllowed = (array) $mAllowed;
			}

			$aMimes = array();

			foreach($mAllowed as $sExt)
			{
				$aMime = $this->mimeTypeByExt($sExt, true, true);

				if( is_array($aMime) )
				{
					$aMimes = array_merge($aMimes, $aMime);
				}
				else
				{
					$aMimes[] = $aMime;
				}
			}

			if( !in_array($this->mimeType($aItem['location'] . DS . $aItem['name']), $aMimes) )
			{
				return SAA_Error::add('The file uploaded does not have the correct file type');
			}
		}

		if( $bBuildDir )
		{
			$mSubDirectory = $this->buildDir($mSubDirectory);
		}

		if( !$sFile )
		{
			$sExt = $this->getExt($aItem['name']);

			$sFile = $this->buildName($sDirectory . $mSubDirectory, $sExt);
		}

		$sDestination = $sDirectory . $mSubDirectory . $sFile;

		if( !$this->moveUpload($aItem, $sDestination) )
		{
			return false;
		}

		return $mSubDirectory . $sFile;
	}

	public function write($sFile, $sText, $iChmod = 0644)
	{
		if( file_exists($sFile) )
		{
			$this->delete($sFile);
		}

		if( !($rFile = @fopen($sFile, 'w')) )
		{
			return false;
		}

		fwrite($rFile, $sText);
		fclose($rFile);

		@chmod($sFile, $iChmod);

		return true;
	}

	private function _createDir($sLocation)
	{
		$aDir = pathinfo($sLocation);

		if( file_exists($aDir['dirname']) )
		{
			return true;
		}

		$sDir = $aDir['dirname'];

		$sCurrent = '';

		$aParts = explode(DS, $sDir);

		foreach ($aParts as $sPart)
		{
			$sCurrent .= DS . $sPart;

			if( !is_dir($sCurrent) )
			{
				@mkdir($sCurrent, 0777);
				@chmod($sCurrent, 0777);
			}
		}

		if( is_dir($sDir) )
		{
			return true;
		}

		return false;
	}

	/**
	* List of safe mime types
	*
	* @return array
	*/
	private function getMimes()
	{
		return array(
		'csv' => array('text/x-comma-separated-values', 'text/comma-separated-values', 'application/octet-stream', 'application/vnd.ms-excel', 'text/x-csv', 'text/csv', 'application/csv', 'application/excel', 'application/vnd.msexcel'),
		'pdf' => array('application/pdf', 'application/x-download'),
		'xls' => array('application/excel', 'application/vnd.ms-excel', 'application/msexcel'),
		'ppt' => array('application/powerpoint', 'application/vnd.ms-powerpoint'),
		'gtar' => 'application/x-gtar',
		'gz' => 'application/x-gzip',
		'js' => 'application/x-javascript',
		'swf' => 'application/x-shockwave-flash',
		'tar' => 'application/x-tar',
		'tgz' => array('application/x-tar', 'application/x-gzip-compressed'),
		'zip' => array('application/x-zip', 'application/zip', 'application/x-zip-compressed'),
		'mid' => 'audio/midi',
		'midi' => 'audio/midi',
		'mpga' => 'audio/mpeg',
		'mp2' => 'audio/mpeg',
		'mp3' => array('audio/mpeg', 'audio/mpg', 'audio/mpeg3'),
		'aif' => 'audio/x-aiff',
		'aiff' => 'audio/x-aiff',
		'aifc' => 'audio/x-aiff',
		'ram' => 'audio/x-pn-realaudio',
		'rm' => 'audio/x-pn-realaudio',
		'rpm' => 'audio/x-pn-realaudio-plugin',
		'ra' => 'audio/x-realaudio',
		'rv' => 'video/vnd.rn-realvideo',
		'wav' => 'audio/x-wav',
		'bmp' => 'image/bmp',
		'gif' => 'image/gif',
		'jpeg' => array('image/jpeg', 'image/pjpeg'),
		'jpg' => array('image/jpeg', 'image/pjpeg'),
		'jpe' => array('image/jpeg', 'image/pjpeg'),
		'png' => array('image/png',  'image/x-png'),
		'tiff' => 'image/tiff',
		'tif' => 'image/tiff',
		'css' => 'text/css',
		'txt' => 'text/plain',
		'text' => 'text/plain',
		'xml' => 'text/xml',
		'xsl' => 'text/xml'
		);

	}
}
?>