<?php
defined('Trapper') or exit('Trespasser!');

/**
*
* @author Hutton
* @name Site Info Library Class
* @license GoLevel LLC
*/

Class SAA_Info
{
	private $_aVars = array();

	/**
	* Class Constructor
	*/
	public function __construct()
	{
		if( !($aVars = SAA_Sys::getLib('cache')->get('site_info')) )
		{
			$aVars = $this->_buildCache();
		}

		$this->_aVars = $aVars;
	}

	/**
	* Gets a given setting
	*
	* @param string $sParam
	* @return mixed
	*/
	public function get($sParam)
	{
		if( !isset($this->_aVars[$sParam]) )
		{
			SAA_Debug::trigger('Site info value does not exist: '. $sParam);
		}

		return $this->_aVars[$sParam];
	}

	/**
	* Rebuilds the site_info cache file
	*/
	public function rebuildCache()
	{
		SAA_Sys::getLib('cache')->remove('site_info');
		
		$this->_aVars = $this->_buildCache();
	}

	/**
	 * Update a site info param
	 *
	 * @param string $sParam
	 * @param string $sValue
	 */
	public function update($sParam, $sValue)
	{
		$oDb = SAA_Sys::getLib('database');

		$oDb->update('site_info', array('info_value' => $sValue), 'var_name = "'. $oDb->esc($sParam) . '"');

		$this->rebuildCache();
	}

	/**
	* Stores site_info data into a cache file
	*
	* @return array
	*/
	private function _buildCache()
	{
		$aVars = array();

		$aRows = SAA_Sys::getLib('database')->select('s.*')
			->from('site_info', 's')
			->get('rows');

		foreach($aRows as $aRow)
		{
			$aVars[$aRow['var_name']] = $aRow['info_value'];
		}

		SAA_Sys::getLib('cache')->create('site_info', $aVars);

		return $aVars;
	}
}
?>