<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Mail Driver Mail Library Class
 * @license GoLevel LLC
 */

Class SAA_Mail_Driver_Mail
{
    private $_oMail = null;

    /**
     * Class Construct
     */
    public function __construct()
    {
        require_once(SAA_SYSTEM_3PARTY . 'PHPmailer' . DS . 'class.phpmailer.php');

        $this->_oMail = new PHPMailer();
    }

    /**
     * Set "from" part of email
     *
     * @param string $sEmail
     * @param string $sName
     * @return SAA_Mail_Driver_Mail
     */
    public function from($sEmail, $sName = '')
    {
        if( !$sName )
        {
            $this->_oMail->SetFrom($sEmail, $sName);
        }
        else
        {
            $this->_oMail->SetFrom($sEmail);
        }

        return $this;
    }

    /**
     * Set "message" part of email
     *
     * @param string $sText
     * @param bool $bTemplateCompile
     * @return SAA_Mail_Driver_Mail
     */
    public function message($sText, $bTemplateCompile = true)
    {
        $sText = nl2br($sText);

        if( $bTemplateCompile )
        {
            $sText = SAA_Sys::compile($sText);
        }

        $this->_oMail->MsgHTML($sText);

        return $this;
    }

    /**
     * Send email
     *
     * @return bool
     */
    public function send()
    {
        if( $this->_oMail->Send() )
        {
            return true;
        }

        return SAA_Error::add("Mailer Error: " . $this->oMail->ErrorInfo);
    }

    /**
     * Set "subject" part of email
     *
     * @param string $sText
     * @param bool $bTemplateCompile
     * @return SAA_Mail_Driver_Mail
     */
    public function subject($sText, $bTemplateCompile = true)
    {
        if( $bTemplateCompile )
        {
            $sText = SAA_Sys::compile($sText);
        }

        $this->_oMail->Subject = $sText;

        return $this;
    }

    /**
     * Set "to" part of email
     *
     * @param string $sEmail
     * @param string $sName
     * @return SAA_Mail_Driver_Mail
     */
    public function to($sEmail, $sName = '')
    {
        if( !$sName )
        {
            $this->_oMail->AddAddress($sEmail, $sName);
        }
        else
        {
            $this->_oMail->AddAddress($sEmail);
        }

        return $this;
    }
}
?>
