<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Mail Driver Smtp Library Class
 * @license GoLevel LLC
 */

Class SAA_Mail_Driver_Smtp
{
    private $_oMail = null;

    /**
     * Class Construct
     */
    public function __construct()
    {
        require_once(SAA_SYSTEM_3PARTY . 'PHPmailer' . DS . 'class.phpmailer.php');

        $this->_oMail = new PHPMailer();

        $this->_oMail->IsSMTP(); // telling the class to use SMTP
        $this->_oMail->SMTPDebug  = 2;                     // enables SMTP debug information (for testing)
                                                   // 1 = errors and messages
                                                   // 2 = messages only
        $this->_oMail->SMTPAuth   = true;                  // enable SMTP authentication
        $this->_oMail->Host       = SAA_Sys::getSett('mail.smtp_host'); // sets the SMTP server
        $this->_oMail->Port       = SAA_Sys::getSett('mail.smtp_port');                    // set the SMTP port for the GMAIL server
        $this->_oMail->Username   = SAA_Sys::getSett('mail.smtp_username'); // SMTP account username
        $this->_oMail->Password   = SAA_Sys::getSett('mail.smtp_password');        // SMTP account password
    }

    /**
     * Set "from" part of email
     *
     * @param string $sEmail
     * @param string $sName
     * @return SAA_Mail_Driver_Mail
     */
    public function from($sEmail, $sName = '')
    {
        if( !$sName )
        {
            $this->_oMail->SetFrom($sEmail, $sName);
        }
        else
        {
            $this->_oMail->SetFrom($sEmail);
        }

        return $this;
    }

    /**
     * Set "message" part of email
     *
     * @param string $sText
     * @param bool $bTemplateCompile
     * @return SAA_Mail_Driver_Mail
     */
    public function message($sText, $bTemplateCompile = true)
    {
        $sText = nl2br($sText);

        if( $bTemplateCompile )
        {
            $sText = SAA_Sys::compile($sText);
        }

        $this->_oMail->MsgHTML($sText);

        return $this;
    }

    /**
     * Send email
     *
     * @return bool
     */
    public function send()
    {
        if( $this->_oMail->Send() )
        {
            return true;
        }

        return SAA_Error::add("Mailer Error: " . $this->oMail->ErrorInfo);
    }

    /**
     * Set "subject" part of email
     *
     * @param string $sText
     * @param bool $bTemplateCompile
     * @return SAA_Mail_Driver_Mail
     */
    public function subject($sText, $bTemplateCompile = true)
    {
        if( $bTemplateCompile )
        {
            $sText = SAA_Sys::compile($sText);
        }

        $this->_oMail->Subject = $sText;

        return $this;
    }

    /**
     * Set "to" part of email
     *
     * @param string $sEmail
     * @param string $sName
     * @return SAA_Mail_Driver_Mail
     */
    public function to($sEmail, $sName = '')
    {
        if( !$sName )
        {
            $this->_oMail->AddAddress($sEmail, $sName);
        }
        else
        {
            $this->_oMail->AddAddress($sEmail);
        }

        return $this;
    }
}
?>
