<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Paginator Class
 * @license GoLevel LLC
 */

Class SAA_Paginator
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Generates the pages provided the current page, page size, and total items
     *
     * @param int $iPage
     * @param int $iPageSize
     * @param int $iTotal
     * @param array $aAttr HTML element attributes
     * @return string
     */
    public function get($iPage, $iPageSize, $iTotal, $aAttr)
    {
        $iPages = 5;

        if( !$iTotal )
	    return '';

	$aPages = array();
	$iPagesTotal = ceil($iTotal/$iPageSize);

	if( $iPagesTotal <= $iPages )
	{
	    $iFirst = 1;
	    $iLast = $iPagesTotal;
	}
	else
	{
	    $iHalfPages = ceil($iPages/2);

	    if( $iPage < $iPages )
	    {
		$iFirst = 1;
		$iLast = ($iPagesTotal > $iPages ? $iPages : $iPagesTotal);
	    }
	    elseif( $iPage >= $iPagesTotal - $iHalfPages )
	    {
		$iFirst = $iPagesTotal - $iPages;
		$iLast = $iPagesTotal;
	    }
	    else
	    {
		$iFirst = $iPage - $iHalfPages;
		$iLast = $iPage + $iHalfPages;
	    }
	}

        if( isset($aAttr['href']) && $aAttr['href'] != '#' )
        {
            $aAttr['href'] = str_replace('_%7Bpg%7D', '_{pg}', $aAttr['href']);
        }

	$bFirst = ($iPage == 1 ? false : true);
	$bNext = ($iPage == $iPagesTotal ? false : true);
	$bPrevious = ($iPage == 1 ? false : true);
	$bLast = ($iPage == $iPagesTotal ? false : true);

        if( $bFirst )
	{
            $sPage = '<a ';
            foreach($aAttr as $sAttribute => $sValue)
            {
                $sPage .= $sAttribute . '="'.str_replace('{pg}', 1, $sValue).'"';
            }
	    $aPages[] = $sPage . '>First</a>';
	}

	if( $bPrevious )
	{
            $iPrevious = $iPage-1;

            $sPage = '<a ';
            foreach($aAttr as $sAttribute => $sValue)
            {
                $sPage .= $sAttribute . '="'.str_replace('{pg}', $iPrevious, $sValue).'"';
            }
            $aPages[] = $sPage . '>Previous</a>';
	}

	for($i = $iFirst; $i <= $iLast; $i++)
	{
            $sPage = '<a ';
            foreach($aAttr as $sAttribute => $sValue)
            {
                $sPage .= $sAttribute . '="'.str_replace('{pg}', $i, $sValue).'"';
            }
	    $aPages[] = $sPage . '>'.$i.'</a>';
	}

	if( $bNext )
	{
            $iNext = $iPage+1;

            $sPage = '<a ';
            foreach($aAttr as $sAttribute => $sValue)
            {
                $sPage .= $sAttribute . '="'.str_replace('{pg}', $iNext, $sValue).'"';
            }
	    $aPages[] = $sPage . '>Next</a>';
	}

	if( $bLast )
	{
            $sPage = '<a ';
            foreach($aAttr as $sAttribute => $sValue)
            {
                $sPage .= $sAttribute . '="'.str_replace('{pg}', $iPagesTotal, $sValue).'"';
            }
	    $aPages[] = $sPage . '>Last</a>';
	}

        return $aPages;
    }

    /**
     * Gets the total number of pages
     *
     * @param string $sTbl
     * @param string $sCondition
     * @param int $iPageSize
     * @param mixed $mOrder
     * @return int
     */
    public function getPageNum($sTbl, $sCondition, $iPageSize, $mOrder = array())
    {
        $oDb = SAA_Sys::getLib('database');

        if( $mOrder )
        {
            $mOrder = (array) $mOrder;

            call_user_func_array(array($oDb, 'order'), $mOrder);
        }

        $iCnt = $oDb->select('COUNT(*)')
            ->from($sTbl)
            ->where($sCondition)
            ->get('column');

        if( $iCnt < 1 )
        {
            return 1;
        }

        return ceil($iCnt/$iPageSize);
    }
}
?>
