<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Parse Abstraction Class
 * @license GoLevel LLC
 */

Abstract Class SAA_Parse_Abstraction
{
    private $_sAlphaNumeric = 'abcdefghijklmnopqrstuvwxyz1234567890';

    private $_sAlphaNumericAll = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890';

    private $_iAlphaNumeric = 35;

    private $_iAlphaNumericAll = 61;

    public function _buildName($sName, $sTable, $mColumn, $iLen = 255)
    {
        $oDb = SAA_Sys::getLib('database');

        if( method_exists(get_called_class(), 'cleanName') )
        {
            $sName = $this->cleanName($sName, $iLen);
        }
        else
        {
            $sName = $this->_cleanName($sName, $iLen);
        }

        if( !is_array($mColumn) )
        {
            $mColumn = (array) $mColumn;
        }

        $aWherePart = array();

        foreach($mColumn as $sColumn => $sValue)
        {
            if( is_numeric($sColumn) )
            {
                $sColumnFocus = $sValue;
            }
            else
            {
                $aWherePart[] = $sColumn . ' = "' . $oDb->esc($sValue).'" AND';
            }
        }

        $aWhere = $aWherePart;
        $aWhere[] = $sColumnFocus . ' LIKE "' . $oDb->esc($sName).'" AND';

        $iCount = $oDb
                ->select('COUNT(*)')
                ->from($sTable)
                ->where($aWhere)
                ->get('column');

        if( $iCount > 0 )
        {
            do
            {
                $sSalt = $this->_salt($this->saltLen(), true);

                if( strlen($sName.$sSalt) > $iLen )
                {
                    $sName = substr($sName, 0, $iLen - $this->_iSaltLen);
                }

                $aWhere = $aWherePart;
                $aWhere[] = $sColumnFocus . ' = "' . $oDb->esc($sName.$sSalt).'" AND';

                $iExists = $oDb
                    ->select('COUNT(*)')
                    ->from($sTable)
                    ->where($aWhere)
                    ->get('column');

                if( $iExists > 0 )
                {
                    $bRun = true;
                }
                else
                {
                    $sName .= $sSalt;
                    $bRun = false;
                }

            }
            while($bRun);
        }

        return $sName;
    }

    /**
     * Sanitize string for name
     *
     * @param string $sName string needing cleaning
     * @param int $iLen maximum length
     * @param string $sSpacer replaces spaces and banned characters
     * @param string $sBanned string list of naughty characters
     * @return string
     */
    public function _cleanName($sName, $iLen, $sSpacer = '-', $sBanned = ' _')
    {

        //removes html and beginning/end spaces
        $sName = trim(strip_tags($sName));

        //removes banned strings with spacer
        if( $sBanned )
        {
            $aBanned = array();
            $iBanned = strlen($sBanned);

            for($i = 0; $i < $iBanned; $i++)
            {
                $aBanned[] = $sBanned[$i];
            }

            if( $aBanned )
            {
                $sName = str_replace($aBanned, $sSpacer, $sName);
            }
        }

        $sName = $this->_toAlphaNumeric($sName, $sSpacer);

        $aDuplicates = array(
            str_repeat($sSpacer, 5),
            str_repeat($sSpacer, 4),
            str_repeat($sSpacer, 3),
            str_repeat($sSpacer, 2),
        );

        //remove duplicates
        $sName = str_replace($aDuplicates, $sSpacer, $sName);

        //remove spacer from front and back
        $sName = trim($sName, $sSpacer);

        //if the whole string was ban, create a name
        if( empty($sName) )
        {
            $sName = $this->_salt(12, true) . SAA_TIME;
        }

        $sName = strtolower($sName);

        //shorten string
        if( strlen($sName) > $iLen )
        {
            $sName = substr($sName, 0, $iLen);
        }

        return $sName;
    }

    /*
     * Gets the alphanumeric character sets
     * 
     * @param bool bAll 
     * @return string
     */
    public function _getAlphaNumeric($bAll = true)
    {
        return ($bAll ? $this->_sAlphaNumericAll : $this->_sAlphaNumeric);
    }

    /**
     * Removes all non alpha-numeric characters from a string
     *
     * @param string sVal string parsing
     * @param string sExceptions string list of characters allowed
     *
     */
    public function _toAlphaNumeric($sVal, $sExceptions = '')
    {
        $sText = '';
        $iLen = strlen($sVal);

        $sPermitted = $this->_sAlphaNumericAll . $sExceptions;

        for($i = 0; $i < $iLen; $i++)
        {
            if( strpos($sPermitted, $sVal[$i]) !== false )
            {
                $sText .= $sVal[$i];
            }
        }

        return $sText;
    }

     /**
     * Create a random string of iLength
     *
     * @param int iLength length of string
     * @param bool bAlphaNumeric determines whether the string is alphanumeric
     * @return string
     *
     */
    public function _salt($iLength = 3, $bAlphaNumeric = false)
    {
        if( $bAlphaNumeric )
        {
            return $this->_saltAlphaNumeric($iLength);
        }
        else
        {
            return $this->_saltFull($iLength);
        }
    }

    /**
     * Creates a random string
     *
     * @param int $iLength length of string
     * @return string
     */
    private function _saltFull($iLength = 3)
    {
        $sSalt = '';

        for ($i = 0; $i < $iLength; $i++)
        {
            $sSalt .= chr(rand(33, 91));
        }

        return $sSalt;
    }

    /**
     * Create a random alphanumeric string
     *
     * @param int $iLength
     * @return string
     */
    private function _saltAlphaNumeric($iLength)
    {
        $sSalt = '';

        for ($i = 0; $i < $iLength; $i++)
        {
            $iChr = rand(0, $this->_iAlphaNumeric);
            $sSalt .= $this->_sAlphaNumeric[$iChr];
        }

        return $sSalt;
    }
}
?>