<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Parse Input Library Class
 * @license GoLevel LLC
 */

Class SAA_Parse_Input
{
    private $_sAlphaNumeric = 'abcdefghijklmnopqrstuvwxyz1234567890';

    private $_sAlphaNumericAll = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890';

    private $_iAlphaNumeric = 35;

    private $_iAlphaNumericAll = 61;

    private $_aDirtyEvents = array(
        'onActivate',
        'onAfterPrint',
        'onBeforePrint',
        'onAfterUpdate',
        'onBeforeUpdate',
        'onErrorUpdate',
        'onAbort',
        'onBeforeDeactivate',
        'onDeactivate',
        'onBeforeCopy',
        'onBeforeCut',
        'onBeforeEditFocus',
        'onBeforePaste',
        'onBeforeUnload',
        'onBlur',
        'onBounce',
        'onChange',
        'onClick',
        'onControlSelect',
        'onCopy',
        'onCut',
        'onDblClick',
        'onDrag',
        'onDragEnter',
        'onDragLeave',
        'onDragOver',
        'onDragStart',
        'onDrop',
        'onFilterChange',
        'onDragDrop',
        'onError',
        'onFilterChange',
        'onFinish',
        'onFocus',
        'onHelp',
        'onKeyDown',
        'onKeyPress',
        'onKeyUp',
        'onLoad',
        'OnLoseCapture',
        'onMouseDown',
        'onMouseEnter',
        'onMouseLeave',
        'onMouseMove',
        'onMouseOut',
        'onMouseOver',
        'onMouseUp',
        'onMove',
        'onPaste',
        'onPropertyChange',
        'onReadyStateChange',
        'onReset',
        'onResize',
        'onResizeEnd',
        'onResizeStart',
        'onScroll',
        'onSelectStart',
        'onSelect',
        'onSelectionChange',
        'onStart',
        'onStop',
        'onSubmit',
        'onUnload',
        'class',
        'style'
    );

    private $_aDirtyCss = array(
        'position',
        'top',
        'left',
        'background',
        'background-image',
        'background-color',
        'width',
        'height',
        'behaviour'
    );

    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Remove dirty attributes from a tag
     *
     * @param string $sText
     * @return string
     */
    public function cleanAttributes($sText)
    {
    	$sText = str_replace(array('xhref', 'xsrc'), array('href', 'src'), $sText);
        $sText = strip_tags($sText);

        foreach($this->_aDirtyEvents as $sRemove)
        {
            $sText = preg_replace('#'.$sRemove.'=#i', 'title=', $sText);
        }

        $sText = preg_replace('#javascript:#i', '', $sText);

        foreach($this->_aDirtyCss as $sCss)
        {
            $sText = preg_replace('/'. $sCss .':(.*?)/i', 'replaced:', $sText);
       	}

        return $sText;
    }

    /**
     * Removes given character(s) for a string
     *
     * @param string $sText string needing cleaning
     * @param string $sBanned string list of naughty characters
     * @param string $sReplace replaces banned characters
     * @param int $iLen maximum length
     * @return string
     */
    public function cleanChrs($sText, $sBanned = '', $sReplace = '', $iLen = false)
    {
        //removes banned strings with spacer
        if( $sBanned )
        {
            $aBanned = array();
            $iBanned = strlen($sBanned);

            for($i = 0; $i < $iBanned; $i++)
            {
                $aBanned[] = $sBanned[$i];
            }

            if( $aBanned )
            {
                $sText = str_replace($aBanned, $sReplace, $sText);
            }
        }

        $aDuplicates = array(
            str_repeat($sReplace, 5),
            str_repeat($sReplace, 4),
            str_repeat($sReplace, 3),
            str_repeat($sReplace, 2),
        );

        //remove duplicates
        $sText = str_replace($aDuplicates, $sReplace, $sText);

        //remove spacer from front and back
        $sText = trim($sText, $sReplace);

        //if the whole string was ban, create a name
        if( empty($sText) )
        {
            $sText = $this->salt(12, true) . SAA_TIME;
        }

        //shorten string
        if( $iLen !== false && strlen($sText) > $iLen )
        {
            $sText = $this->shorten($sText, $iLen);
        }

        return $sText;
    }

    /**
     * Remove dirty HTML from string
     *
     * @param string $sText
     * @param sting $sAllowed the setting var name to be used
     * @param bool $bIsSetting whether or not $sAllowed is a setting
     * @return string
     */
    public function cleanTags($sText, $sAllowed = '', $bIsSetting = true)
    {
        $sText = stripslashes($sText);

        preg_match_all('/<\/?\w+((\s+\w+(\s*=\s*(?:".*?"|\'.*?\'|[^\'">\s]+))?)+\s*|\s*)\/?>/i', $sText, $aMatched);

        if( $sAllowed && $bIsSetting )
        {
            $sAllowed = $this->getTags($sAllowed, 'string');
        }

        $sTempTagOpen = ' ' .uniqid();
        $sTempTagClose = uniqid() . ' ';

        //Loop through all HTML tags
        foreach($aMatched[0] as $iKey => $sTag)
        {
            $sCheck = str_replace($aMatched[1][$iKey], '', $sTag);
            $sCheck = strtolower($sCheck);

            //If tag is not in text, skip iteration
            if( strpos($sAllowed, $sCheck) === false )
            {
                continue;
            }

            $sTemp = str_replace($aMatched[1][$iKey], $this->cleanAttributes($aMatched[1][$iKey]), $sTag);
            $sNewTag = str_replace(array('<', '>'), array($sTempTagOpen, $sTempTagClose), $sTemp);
            $sText = str_replace($sTag, $sNewTag, $sText);

        }

        $sText = str_replace(array('<', '>'), array('&lt;', '&gt;'), $sText);
        $sText = str_replace(array($sTempTagOpen, $sTempTagClose), array('<', '>'), $sText);

        $sText = $this->rareToEntities($sText);

        return $sText;
    }

    /**
     * Gets the tags from a given setting
     *
     * @param string $sSetting setting name
     * @param string $sReturn either array or string
     * @return mixed
     */
    public function getTags($sSetting, $sReturn = 'array')
    {
        $sSetting = SAA_Sys::getSett($sSetting);

        $oCache = SAA_Sys::getLib('cache');

        $sCache = 'settings_' . md5($sSetting) . '_' . $sReturn;

        if( ($mSetting = $oCache->get($sCache)) )
        {
            return $mSetting;
        }

        preg_match_all('/<([A-Z][A-Z0-9]*)\b[^>]*>/i', $sSetting, $aMatch);

        if( $sReturn == 'string' )
        {
            $mSetting = '';

            foreach($aMatch[1] as $sTag)
            {
                $mSetting .= '<'.$sTag.'></'.$sTag.'><'.$sTag.'/><'.$sTag.' />';
            }
        }
        else
        {
            $mSetting = array();

            foreach($aMatch[1] as $sTag)
            {
                $mSetting[] = '<'.$sTag.'>';
                $mSetting[] = '</'.$sTag.'>';
                $mSetting[] = '<'.$sTag.'/>';
                $mSetting[] = '<'.$sTag.' />';
            }
        }

        $oCache->create($sCache, $mSetting);

        return $mSetting;
    }

    public function getUnique($sText, $sTable, $mColumn, $iLen = 255, $iSaltLen = 3, $bSaltAlphaNumeric = true)
    {
        $oDb = SAA_Sys::getLib('database');

        if( !is_array($mColumn) )
        {
            $mColumn = (array) $mColumn;
        }

        $aWherePart = array();

        foreach($mColumn as $sColumn => $sValue)
        {
            if( is_numeric($sColumn) )
            {
                $sColumnFocus = $sValue;
            }
            else
            {
                $aWherePart[] = $sColumn . ' = "' . $oDb->esc($sValue).'" AND';
            }
        }

        $aWhere = $aWherePart;
        $aWhere[] = $sColumnFocus . ' LIKE "' . $oDb->esc($sText).'" AND';

        $iCount = $oDb
                ->select('COUNT(*)')
                ->from($sTable)
                ->where($aWhere)
                ->get('column');

        if( $iCount > 0 )
        {
            do
            {
                $sSalt = $this->salt($iSaltLen, $bSaltAlphaNumeric);

                if( strlen($sText . $sSalt) > $iLen )
                {
                    $sText = substr($sText, 0, $iLen - $iSaltLen);
                }

                $aWhere = $aWherePart;
                $aWhere[] = $sColumnFocus . ' = "' . $oDb->esc($sText . $sSalt).'" AND';

                $iExists = $oDb
                    ->select('COUNT(*)')
                    ->from($sTable)
                    ->where($aWhere)
                    ->get('column');

                if( $iExists > 0 )
                {
                    $bRun = true;
                }
                else
                {
                    $sText .= $sSalt;
                    $bRun = false;
                }

            }
            while($bRun);
        }

        return $sText;
    }

    /**
     * Convert non-latin characters to entities
     *
     * @param string $mVal
     * @return string
     */
    public function rareToEntities($mVal, $iLen = false)
    {
        if( is_array($mVal) )
        {
            return array_map(array($this, 'rareToEntities'), $mVal);
	}

        $sStr = '';

        $aVal = array();
        $iEnd = 1;

        for ($i = 0, $iCnt = strlen($mVal); $i < $iCnt; $i++ )
        {
            $iCurrent = ord($mVal[$i]);

            if ( $iCurrent < 128 )
            {
                $sStr .= ($iCurrent == 42 || $iCurrent > 127 ? '&#' . $iCurrent . ';' : chr($iCurrent));
            }
            else
            {
                if( count($aVal) == 0 )
                {
                    $iEnd = ($iCurrent < 224 ? 2 : 3);
                }

                $aVal[] = $iCurrent;

                if( count($aVal) == $iEnd )
                {
                    $iUnicode = ( $iEnd == 3 ) ?
                        ( ( $aVal[0] % 16 ) * 4096 ) + ( ( $aVal[1] % 64 ) * 64 ) + ( $aVal[2] % 64 ):
                    	( ( $aVal[0] % 32 ) * 64 ) + ( $aVal[1] % 64 );

                    $sStr .= ($iUnicode == 42 || $iUnicode > 127 ? '&#' . $iUnicode . ';' : chr($iUnicode));

                    $aVal = array();
                    $iEnd = 1;
                }
            }
        }

        if( $iLen && strlen($sStr) > $iLen )
        {
            $sStr = $this->shorten($sStr, $iLen);
        }

        return $sStr;
    }

    /**
     * Removes tags from string
     *
     * @param string $sText
     * @return string
     */
    public function removeTags($sText, $bToEntities = true)
    {
        $sText = stripslashes($sText);
        $sText = strip_tags($sText);

        if( $bToEntities )
        {
            $sText = $this->rareToEntities($sText);
        }

        return $sText;
    }

    /**
     * Create a random string of iLength
     *
     * @param int iLength length of string
     * @param bool bAlphaNumeric determines whether the string is alphanumeric
     * @return string
     *
     */
    public function salt($iLength = 3, $bAlphaNumeric = false)
    {
        if( $bAlphaNumeric )
        {
            return $this->_saltAlphaNumeric($iLength);
        }
        else
        {
            return $this->_saltFull($iLength);
        }
    }

    /**
     * Shortens a string to a given length
     *
     * @param string $sText
     * @param int $iLen
     * @return string
     */
    public function shorten($sText, $iLen)
    {
        if( !preg_match('/(&#[0-9]+;)/', $sText) )
        {
            return substr($sText, 0, $iLen);
        }

        $sShort = '';

        for($iPos = 0, $iOutLen = 0, $iTxtLen = strlen($sText); $iPos < $iTxtLen && $iOutLen <= $iLen; $iPos++)
        {
            if ($sText[$iPos] == '&')
            {
                $iEnd = strpos($sText, ';', $iPos) + 1;
                $sTemp = substr($sText, $iPos, $iEnd - $iPos);

                if (preg_match('/(&#[0-9]+;)/', $sTemp))
                {
                    $sShort .= $sTemp;

                    $iOutLen++;
                    $iPos = $iEnd-1;

                    continue;
                }
            }

            $sShort .= $sText[$iPos];
            $iOutLen++;
        }

        return $sShort;
    }

    /**
     * Removes non-alphanumeric characters from a string
     * 
     * @param string $sText
     * @param int $iLen
     * @param string $sExceptions
     * @param string $sReplacement
     * @return string
     */
    public function toAlphaNumeric($sText, $iLen = 0, $sExceptions = '', $sReplacement = '', $bLowerCase = true)
    {
        $sNew = '';
        $iLen = strlen($sText);

        $sPermitted = $this->_sAlphaNumericAll . $sExceptions;

        for($i = 0; $i < $iLen; $i++)
        {
            $sNew .= (strpos($sPermitted, $sText[$i]) !== false ? $sText[$i] : $sReplacement);
        }

        if( $iLen > 0 && strlen($sText) > $iLen )
        {
            $sNew = $this->shorten($sNew, $iLen);
        }

        $sNew = trim($sNew, $sExceptions);

        //if the whole string was bad, create a random string
        if( empty($sNew) )
        {
            $sNew = $this->salt(12, true) . SAA_TIME;
        }

        if( $sReplacement )
        {
            $l = strlen($sReplacement);

            for($i=0; $i < $l; $i++)
            {
                $sStr = $sReplacement[$i];

                $sNew = str_replace(array(
                    $sStr,
                    $sStr . $sStr,
                    $sStr . $sStr . $sStr,
                    $sStr . $sStr . $sStr . $sStr,
                ), $sStr, $sNew);
            }
        }

        return strtolower($sNew);
    }

    /**
     * Creates a random string
     *
     * @param int $iLength length of string
     * @return string
     */
    private function _saltFull($iLength = 3)
    {
        $sSalt = '';

        for ($i = 0; $i < $iLength; $i++)
        {
            $sSalt .= chr(rand(33, 91));
        }

        return $sSalt;
    }

    /**
     * Create a random alphanumeric string
     *
     * @param int $iLength
     * @return string
     */
    private function _saltAlphaNumeric($iLength)
    {
        $sSalt = '';

        for ($i = 0; $i < $iLength; $i++)
        {
            $iChr = rand(0, $this->_iAlphaNumeric);
            $sSalt .= $this->_sAlphaNumeric[$iChr];
        }

        return $sSalt;
    }
}
?>
