<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Request Library Class
 * @license GoLevel LLC
 */

Class SAA_Request
{
    private $_aRequest = array();

    private $_aSlug = array();

    private $_sController = '';

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $this->_addMissing();

	$this->_aRequest = array_merge($_POST, $_GET, $_REQUEST);

	$this->_getFromLink();
    }

    /**
     * Gets a given var with an optional value if var is not found
     *
     * @param string $sVar
     * @param mixed $mDefault
     * @return mixed
     */
    public function get($sVar, $mDefault = false)
    {
    	return (array_key_exists($sVar, $this->_aRequest) ? $this->_aRequest[$sVar] : $mDefault);
    }

    /**
     * Gets a given var and cast the value as an array
     *
     * @param string $sVar
     * @param array $aDefault
     * @return array
     */
    public function getArray($sVar, $aDefault = array())
    {
	return (array) $this->get($sVar, $aDefault);
    }

    /**
     * Gets the controller from the url
     *
     * @return string
     */
    public function getController()
    {
	return $this->_sController;
    }

    /**
     * Gets a given var and cast the value as an int
     *
     * @param string $sVar
     * @param string $sDefault
     * @return int
     */
    public function getInt($sVar, $sDefault = 0)
    {
	return (int) $this->get($sVar, $sDefault);
    }

    /**
     * Gets the url slugs for the given page with an option to exclude the controller from slugs
     *
     * @param bool $bExclude
     * @return array
     */
    public function getSlugs($bExclude = true)
    {
        if( !$bExclude )
        {
            return $this->_aSlug;
        }

        $aSlugs = array();

        $aController = explode('.', $this->_sController);

        foreach($this->_aSlug as $i => $sSlug)
        {
            if( isset($aController[$i]) && $aController[$i] == $sSlug )
            {
                continue;
            }

            $aSlugs[] = $sSlug;
        }

        return $aSlugs;
    }

    /**
     * Removes a given var from the set of stored vars
     *
     * @param string $sVar
     * @return bool
     */
    public function remove($sVar)
    {
	if( !array_key_exists($sVar, $this->_aRequest) )
	{
	    return false;
	}

	unset($this->_aRequest[$sVar]);

	return true;
    }

    /**
     * Sets a given value to the stored variables
     *
     * @param string $sVar
     * @param mixed $mValue
     * @return true
     */
    public function set($sVar, $mValue)
    {
	$this->_aRequest[$sVar] = $mValue;

	return true;
    }

    /**
     * Adds any lost vars
     */
    private function _addMissing()
    {
        $sMissing = parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY);

        parse_str($sMissing, $aMissing);

        if( is_array($aMissing) )
        {
            foreach($aMissing as $sKey => $sValue)
            {
                $_REQUEST[$sKey] = $sValue;
            }
        }
    }

    /**
     * Gets relevant vars from link
     */
    private function _getFromLink()
    {
	if( !($sUrl = $this->get('u')) )
	{
	    return false;
	}

	$bController = true;

	$sUrl = trim($sUrl, '/');
	$aUrl = explode('/', $sUrl);

	foreach($aUrl as $i => $sUrl)
	{
	    if( strpos($sUrl, '_') !== false )
	    {
		$aPart = explode('_', $sUrl);
		if( count($aPart) != 2 || strlen($aPart[0]) == '' )
		{
		    continue;
		}

		$this->set($aPart[0], $aPart[1]);

		$bController = false;
	    }
	    else
	    {
		if( $bController )
		{
		    $this->_sController .= $sUrl . '.';
		}
	    }

            $this->set('slug_' . $i, $sUrl);

            $this->_aSlug[] = $sUrl;
	}

	$this->_sController = rtrim($this->_sController, '.');

	if( !strlen($this->_sController) )
	{
	    $this->_sController = 'index';
	}
    }
}
?>
