<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name System Class
 * @license GoLevel LLC
 */

Final Class SAA_Sys
{
    private static $_sVersion = '2.0.2';

    private static $_aLib = array();

    private static $_aModel = array();

    private static $_oController;

    private static $_sView;

    private static $_sViewError = 'frame/error';

    /**
     * Stores a given page message to the session
     *
     * @param string sMessage message
     *
     */
    public static function addMessage($sMessage)
    {
		SAA_Sys::getLib('session')->add('load_message', $sMessage);
    }

    /**
     * Compiles data using template system
     * 
     * @param mixed $mData
     * @param string $sType
     * @return string
     */
    public static function compile($mData, $sType = 'string')
    {
        $aVars = SAA_Sys::getLib('template')->getTemplateVars();
        $oTpl = new SAA_Template();

        if( count($aVars) )
        {
            $oTpl->assign($aVars);
        }

        return $oTpl->fetch($sType.':'.$mData);
    }

    /**
     * Gets the current controller
     *
     * @return string
     */
    public static function getController()
    {
        return self::$_sView;
    }

    /**
     * Returns a given library class
     *
     * @param string sClass library class name
     *
     */
    public static function getLib($sClass)
    {
		if( isset(self::$_aLib[$sClass]) )
		{
			return self::$_aLib[$sClass];
		}

		$sObj = SAA_Sys::loadClass(SAA_SYSTEM_LIBRARY, $sClass);

		self::$_aLib[$sClass] = new $sObj();

		if( method_exists(self::$_aLib[$sClass], 'getInstance') )
		{
			self::$_aLib[$sClass] = self::$_aLib[$sClass]->getInstance();
		}

		return self::$_aLib[$sClass];
    }

    /**
     * Returns a given model class
     *
     * @param string sClass model class name
     *
     */
    public static function getModel($sClass)
    {
		if( isset(self::$_aModel[$sClass]) )
		{
			return self::$_aModel[$sClass];
		}

		$sObj = SAA_Sys::loadClass(SAA_APP_MODEL, $sClass, 'Model');

		self::$_aModel[$sClass] = new $sObj();

		return self::$_aModel[$sClass];
    }

    /**
     * Returns a given phrase
     *
     * @param string sVar phrase var name
     *
     */
    public static function getPhrase($sVar, $aVars = array())
    {
        return SAA_Sys::getLib('language')->getPhrase($sVar, $aVars);
    }

    /**
     * Returns the setting from setting library
     *
     * @param string sParam setting name
     *
     */
    public static function getSett($sParam)
    {
		return SAA_Sys::getLib('setting')->get($sParam);
    }

    /**
     * Returns the current user's id
     *
     */
    public static function getUserId()
    {
        return SAA_Sys::getModel('user.session')->getUserId();
    }

    /**
     * Returns the SAA version number
     * 
     * @return string
     */
    public static function getVersion()
    {
        return self::$_sVersion;
    }

    /**
     * Returns whether or not the current user is admin
     *
     */
    public static function isAdmin()
    {
        return SAA_Sys::getModel('user.session')->isAdmin();
    }

    /**
     * Returns whether or not the current user is logged in
     *
     */
    public static function isUser()
    {
        return SAA_Sys::getModel('user.session')->isUser();
    }

    /**
     * Executes the script
     *
     *
     */
    public static function load($sController = null, $aStored = array())
    {
		$oTpl = SAA_Sys::getLib('template');

        //Enable Template Security
        $oTpl->enableSecurity('SAA_Template_Security_Policy');

        //get controller name
        if( !$sController )
        {
            $sController = SAA_Sys::getLib('request')->getController();
        }

        //Load controller
		$mAction = SAA_Sys::loadController($sController, $aStored);

        //Get template location
        if( $mAction === false )
        {
            $sView = self::$_sViewError;
        }
        elseif( method_exists(self::$_oController, 'view') )
		{
			$sView = self::$_oController->view();
            $sView = trim(str_replace(array('/', '\\'), DS, $sView), DS);
        }
        else
        {
            $sView = self::$_sView;
        }

        $aTheme = SAA_Sys::getLib('theme')->getDefault();

        //Get template inheritance
        $sTplInherit = $oTpl->getInheritance();

        $sTplInherit = ($sTplInherit ? ($oTpl->templateExists($aTheme['folder_loc'] . DS . $sTplInherit) ? 'extends:'. $aTheme['folder_loc'] . DS . $sTplInherit . '|' : 'extends:default' . DS . $sTplInherit . '|') : '');

        //Get template from theme
        if( $oTpl->templateExists($sTplInherit . $aTheme['folder_loc'] . DS . $sView . SMARTY_EXT) )
        {
            $sView = $aTheme['folder_loc'] . DS . $sView;
        }
        elseif( $oTpl->templateExists($sTplInherit . 'default' . DS . $sView . SMARTY_EXT) )
        {
            $sView = 'default' . DS . $sView;
        }
        else
        {
            SAA_Debug::trigger('Template "'. $sView . SMARTY_EXT .'" does not exist in theme folder "'. $aTheme['folder_loc'].'"');
        }

        //Load any messages saved in the cache
		$aMessages = SAA_Sys::getLib('session')->get('load_message');
		if( $aMessages )
		{
			$oTpl->assign('aMessages', $aMessages);

			SAA_Sys::getLib('session')->delete('load_message');
		}

        $aLanguage = SAA_Sys::getLib('language')->getDefault(false);

		$oTpl->assign(array(
            'aErrors' => SAA_Error::get(), //Load any errors to template
            'sTheme' => $aTheme['folder_loc'],
            'sHeadTitle' => $oTpl->getHeader(),
            'sMetaDesc' => $oTpl->getDescription(),
            'sTitle' => $oTpl->getTitle(),
            'sHTMLDir' => $aLanguage['direction'],
            'sHTMLLang' => $aLanguage['language_abrv']
        ));

        //Return Loaded Template
        if( SAA_IS_AJAX )
        {
            return $oTpl->fetch($sTplInherit . $sView . SMARTY_EXT);
        }

        //Display Loaded Template
        $oTpl->display($sTplInherit . $sView . SMARTY_EXT);
    }

    /**
     * Loads a class using the protocols for library and models
     *
     *	Note: sClass can be parsed by '.' for subfolder locations. For the location
     *	'admin.process' will look for a class name 'Process' in the admin/process.class.php,
     *	then if necessary, look for class name 'Process' in the 'admin/process/process.class.php.
     *
     * @param string sLocation directory
     * @param string sClass string to parse to get class name
     * @param string sClassPrefix prefix for the class name
     *
     */
    public static function loadClass($sLocation, $sClass, $sClassPrefix = '')
    {

        $iMiddle = strrpos($sClass, '.');
		$iEnd = strlen($sClass);

		$sFile = trim(substr($sClass, $iMiddle, $iEnd), '.');
		$sFolderSub = ($sFile != $sClass ? substr($sClass, 0, $iMiddle) : '');

        if( $sFolderSub )
        {
            $sLocation .= str_replace('.', DS, $sFolderSub) . DS;
        }

		if( file_exists($sLocation . $sFile . '.class.php') )
		{
			$sLocation .= $sFile . '.class.php';
		}
		elseif( file_exists($sLocation . $sFile . DS . $sFile . '.class.php') )
		{
			$sLocation .= $sFile . DS . $sFile . '.class.php';
		}
		else
		{
			SAA_Debug::trigger('File does not exist: '. $sLocation . $sFile . DS . $sFile . '.class.php');
		}

		require_once($sLocation);

		$aClass = explode('.', $sClass);

		$sName = '';
		foreach($aClass as $sPart)
		{
			$sName .= ucfirst($sPart) . '_';
		}
		$sName = rtrim($sName, '_');

        $sClass = SAA_CLASS_PREFIX . ($sClassPrefix == '' ? '' : $sClassPrefix . '_') . $sName;

        if( !class_exists($sClass) )
        {
            SAA_Debug::trigger('Class '. $sClass .' does not exist in file '. $sLocation . $sFile . DS . $sFile . '.class.php');
        }

        return $sClass;
    }

    /**
     * Loads the controller from a string containing the controller name
     *
     *	Note: The controller name will be searched from the end of the string to the front. A
     *  controller name referring to a sub-folder will try and load the index of that sub-folder.
     *  If no controllers can be found, then 404 controller will load
     *
     * @param string sController raw controller name
     * @param array aStored array of values to be passed to controller
     *
     */
    public static function loadController($sController, $aStored = array())
    {
        $bLoop = true;
		$sLocation = (!SAA_IS_AJAX ? SAA_APP_CONTROLLER : SAA_APP_CONTROLLER_AJAX);

        if( !$sController )
        {
            $sController = 'index';
        }

		$sController = str_replace('.', DS, $sController) . DS;

		do
		{
            $iEnd = strrpos($sController, DS);

            if( $iEnd === false )
            {
                if( !strlen($sController) || isset($bIsLast) )
                {
                   $sController = '404';
                   break;
                }
                else
                {
                    $bIsLast = true;
                }
            }
            else
            {
                $sController = trim(substr($sController, 0, $iEnd), DS);
            }


            if( file_exists($sLocation . $sController . DS . 'index.class.php') )
            {
                $sController .= DS . 'index';
                $bLoop = false;
            }
            elseif( file_exists($sLocation . $sController . '.class.php') )
            {
                $bLoop = false;
            }
		}
		while($bLoop);

        require_once($sLocation . $sController. '.class.php');

        $aController = explode(DS, $sController);

		$sClass = SAA_CLASS_PREFIX . 'Controller_';
		foreach($aController as $sPart)
		{
			$sClass .= ucfirst($sPart) . '_';
		}
		$sClass = rtrim($sClass, '_');

        self::$_oController = new $sClass();
		self::$_sView = str_replace('.', DS, $sController);

        $mAction = false;

        //Save stored data
        if( $aStored )
        {
            self::$_oController->setStored($aStored);
        }

        //Run pre function within controller
        if( method_exists(self::$_oController, 'preload') )
		{
            $mAction = self::$_oController->preload();
        }

        //Run load function within controller
        if( method_exists(self::$_oController, 'load') )
		{
            $mAction = self::$_oController->load();
        }

        return $mAction;
    }

	/**
	 * Updates analytics data
	 */
	public static function ping()
	{
		if( SAA_Sys::getLib('info')->get('last_ping') < SAA_TIME - 604800 )
		{
			SAA_Sys::getLib('file')->downloadRemote(SAA_API . '?u=' . urlencode(SAA_Sys::getSett('url.path')), SAA_APP_CACHE . 'adder.tmp.txt');

			SAA_Sys::getLib('info')->update('last_ping', SAA_TIME);
		}
	}

    /**
     * Sets the error page
     *
     * @param sting $sTemplate
     */
    public static function setViewError($sTemplate)
    {
        self::$_sViewError = trim(rtrim($sTemplate, SMARTY_EXT));
    }
}
?>
