<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Template Library Class
 * @license GoLevel LLC
 */

require_once SMARTY_DIR . 'Smarty.class.php';

class SAA_Template_Security_Policy extends Smarty_Security
{
  // disable all PHP functions
  public $php_functions = array();
  // remove PHP tags
  public $php_handling = Smarty::PHP_ALLOW;
  // allow everthing as modifier
  public $modifiers = array();
}

class SAA_Template extends Smarty
{
    private $_sDescription = '';
    private $_sExtend = '';
    private $_aHeader = array();
    private $_aTitle = array();

    /**
     * Class Constructor
     */
    public function __construct()
    {
	parent::__construct();

	$this->template_dir = (SAA_IS_AJAX ? array(SAA_APP_VIEW_AJAX, SAA_APP_VIEW) : SAA_APP_VIEW);		// Where your templates are to be loaded from
	$this->compile_dir  = SMARTY_CACHE_COMPILED;	// Where templates are compiled
	$this->cache_dir    = SMARTY_CACHE_CACHED;	// Where templates are cached
	$this->config_dir   = SMARTY_CONFIG;		// Where Smarty configs are located

	$this->assign(array(
	    'aErrors' => array(),
	    'aMessages' => array()
	));
    }

    /**
     * Reset template meta description
     *
     * @return SAA_Template
     */
    public function clearDescription()
    {
        $this->_sDescription = '';

        return $this;
    }

    /**
     * Reset template inheritance
     *
     * @return SAA_Template
     */
    public function clearInheritance()
    {
        $this->_sExtend = '';

        return $this;
    }

    /**
     * Reset template header
     *
     * @return SAA_Template
     */
    public function clearHeader()
    {
        $this->_aHeader = array();

        return $this;
    }

    /**
     * Reset template title
     *
     * @return SAA_Template
     */
    public function clearTitle()
    {
        $this->_aTitle = array();

        return $this;
    }

    /**
     * Gets template meta description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->_sDescription;
    }

    /**
     * Gets template inheritance
     *
     * @return string
     */
    public function getInheritance()
    {
        return $this->_sExtend;
    }

    /**
     * Gets template header
     *
     * @return string
     */
    public function getHeader()
    {
        return implode(' - ', $this->_aHeader);
    }

    /**
     * Gets template title
     *
     * @return string
     */
    public function getTitle()
    {
        return implode(' - ', $this->_aTitle);
    }

    /**
     * Set template meta description
     * 
     * @param string $sText
     * @return SAA_Template 
     */
    public function setDescription($sText)
    {
        $this->_sDescription .= $sText;

        return $this;
    }

    /**
     * Set template inheritance
     *
     * @param string $sTpl
     * @param string $sExt file extension
     * @return SAA_Template
     */
    public function setInheritance($sTpl, $sExt = '')
    {
        $sTpl = rtrim(trim(str_replace(array('/', '\\'), DS, $sTpl), DS), '|');

        $this->_sExtend = $sTpl . ($sExt ? $sExt : SMARTY_EXT);

        return $this;
    }

    /**
     * Sets template header
     *
     * @param mixed $mTitle array or string
     * @param bool $bSetTitle when true, set template title
     * @return SAA_Template
     */
    public function setHeader($mTitle, $bSetTitle = false)
    {
        if( is_string($mTitle) )
        {
            if( !strlen($mTitle) )
            {
                return $this;
            }

            $mTitle = (array) $mTitle;
        }

        foreach($mTitle as $sTitle)
        {
            $this->_aHeader[] = $sTitle;
        }

        if( $bSetTitle )
        {
            $this->setTitle($mTitle);
        }

        return $this;
    }

    /**
     * Sets template title
     *
     * @param mixed $mTitle array or string
     * @return SAA_Template
     */
    public function setTitle($mTitle)
    {
        if( is_string($mTitle) )
        {
            if( !strlen($mTitle) )
            {
                return $this;
            }

            $mTitle = (array) $mTitle;
        }

        foreach($mTitle as $sTitle)
        {
            $this->_aTitle[] = $sTitle;
        }

        return $this;
    }
}
?>
