<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Theme Class
 * @license GoLevel LLC
 */

Class SAA_Theme
{
    private $_aThemes = array();
    private $_iDefault;

     /**
     * Class Constructor
     *
     * Gets the list of themes from cache
     */
    public function  __construct()
    {
        $oCache = SAA_Sys::getLib('cache');

        if( !($this->_aThemes = $oCache->get('themes')) )
        {
            $this->_aThemes = SAA_Sys::getModel('theme')->getAll();

            $oCache->create('themes', $this->_aThemes);
        }

        $this->_setDefault();
    }

    /**
     * Get the default theme
     *
     * @return array
     */
    public function getDefault()
    {
        return $this->_aThemes[$this->_iDefault];
    }

    /**
     * Gets the location of a given theme file
     *
     * @param string $sLocation location of the theme folders
     * @param string $sSubPath the path within the theme folder
     * @param string $sFile the file
     * @return string
     */
    public function getFile($sLocation, $sSubPath, $sFile, $bPath = true)
    {
        $sLocationDS = str_replace('.', DS, $sLocation);
        $sSubPathDS = str_replace('.', DS, $sSubPath);

        if( file_exists(SAA_DIR. $sLocationDS . DS . $this->_aThemes[$this->_iDefault]['folder_loc'] . DS . $sSubPathDS . DS . $sFile) )
        {
            $sPath = ($bPath ? $sLocation . '.' . $this->_aThemes[$this->_iDefault]['folder_loc'] . '.'. $sSubPath : SAA_DIR. $sLocationDS . DS . $this->_aThemes[$this->_iDefault]['folder_loc'] . DS . $sSubPathDS . DS . $sFile);
        }
        elseif( file_exists(SAA_DIR. $sLocationDS . DS . 'default' . DS . $sSubPathDS . DS . $sFile) )
        {
            $sPath = ($bPath ? $sLocation . '.default.'. $sSubPath : SAA_DIR. $sLocationDS . DS . 'default' . DS . $sSubPathDS . DS . $sFile);
        }
        else
        {
            SAA_Debug::trigger('The theme file: ' . $sLocationDS . DS . $this->_aThemes[$this->_iDefault]['folder_loc'] . DS . $sSubPathDS . ' does not exist.');
        }

        return $sPath;
    }

    /**
     * Get the theme by id
     *
     * @param int $iId
     * @return array
     */
    public function get($iId)
    {
        return $this->_aThemes[$iId];
    }

    /**
     * Gets all themes
     * 
     * @return array
     */
    public function getThemes()
    {
        return $this->_aThemes;
    }

    /**
     * Set a theme as default
     *
     * @param int $iId
     * @return bool
     */
    public function setDefault($iId)
    {
        if( !isset($this->_aThemes[$iId]) )
        {
            return false;
        }

        $this->_iDefault = $iId;

        return true;
    }

    /**
     * Determines the default theme
     *
     * @return true
     */
    private function _setDefault()
    {
        //use user's data
        if( SAA_Sys::isUser() )
        {
            $iTheme = SAA_Sys::getModel('user.session')->get('theme_id');

            if( isset($this->_aThemes[$iTheme]) && $this->_aThemes[$iTheme]['is_active'] )
            {
                $this->_iDefault = $iTheme;

                return true;
            }
        }

        //use the defaulted theme
        foreach($this->_aThemes as $iTheme => $aTheme)
        {
            if( $aTheme['is_default'] )
            {
                $this->_iDefault = $iTheme;

                break;
            }
        }
    }
}
?>
