<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Url Library Class
 * @license GoLevel LLC
 */

Class SAA_Url
{
    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Similar to php's urldecode
     *
     * @param string $sValue
     * @return string
     */
    public function decode($sValue)
    {
	return urldecode($sValue);
    }

    /**
     * Decodes data with MIME base64
     *
     * @param string $sValue
     * @return string
     */
    public function decodeBase64($sValue)
    {
	return base64_decode($sValue);
    }

    /**
     * Gets the domain of a given url path
     *
     * @param string $sUrl
     * @return string
     */
    public function domain($sUrl)
    {
        $sUrl = (strpos($sUrl, 'http://') === false ? 'http://' : '') . $sUrl;

        $aPath = parse_url($sUrl);

        if( isset($aPath['host']) )
        {
            return $aPath['scheme'] . '://' . $aPath['host'];
        }

        return $sUrl;
    }

    /**
     * Similar to php's urlencode
     *
     * @param string $sValue
     * @return string
     */
    public function encode($sValue)
    {
	$sValue = urlencode($sValue);
	$sValue = str_replace('_', '%26%2395%3B',$sValue);

	return $sValue;
    }

    /**
     * Encodes data with MIME base64
     *
     * @param string $sValue
     * @return string
     */
    public function encodeBase64($sValue)
    {
	return base64_encode($sValue);
    }

    /**
     * Gets data from db relating to crumb
     *
     * @param string $sSlug
     * @param bool $bDelete
     */
    public function getCrumb($sSlug, $bDelete = true)
    {
        $oDb = SAA_Sys::getLib('database');

        $sUrl = $oDb->select('link_content')
            ->from('crumb')
            ->where('link_slug = "'.$oDb->esc($sSlug).'"')
            ->get('column');

        if( !$sUrl )
        {
            return false;
        }

        if( $bDelete )
        {
            $oDb->delete('crumb', 'link_slug = "'.$oDb->esc($sSlug).'" || time_added < '. (SAA_TIME - 604800) );
        }

        return $sUrl;
    }

    /**
     * Similar to php's http_build_query with an additional option to sort query data
     *
     * @param array $aData
     * @param mixed $mSort
     * @param string $sPrefix
     * @param string $sSeparator
     * @param string $sEncType
     * @return string
     */
    public function http_build_query($aData, $mSort = '', $sPrefix = '', $sSeparator = '&', $sEncType = 'RFC_3986')
    {
        if( $mSort == 'ksort' )
        {
            ksort($aData);
        }
        elseif( $mSort == 'krsort' )
        {
            krsort($aData);
        }
        elseif( $mSort == 'rsort' )
        {
            rsort($aData);
        }
        elseif( $mSort == 'sort' )
        {
            sort($aData);
        }

        $sUrl = '';

        if( $sEncType == 'RFC_1738' )
        {
            foreach($aData as $sVar => $sVal)
            {
                $sUrl .= $sSeparator . urlencode($sPrefix . $sVar) . '=' . urlencode($sVal);
            }
        }
        else
        {
            foreach($aData as $sVar => $sVal)
            {
                $sUrl .= $sSeparator . rawurlencode($sPrefix . $sVar) . '=' . rawurlencode($sVal);
            }
        }

        return ltrim($sUrl, $sSeparator);
    }

    /**
     * Gets a url provided the controller
     *
     * @param string $sController
     * @return string
     */
    public function makeSimple($sController)
    {
        return $this->makeUrl(array('controller' => $sController));
    }

    /**
     * Gets a url from the provided parameters
     *
     * @param array $aParam
     * @return string
     */
    public function makeUrl($aParam)
    {
	$sUrl = '';

        $sUrl .= (isset($aParam['domain']) ? $aParam['domain'] : SAA_Sys::getSett('url.path'));

	if( isset($aParam['folder']) )
	{
	    $sUrl .= str_replace('.', '/', trim(rtrim($aParam['folder']), '/')) . '/';
	}

	if( SAA_Sys::getSett('url.short') )
	{
            if( isset($aParam['file']) )
            {
                $sUrl .= $aParam['file'];
            }
	}
        elseif( !isset($aParam['no_file']) )
        {
            if( isset($aParam['file']) )
            {
                $sUrl .= $aParam['file'];
            }
            else
            {
                $sUrl .= 'index.php';
            }
        }

	if( isset($aParam['controller']) )
	{
	    $sUrl .= (!SAA_Sys::getSett('url.short') ? '?u=' : '') . str_replace('.', '/', $aParam['controller']) . ($aParam['controller'] ? '/' : '');
	}

	if( isset($aParam['vars']) )
	{
	    $sUrl = rtrim($sUrl, '/');

	    $aParam['vars'] = (array) $aParam['vars'];

	    foreach($aParam['vars'] as $mKey => $sValue)
	    {
		$sUrl .= '/' . (!is_numeric($mKey) ? $this->encode($mKey) . '_' : ''). $this->encode($sValue);
	    }

	    if( count($aParam['vars']) )
	    {
		$sUrl .= '/';
	    }
	}

	return $sUrl;
    }


    /**
     * Reloads the current controller
     *
     * @param array $aVars
     * @param string $sMessage
     */
    public function reload($aVars = array(), $sMessage = '')
    {
        $sController = SAA_Sys::getController();

        $this->send($sController, $aVars, $sMessage);
    }

    /**
     * Redirect to a given page
     *
     * @param string $sController
     * @param array $aVars
     * @param string $sMessage
     */
    public function send($sController, $aVars = array(), $sMessage = '')
    {
	$aParam = array(
	    'controller' => $sController,
	);

	if( is_array($aVars) && count($aVars) )
	{
	    $aParam['vars'] = $aVars;
	}

	$sUrl = $this->makeUrl($aParam);

	if( $sMessage )
	{
	    SAA_Sys::addMessage($sMessage);
	}

	header('Location: '. $sUrl);
	exit;
    }

    /**
     * Redirects to a given page with relevant slug variables 
     *
     * @param string $sPage
     * @param string $sMessage
     * @param string $sVar
     */
    public function sendWithCrumb($sPage = 'login', $sMessage = '', $sVar = 'crumb')
    {
        $oParseInput = SAA_Sys::getLib('parse.input');
        $oReq = SAA_Sys::getLib('request');

        $sSlug = $oParseInput->salt(5, true);
        $sSlug = $oParseInput->getUnique($sSlug, 'crumb', 'link_slug', 5);

        SAA_Sys::getLib('database')->insert('crumb', array(
            'link_slug' => $oParseInput->getUnique($sSlug, 'crumb', 'link_slug', 5),
            'link_content' => implode('.', $oReq->getSlugs(false)),
            'time_added' => SAA_TIME
        ));

        $this->send($sPage, array($sVar => $sSlug), $sMessage);
    }
}
?>
