<?php
/**
  * Smarty plugin
  *
  * @package Smarty
  * @subpackage PluginsFunction
  */

/**
  * Smarty {input} function plugin
  *
  * Type:     function<br>
  * Name:     input<br>
  * Purpose:  produce a form item
  * @author   StandAloneArcade <standalonearcade dot com>
  * @param array $params parameters
  * @param object $template template object
  * @return string 
  */

function smarty_function_input($params, $template)
{
    if( !isset($params['type']) )
    {
        return '';
    }

    $sType = $params['type'];
    unset($params['type']);

    if( isset($params['name']) )
    {
        $i = strpos($params['name'], '[');

        $sVar = ($i === false ? $params['name'] : substr($params['name'], 0, $i));
        if( ($mValue = SAA_Sys::getLib('request')->get($sVar, false)) )
        {
            if( $i !== false && is_array($mValue) && preg_match_all("/\[(.*?)\]/i", $params['name'], $aMatches) )
            {
                foreach($aMatches[1] as $sMatch)
                {
                    if( isset($mValue[$sMatch]) )
                    {
                        $mValue = $mValue[$sMatch];
                    }
                }
            }

            if( !is_string($mValue) )
            {
                $mValue = '';
            }
            else
            {
                $mValue = (string) $mValue;
            }
        }
        else
        {
            unset($mValue);
        }
    }

    if( isset($params['default']) )
    {
        $sDefault = (string) $params['default'];
        $sDefaultEsc = str_replace('"', '&quot;', $sDefault);

        unset($params['default']);
    }

    $sText = '';

    switch( $sType )
    {
        case 'checkbox':
            $sText .= '<input type="checkbox" ';

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }

            if( isset($params['value']) )
            {
                $sText .= (isset($mValue) && $mValue == $params['value'] || isset($sDefault) && $sDefault == $params['value'] ? 'checked' : '');
            }

            $sText .= '/>';
            break;
        case 'hidden':
            $sText .= '<input type="hidden" ';

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }

            $sText .= (isset($mValue) ? 'value="' . $mValue . '" ' : (isset($sDefault) ? 'value="' . $sDefault . '" ': '')).'/>';
            break;
        case 'password':
            $sText .= '<input type="password" ';

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }

            $sText .= (isset($mValue) ? 'value="' . $mValue . '" ' : (isset($sDefault) ? 'value="' . $sDefault . '" ': '')).'/>';
            break;
        case 'radio':
            $sText .= '<input type="radio" ';

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }

            if( isset($params['value']) )
            {
                $sText .= (isset($mValue) && $mValue == $params['value'] || isset($sDefault) && $sDefault == $params['value'] ? 'checked' : '');
            }

            $sText .= '/>';
            break;
        case 'select':
            $sText .= '<select ';

            if( isset($params['options']) )
            {
                $aOptions = $params['options'];

                unset($params['options']);

                if( isset($params['key']) )
                {
                    if( $params['key'] == 'values' )
                    {
                        $aKeys = $aOptions;
                    }
                    else
                    {
                        $aKeys = $params['key'];
                    }

                    $aOptions = array_combine($aKeys, $aOptions);

                    unset($params['key']);
                }
            }

            if( isset($params['first']) )
            {
                if( is_string($params['first']) )
                {
                    $sOption = '<option value="">'.$params['first'].'</option>';
                }
                elseif( is_array($params) )
                {
                    $sOption = '<option value="'.$params['first'][0].'">'.$params['first'][1].'</option>';
                }
                else
                {
                    $sOption = '<option value=""></option>';
                }

                unset($params['first']);
            }

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }
            $sText .= '>';

            if( isset($sOption) )
            {
                $sText .= $sOption;
            }

            if( isset($aOptions) )
            {
                foreach($aOptions as $sOptionValue => $sOptionText)
                {
                    $sText .= '<option value="'. $sOptionValue . '"'.(isset($mValue) && $mValue == $sOptionValue || isset($sDefault) && $sDefault == $sOptionValue ? 'selected' : '').'>'. $sOptionText .'</option>';
                }
            }

            $sText .= '</select>';
            break;
        case 'text':
            $sText .= '<input type="text" ';

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }

            $sText .= (isset($mValue) ? 'value="' . $mValue . '" ' : (isset($sDefault) ? 'value="' . $sDefaultEsc . '" ': '')).'/>';
            break;
        case 'textarea':
             $sText .= '<textarea ';

            foreach($params as $sAttr => $sAttrValue)
            {
                $sText .= $sAttr . '="'.$sAttrValue.'" ';
            }

            $sText .= '>'.(isset($mValue) ? $mValue : (isset($sDefault) ? $sDefault : '')).'</textarea>';
            break;
    }

    return $sText;
}
?>